-- モード
local M = W_CUSTOM:GV('Mode')
local mode

local AvatarActor = W_AVATAR:Actor('<default>', 'avatar')
local ActorRadius = LoadActor(W:Object('radius'), {
    Width = 600,
    Height = 60,
    Color = color('0.05, 0.1, 0.15, 0.95'),
    Radius = 8,
})
local ActorBorder = LoadActor(W:Object('radius'), {
    Width = 600,
    Height = 60,
    Color = color('0.4, 0.8, 1.0, 0.9'),
    Radius = 8,
    Fill = false,
})

-- テンプレートカラー一覧
local function ActorColorList()
    local tmpColor = W_CUSTOM:GV('TemplateColorList')
    local actors = Def.ActorFrame({})
    actors[#actors+1] = Def.Quad({
        Name = 'ColorListCursor',
        InitCommand = function(self)
            self:diffuseshift()
            self:zoomto(24, 24)
            self:xy(-48, -48)
            self:playcommand('SetColorListCursor', {Cursor = 0})
        end,
        SetColorListCursorMessageCommand = function(self, params)
            self:xy(-48 + (params.Cursor%5) * 24, -48 + math.floor(params.Cursor/5) * 24)
        end,
    })
    actors[#actors+1] = LoadActor(W:Object('boxline'), 'id', {
        Width = 22, Height = 22, Border = 1, Color = color('#ffffff80'),
    })..{
        Name = 'ColorListDefault',
        InitCommand = function(self)
            self:xy(-48, -48)
        end
    }
    for k,v in pairs(tmpColor or {}) do
        actors[#actors+1] = Def.Quad({
            Name = 'ColorList'..k,
            InitCommand = function(self)
                self:diffuse(v)
                self:xy(-48 + (k%5) * 24, -48 + math.floor(k/5) * 24)
                self:zoomto(18, 18)
            end
        })
    end
    return actors
end

-- RGBカラーバー
local function ActorRGBColor()
    local actors = Def.ActorFrame({})
    for k,v in pairs({'#ff0000ff', '#00ff00ff', '#0000ffff'}) do
        actors[#actors+1] = Def.Quad({
            InitCommand = function(self)
                self:y(-30 + 25*k)
                self:zoomto(4, 15)
            end,
            SetRGBColorCursorMessageCommand = function(self, params)
                self:x(-48 + 96 * params.Color[k])
            end,
        })
        actors[#actors+1] = Def.Quad({
            Name = 'RGBColor'..v,
            InitCommand = function(self)
                self:y(-30 + 25*k)
                self:zoomto(100, 5)
                self:diffuse(0, 0, 0, 1)
                self:diffuserightedge(color(v))
            end,
        })
    end
    actors[#actors+1] = Def.Quad({
        InitCommand = function(self)
            self:y(-40)
            self:zoomto(40, 20)
            self:diffuse(1, 1, 1, 1)
        end,
    })
    actors[#actors+1] = Def.Quad({
        InitCommand = function(self)
            self:y(-40)
            self:zoomto(38, 18)
        end,
        SetRGBColorCursorMessageCommand = function(self, params)
            self:diffuse(params.Color[1], params.Color[2], params.Color[3], params.Color[4])
        end,
    })
    return actors
end

-- 色の共通化
local function ActorBothColor()
    local actors = Def.ActorFrame({})
    actors[#actors+1] = LoadFont('Common Normal')..{
        InitCommand = function(self)
            self:y(-30)
            self:zoom(0.6)
			self:maxwidth(100/0.6)
			self:wrapwidthpixels(100/0.6)
        end,
        SetBothColorTextMessageCommand = function(self, params)
            self:settext(params.Text)
        end,
    }
    -- 共通個別選択
    actors[#actors+1] = Def.ActorFrame({
        InitCommand = function(self)
            self:y(30)
        end,
        ShowBothColorMessageCommand = function(self, params)
            self:visible(params.Type == 'Both')
        end,
        HideBothColorMessageCommand = function(self)
            self:visible(false)
        end,
        Def.Quad({
            InitCommand = function(self)
                self:zoomto(50, 20)
                self:diffuseshift()
            end,
            SetBothColorCursorMessageCommand = function(self, params)
                self:x(-30 + params.Cursor * 60)
            end,
        }),
        LoadFont('Common Normal')..{
            Text = W_CUSTOM:String('Both', 'False'),
            InitCommand = function(self)
                self:x(-30)
                self:zoom(0.75)
                self:maxwidth(50/0.75)
            end,
        },
        LoadFont('Common Normal')..{
            Text = W_CUSTOM:String('Both', 'True'),
            InitCommand = function(self)
                self:x(30)
                self:zoom(0.75)
                self:maxwidth(50/0.75)
            end,
        }
    })
    -- 描画順序選択
    local function ActorOrder()
        local actors = Def.ActorFrame({})
        for k,v in pairs(split(',', W_CUSTOM:String('Order', 'Value'))) do
            actors[#actors+1] = LoadFont('Common Normal')..{
                Text = v,
                InitCommand = function(self)
                    self:x(-40 + (k-1) * 20)
                    self:zoom(0.6)
                end,
            }
        end
        return actors
    end
    actors[#actors+1] = Def.ActorFrame({
        InitCommand = function(self)
            self:y(30)
        end,
        ShowBothColorMessageCommand = function(self, params)
            self:visible(params.Type == 'Order')
        end,
        HideBothColorMessageCommand = function(self)
            self:visible(false)
        end,
        Def.Quad({
            InitCommand = function(self)
                self:zoomto(20, 20)
                self:diffuseshift()
            end,
            SetBothColorCursorMessageCommand = function(self, params)
                self:x(-40 + params.Cursor * 20)
            end,
        }),
        ActorOrder(),
    })
    -- ブレンド方法選択
    actors[#actors+1] = Def.ActorFrame({
        InitCommand = function(self)
            self:y(30)
        end,
        ShowBothColorMessageCommand = function(self, params)
            self:visible(params.Type == 'Blend')
        end,
        HideBothColorMessageCommand = function(self)
            self:visible(false)
        end,
        Def.Quad({
            InitCommand = function(self)
                self:zoomto(50, 20)
                self:diffuseshift()
            end,
            SetBothColorCursorMessageCommand = function(self, params)
                self:x(-30 + params.Cursor * 60)
            end,
        }),
        LoadFont('Common Normal')..{
            Text = W_CUSTOM:String('Blend', 'Normal'),
            InitCommand = function(self)
                self:x(-30)
                self:zoom(0.75)
                self:maxwidth(50/0.75)
            end,
        },
        LoadFont('Common Normal')..{
            Text = W_CUSTOM:String('Blend', 'Add'),
            InitCommand = function(self)
                self:x(30)
                self:zoom(0.75)
                self:maxwidth(50/0.75)
            end,
        }
    })
    return actors
end

local function ActorExtraText()
    local actors = Def.ActorFrame({})
    for i=1, 2 do
        actors[#actors+1] = LoadFont('Common Normal')..{
            InitCommand = function(self)
                self:horizalign(left)
                self:xy(-50, -82 + i*50)
                self:zoom(0.6)
                self:maxwidth(100/0.6)
            end,
            SetExtraTextMessageCommand = function(self, params)
                self:settext(params['Text'..(i*2-1)] or '')
            end,
        }
        actors[#actors+1] = LoadFont('Common Small Normal')..{
            InitCommand = function(self)
                self:horizalign(left)
                self:xy(-45, -62 + i*50)
                self:zoom(0.5)
                self:maxwidth(90/0.5)
            end,
            SetExtraTextMessageCommand = function(self, params)
                self:settext(params['Text'..(i*2)] or '')
            end,
        }
    end
    return actors
end

return Def.ActorFrame({
    InitCommand = function(self)
        self:Center()
    end,
    ChangeModeMessageCommand = function(self, params)
        mode = params.Mode
    end,
    
    -- 一覧表示
	W_CUSTOM:Template('list', {
        Id     = 'Avatar',
        Width  = 220,
        Height = 300,
        Line   = 30,
        Data   = {},
        Buffer = 15,
    })..{
        InitCommand = function(self)
            self:xy(-200, 30)
        end,
    },
    LoadFont('Common Normal')..{
        Text = '&MenuLeft;',
        InitCommand = function(self)
            self:xy(-200 - 90, 30)
            self:visible(false)
        end,
        ChangeModeMessageCommand = function(self, params)
            self:visible(params.Mode == M.Parts or params.Mode == M.Color)
        end,
    },
    LoadFont('Common Normal')..{
        Text = '&MenuRight;',
        InitCommand = function(self)
            self:xy(-200 + 90, 30)
            self:visible(false)
        end,
        ChangeModeMessageCommand = function(self, params)
            self:visible(params.Mode == M.Parts or params.Mode == M.Color)
        end,
    },

    LoadActor('bg'),
    LoadActor('description')..{
        InitCommand = function(self)
            self:y(-165)
        end,
    },

	LoadFont('Common Normal')..{
        Name = 'Title',
		Text = '',
		InitCommand = function(self)
			self:x(110)
			self:maxwidth(360)
			self:diffuse(Color('White'))
			self:strokecolor(Color('Outline'))
		end,
		ChangeTitleMessageCommand = function(self, params)
			self:stoptweening()
            self:y(string.find(params.Text or '', '\n') and -103 or -95)
            local text = split('\n', params.Text)[1] or ''
            if mode == M.Avatar then
    			self:settext(((params.Type == 'Template') and '* ' or '')..text)
            else
                self:settext(text or '')
            end
			self:diffusealpha(0)
			self:linear(0.1)
			self:diffusealpha(1)
		end,
	},
	LoadFont('Common Normal')..{
        Name = 'SubTitle',
		Text = '',
		InitCommand = function(self)
			self:xy(110, -85)
            self:zoom(0.5)
			self:maxwidth(360/0.5)
			self:diffuse(Color('White'))
			self:strokecolor(Color('Outline'))
		end,
		ChangeTitleMessageCommand = function(self, params)
			self:stoptweening()
            local text = split('\n', params.Text)[2] or ''
            self:settext(text or '')
			self:diffusealpha(0)
			self:linear(0.1)
			self:diffusealpha(1)
		end,
	},
	LoadFont('Common Normal')..{
        Name = 'Body',
		Text = '',
		InitCommand = function(self)
            self:horizalign(left)
			self:xy(-60, 130)
            self:zoom(0.6)
			self:maxwidth(360/0.6)
			self:wrapwidthpixels(360/0.6)
            self:vertspacing(10)
			self:diffuse(Color('White'))
			self:strokecolor(Color('Outline'))
		end,
		ChangeModeMessageCommand = function(self, params)
            if params.Mode == M.Avatar then
                self:settext(W_CUSTOM:String('Body', 'Avatar'))
            elseif params.Mode == M.Proc then
                self:settext(W_CUSTOM:String('Body', 'Proc'))
            elseif params.Mode == M.Profile then
                self:settext(W_CUSTOM:String('Body', 'Profile'))
            elseif params.Mode == M.Parts then
                self:settext(W_CUSTOM:String('Body', 'Parts'))
            elseif params.Mode == M.Color then
                self:settext(W_CUSTOM:String('Body', 'Color'))
            end
		end,
	},
    Def.Quad({
        InitCommand = function(self)
            self:zoomto(390, 150)
            self:xy(115, 15)
            self:diffuseshift()
            self:effectperiod(20)
            self:effectcolor1(1, 1, 1, 0.05)
            self:effectcolor2(1, 1, 1, 0.33)
            self:fadetop(0.25)
            self:fadebottom(0.25)
        end,
    }),

    -- 色、重ね順序詳細
    Def.ActorFrame({
        InitCommand = function(self)
            self:xy(-10, 15)
        end,
        Def.Quad({
            InitCommand = function(self)
                self:diffuse(0, 0, 0, 0.75)
                self:zoomto(120, 120)
            end,
        }),
        ActorColorList()..{
            InitCommand = function(self)
                self:visible(false)
            end,
            ShowColorListMessageCommand = function(self)
                self:visible(true)
            end,
            HideColorListMessageCommand = function(self)
                self:visible(false)
            end,
        },
        ActorRGBColor()..{
            InitCommand = function(self)
                self:visible(false)
            end,
            ShowRGBColorMessageCommand = function(self)
                self:visible(true)
            end,
            HideRGBColorMessageCommand = function(self)
                self:visible(false)
            end,
        },
        ActorBothColor()..{
            InitCommand = function(self)
                self:visible(false)
            end,
            ShowBothColorMessageCommand = function(self)
                self:visible(true)
            end,
            HideBothColorMessageCommand = function(self)
                self:visible(false)
            end,
        },
        ActorExtraText(),
    }),

    -- アバター画像
    Def.ActorFrame({
        InitCommand = function(self)
            self:xy(130, 20)
        end,
        AvatarActor..{
            ChangeEmotionMessageCommand = function(self, params)
                self:playcommand(params.Emotion..'Emotion')
            end,
            GetSaveDataMessageCommand = function(self)
                self:playcommand('CallGetAvatarData', {Id = 'avatar'})
            end,
        },
    }),
    Def.ActorFrame({
        InitCommand = function(self)
            self:diffusealpha(0)
            self:y(-SCREEN_CENTER_Y + SCREEN_HEIGHT - 70 + 200)
        end,
        ShowToastMessageCommand = function(self)
            self:finishtweening()
            self:y(-SCREEN_CENTER_Y + SCREEN_HEIGHT - 70 + 200)
            self:bounceend(0.3)
            self:diffusealpha(1)
            self:y(-SCREEN_CENTER_Y + SCREEN_HEIGHT - 70)
            self:sleep(5.0)
            self:bouncebegin(0.3)
            self:diffusealpha(0)
            self:y(-SCREEN_CENTER_Y + SCREEN_HEIGHT - 70 + 200)
        end,
        ActorRadius:Actor(),
        ActorBorder:Actor(),
        LoadFont('Common Normal')..{
            Text = '',
            InitCommand = function(self)
                self:diffuse(Color('White'))
                self:zoom(0.8)
            end,
            ShowToastMessageCommand = function(self, params)
                self:settext(params.Text)
            end,
        },
    }),


    W_CUSTOM:Template('Header'),
	W_CUSTOM:Template('Helper', {
		Text = THEME:GetString('ScreenCustom', 'HelperText'),
	}),
})