-- モード
local M = {
    Profile = 0,  -- 0:PROFILE選択
    Slot    = 1,  -- 1:対象選択
}
W_CUSTOM:SV('Mode', M)

local screen
local lock = 0.6
local actorSelf
local listData = W_PLAYER:List()
local current = 1
local mode = M.Profile

local targetList = {
    {Id = '', Name = W_CUSTOM:String('Text', 'NotSet')}
}
for k,v in pairs(W_TARGET:List(true)) do
    targetList[#targetList+1] = {
        Id   = v,
        Name = W_TARGET:Name(v),
    }
end
local slotList = {
    {Key = 'Main', Target = 1},
    {Key = 'Sub1', Target = 1},
    {Key = 'Sub2', Target = 1},
}
local slot = 1

-- キー・マウス操作
local function CallbackInput(params)
    local function CheckButton(params, key)
        return (params.button and params.button == key)
            or (params.GameButton and params.GameButton == key)
    end

    if mode == M.Profile then
        if params.type == 'InputEventType_Release' then
            if CheckButton(params, 'Back') then
                actorSelf:playcommand('EndCustom')
            end
            if CheckButton(params, 'Start') then
                if #listData > 0 then
                    actorSelf:playcommand('ModeSlot')
                end
            end
        end
        if params.type == 'InputEventType_FirstPress' or params.type == 'InputEventType_Repeat' then
            if CheckButton(params, 'Up') or CheckButton(params, 'Left') then
                if current > 1 then
                    current = current-1
                else
                    current = #listData
                end
                actorSelf:playcommand('CursorProfile')
            end
            if CheckButton(params, 'Down') or CheckButton(params, 'Right') then
                if current < #listData then
                    current = current+1
                else
                    current = 1
                end
                actorSelf:playcommand('CursorProfile')
            end
        end
    elseif mode == M.Slot then
        if params.type == 'InputEventType_Release' then
            if CheckButton(params, 'Back') then
                actorSelf:playcommand('PlayCancelSound')
                actorSelf:playcommand('ModeProfile')
            end
            if CheckButton(params, 'Start') then
                -- 保存の場合は保存処理、それ以外の場合は一つ下へ
                if slot < #slotList+1 then
                    slot = slot+1
                    actorSelf:playcommand('CursorSlot')
                else
                    actorSelf:playcommand('SaveTarget')
                end
            end
        end
        if params.type == 'InputEventType_FirstPress' or params.type == 'InputEventType_Repeat' then
            if CheckButton(params, 'Up') then
                if slot > 1 then
                    slot = slot-1
                else
                    slot = #slotList+1
                end
                actorSelf:playcommand('CursorSlot')
            end
            if CheckButton(params, 'Down') then
                if slot < #slotList+1 then
                    slot = slot+1
                else
                    slot = 1
                end
                actorSelf:playcommand('CursorSlot')
            end
            if CheckButton(params, 'Left') then
                if slot > #slotList then return end
                if slotList[slot].Target > 1 then
                    slotList[slot].Target = slotList[slot].Target-1
                else
                    slotList[slot].Target = #targetList
                end
                actorSelf:playcommand('CursorTarget')
            end
            if CheckButton(params, 'Right') then
                if slot > #slotList then return end
                if slotList[slot].Target < #targetList then
                    slotList[slot].Target = slotList[slot].Target+1
                else
                    slotList[slot].Target = 1
                end
                actorSelf:playcommand('CursorTarget')
            end
        end
    end
end

return Def.ActorFrame({
    InitCommand = function(self)
        actorSelf = self
    end,
    OnCommand = function(self)
        MESSAGEMAN:Broadcast('ScrollMenu', {Cursor = current, Init = true})
        screen = SCREENMAN:GetTopScreen()
        screen:lockinput(lock)
        screen:AddInputCallback(CallbackInput)
        MESSAGEMAN:Broadcast('UpdateHelper', {Text = W_CUSTOM:String('Helper', 'Profile')})
    end,
    -- PROFILE選択に遷移
    ModeProfileCommand = function(self)
        mode = M.Profile
        MESSAGEMAN:Broadcast('UpdateList', {Active = true})
        MESSAGEMAN:Broadcast('ChangedMode', {Mode = mode})
        MESSAGEMAN:Broadcast('UpdateHelper', {Text = W_CUSTOM:String('Helper', 'Profile')})
    end,
    -- PROFILEを選択
    CursorProfileCommand = function(self)
        MESSAGEMAN:Broadcast('ScrollMenu', {Cursor = current})
        MESSAGEMAN:Broadcast('ChangedProfile', {Name = listData[current].Name})
    end,
    -- スロット選択に遷移
    ModeSlotCommand = function(self)
        mode = M.Slot
        slot = 1
        self:playcommand('PlayStartSound')
        MESSAGEMAN:Broadcast('ChangedSlot', {Slot = slotList[slot].Key})
        -- 設定の一覧を取得
        local profileTarget = split('|', W_PLAYER:ProfilePref(listData[current].Id, 'TargetList', 1, ''))
        -- Main/Sub1/Sub2をすべて確認
        for i=1, #slotList do
            slotList[i].Target = 1
            if i <= #profileTarget and profileTarget[i] ~= '' then
                -- 設定されている名前がターゲット一覧にいるかどうかチェック
                for index,target in pairs(targetList) do
                    if string.lower(target.Id) == string.lower(profileTarget[i]) then
                        slotList[i].Target = index
                        break
                    end
                end
            end
            MESSAGEMAN:Broadcast('UpdateTarget', {Slot = slotList[i].Key, Name = targetList[slotList[i].Target].Name})
        end
        MESSAGEMAN:Broadcast('UpdateList', {Active = false})
        MESSAGEMAN:Broadcast('ChangedMode', {Mode = mode})
        MESSAGEMAN:Broadcast('UpdateHelper', {Text = W_CUSTOM:String('Helper', 'Slot')})
    end,
    -- スロットを選択
    CursorSlotCommand = function(self)
        self:playcommand('PlayCursorSound')
        MESSAGEMAN:Broadcast('ChangedSlot', {Slot = (slot <= #slotList) and slotList[slot].Key or nil})
    end,
    -- ターゲットを選択
    CursorTargetCommand = function(self, params)
        self:playcommand('PlayCursorSound')
        MESSAGEMAN:Broadcast('UpdateTarget', {Slot = slotList[slot].Key, Name = targetList[slotList[slot].Target].Name})
    end,
    -- ターゲット設定を保存
    SaveTargetCommand = function(self)
        self:playcommand('PlayStartSound')
        local profileTarget = {}
        -- 設定なしの場合は空文字
        for i=1, #slotList do
            profileTarget[#profileTarget+1] = (slotList[i].Target > 1) and targetList[slotList[i].Target].Id or ''
        end
        W_PLAYER:ProfilePref(listData[current].Id, 'TargetList', 2, join('|', profileTarget))
        self:playcommand('ModeProfile')
    end,
    W_CUSTOM:Template('Basic')
})