local id, actorParams = ...
-- 影の大きさ
local length = actorParams['Length'] or 8
-- 影の色
local color  = actorParams['Color'] or color('0, 0, 0, 0.75')
-- 中央の色
local center = actorParams['Center'] or color
local alpha = color[4]

local actors = Def.ActorFrame({
    -- 左上辺
    LoadActor('boxshadow')..{
        InitCommand = function(self)
            self:animate(false)
            self:setstate(0)
            self:horizalign(right)
            self:vertalign(bottom)
            self:zoomto(length, length)
            self:diffuse(color)
            self:diffusealpha(alpha)
        end,
        UpdateBoxShadowMessageCommand = function(self, params)
            if params.Id ~= id then return end
            self:stoptweening()
            if not params.AnimeSize then
                self:xy(-params.Width/2, -params.Height/2)
            end
            if params.AnimeAlpha then
                self:diffusealpha(0)
            end
            self:linear(params.Wait or 0)
            length = params.Length or length
            self:xy(-params.Width/2, -params.Height/2)
            self:zoomto(length, length)
            self:diffuse(params.Color or color)
            self:diffusealpha(alpha)
        end,
    },
    -- 上辺
    LoadActor('boxshadow')..{
        InitCommand = function(self)
            self:animate(false)
            self:setstate(1)
            self:vertalign(bottom)
            self:diffuse(color)
            self:diffusealpha(alpha)
        end,
        UpdateBoxShadowMessageCommand = function(self, params)
            if params.Id ~= id then return end
            self:stoptweening()
            if not params.AnimeSize then
                self:xy(0, -params.Height/2)
                self:zoomto(params.Width, length)
            end
            if params.AnimeAlpha then
                self:diffusealpha(0)
            end
            self:linear(params.Wait or 0)
            length = params.Length or length
            self:xy(0, -params.Height/2)
            self:zoomto(params.Width, length)
            self:diffuse(params.Color or color)
            self:diffusealpha(alpha)
        end,
    },
    -- 右上辺
    LoadActor('boxshadow')..{
        InitCommand = function(self)
            self:animate(false)
            self:setstate(2)
            self:horizalign(left)
            self:vertalign(bottom)
            self:zoomto(length, length)
            self:diffuse(color)
            self:diffusealpha(alpha)
        end,
        UpdateBoxShadowMessageCommand = function(self, params)
            if params.Id ~= id then return end
            self:stoptweening()
            if not params.AnimeSize then
                self:xy(params.Width/2, -params.Height/2)
            end
            if params.AnimeAlpha then
                self:diffusealpha(0)
            end
            self:linear(params.Wait or 0)
            length = params.Length or length
            self:xy(params.Width/2, -params.Height/2)
            self:zoomto(length, length)
            self:diffuse(params.Color or color)
            self:diffusealpha(alpha)
        end,
    },
    -- 左辺
    LoadActor('boxshadow')..{
        InitCommand = function(self)
            self:animate(false)
            self:setstate(3)
            self:horizalign(right)
            self:diffuse(color)
            self:diffusealpha(alpha)
        end,
        UpdateBoxShadowMessageCommand = function(self, params)
            if params.Id ~= id then return end
            self:stoptweening()
            if not params.AnimeSize then
                self:xy(-params.Width/2, 0)
                self:zoomto(length, params.Height)
            end
            if params.AnimeAlpha then
                self:diffusealpha(0)
            end
            self:linear(params.Wait or 0)
            length = params.Length or length
            self:xy(-params.Width/2, 0)
            self:zoomto(length, params.Height)
            self:diffuse(params.Color or color)
            self:diffusealpha(alpha)
        end,
    },
    -- 中央
    Def.Quad({
        InitCommand = function(self)
            self:diffuse(center)
        end,
        UpdateBoxShadowMessageCommand = function(self, params)
            if params.Id ~= id or not center then return end
            self:stoptweening()
            if not params.AnimeSize then
                self:zoomto(params.Width, params.Height)
            end
            self:linear(params.Wait or 0)
            self:zoomto(params.Width, params.Height)
            center = params.Center or center
            self:diffuse(center)
        end,
    }),
    -- 右辺
    LoadActor('boxshadow')..{
        InitCommand = function(self)
            self:animate(false)
            self:setstate(5)
            self:horizalign(left)
            self:diffuse(color)
            self:diffusealpha(alpha)
        end,
        UpdateBoxShadowMessageCommand = function(self, params)
            if params.Id ~= id then return end
            self:stoptweening()
            if not params.AnimeSize then
                self:xy(params.Width/2, 0)
                self:zoomto(length, params.Height)
            end
            if params.AnimeAlpha then
                self:diffusealpha(0)
            end
            self:linear(params.Wait or 0)
            length = params.Length or length
            self:xy(params.Width/2, 0)
            self:zoomto(length, params.Height)
            self:diffuse(params.Color or color)
            self:diffusealpha(alpha)
        end,
    },
    -- 左下辺
    LoadActor('boxshadow')..{
        InitCommand = function(self)
            self:animate(false)
            self:setstate(6)
            self:horizalign(right)
            self:vertalign(top)
            self:zoomto(length, length)
            self:diffuse(color)
            self:diffusealpha(alpha)
        end,
        UpdateBoxShadowMessageCommand = function(self, params)
            if params.Id ~= id then return end
            self:stoptweening()
            if not params.AnimeSize then
                self:xy(-params.Width/2, params.Height/2)
            end
            if params.AnimeAlpha then
                self:diffusealpha(0)
            end
            self:linear(params.Wait or 0)
            length = params.Length or length
            self:xy(-params.Width/2, params.Height/2)
            self:zoomto(length, length)
            self:diffuse(params.Color or color)
            self:diffusealpha(alpha)
        end,
    },
    -- 下辺
    LoadActor('boxshadow')..{
        InitCommand = function(self)
            self:animate(false)
            self:setstate(7)
            self:vertalign(top)
            self:diffuse(color)
            self:diffusealpha(alpha)
        end,
        UpdateBoxShadowMessageCommand = function(self, params)
            if params.Id ~= id then return end
            self:stoptweening()
            if not params.AnimeSize then
                self:xy(0, params.Height/2)
                self:zoomto(params.Width, length)
            end
            if params.AnimeAlpha then
                self:diffusealpha(0)
            end
            self:linear(params.Wait or 0)
            length = params.Length or length
            self:xy(0, params.Height/2)
            self:zoomto(params.Width, length)
            self:diffuse(params.Color or color)
            self:diffusealpha(alpha)
        end,
    },
    -- 右下辺
    LoadActor('boxshadow')..{
        InitCommand = function(self)
            self:animate(false)
            self:setstate(8)
            self:horizalign(left)
            self:vertalign(top)
            self:zoomto(length, length)
            self:diffuse(color)
            self:diffusealpha(alpha)
        end,
        UpdateBoxShadowMessageCommand = function(self, params)
            if params.Id ~= id then return end
            self:stoptweening()
            if not params.AnimeSize then
                self:xy(params.Width/2, params.Height/2)
            end
            if params.AnimeAlpha then
                self:diffusealpha(0)
            end
            self:linear(params.Wait or 0)
            length = params.Length or length
            self:xy(params.Width/2, params.Height/2)
            self:zoomto(length, length)
            self:diffuse(params.Color or color)
            self:diffusealpha(alpha)
        end,
    },
})

return actors