local params = ...
local w = params.Width or 60
local h = params.Height or 60

local data = {
    Glass = {File = 'glass', Radius = 20}
}
local file = (params.File and data[params.File]) and params.File or 'Glass'

local r = params.Radius or data[file].Radius
local c = params.Color or color('#ffffffff')
local o = params.Overlay or color('#ffffffff')

local function GlassActor(index)
    local GraphicActor = function(fileName, isBase)
        return LoadActor(fileName..(isBase and '_base' or '_over'))..{
            InitCommand = function(self)
                self:animate(false)
                self:setstate(index)
                self:playcommand('Update', {
                    Width  = w,
                    Height = h,
                    Linear = 0,
                })
            end,
            UpdateCommand = function(self, params)
                w = params.Width or w
                h = params.Height or h
                r = params.Radius or r
                c = params.Color or c
                o = params.Overlay or o
                if params.Linear and params.Linear > 0 then
                    self:linear(params.Linear)
                end
                self:diffuse(isBase and c or o or color('#ffffffff'))
                -- 横
                local indexW = index%3
                local sideW = (w >= r*2) and r or w/2
                self:horizalign((indexW ~= 1) and ((indexW == 0) and right or left) or center)
                if indexW ~= 1 then
                    -- 端
                    self:zoomtowidth(sideW)
                    self:x((sideW < r) and 0 or ((w-sideW*2)/2 * ((indexW == 0) and -1 or 1)))
                else
                    -- 中央
                    self:zoomtowidth((sideW < r) and 0 or (w-sideW*2))
                    self:x(0)
                end
                -- 縦
                local indexH = math.floor(index/3)
                local sideH = (h >= r*2) and r or h/2
                self:vertalign((indexH ~= 1) and ((indexH == 0) and bottom or top) or middle)
                if indexH ~= 1 then
                    -- 端
                    self:zoomtoheight(sideH)
                    self:y((sideH < r) and 0 or ((h-sideH*2)/2 * ((indexH == 0) and -1 or 1)))
                else
                    -- 中央
                    self:zoomtoheight((sideH < r) and 0 or (h-sideH*2))
                    self:y(0)
                end
            end,
        }
    end
    return Def.ActorFrame({
        GraphicActor(data[file].File, true),
        GraphicActor(data[file].File, false),
    })
end

local actor = Def.ActorFrame({})
for i=1, 9 do
    actor[#actor+1] = GlassActor(i-1)
end
return actor