local mode = ...
mode = mode or 'Gameplay'
local w, h = 100, 14
local posX = {
    SelectMusic = {
        SCREEN_LEFT - 100,
        SCREEN_CENTER_X - tonumber(THEME:GetMetric('ScreenSelectMusic', 'GrooveRadarXPositionOffset')),
        SCREEN_LEFT + 40,
    },
    Options = {
        SCREEN_LEFT + 40,
        SCREEN_LEFT + 40,
        SCREEN_LEFT + 40,
    },
    Gameplay = {
        SCREEN_LEFT + 40,
        SCREEN_LEFT + 40,
        SCREEN_LEFT - 100,
    },
    StageInformation = {
        SCREEN_LEFT + 40,
        SCREEN_LEFT + 40,
        SCREEN_LEFT + 40,
    },
}
local gameY = 56
local posY = {
    SelectMusic = {
        SCREEN_CENTER_Y + 55 + (GAMESTATE:IsCourseMode() and 12 or 0),
        SCREEN_CENTER_Y + 55 + (GAMESTATE:IsCourseMode() and 12 or 0),
        gameY,
    },
    Options = {
        gameY,
        70,
        gameY,
    },
    Gameplay = {
        gameY,
        gameY,
        gameY,
    },
    StageInformation = {
        gameY,
        gameY,
        gameY,
    },
}
local bpmAverage = 0
local function PlayerActor(pn)
    -- if not W_PLAYER:IsEnabled(pn) then
    --     return Def.ActorFrame({})
    -- end
    local po = GAMESTATE:GetPlayerState(pn):GetPlayerOptions("ModsLevel_Preferred")
    local function HsToColor(hs)
        if string.match(hs, 'C[0-9]+') then
            return color('0.5, 0, 0, 0.9')
        elseif string.match(hs, 'm[0-9]+') then
            return color('0.1, 0.5, 0, 0.9')
        elseif string.match(hs, 'ca[0-9]+') then
            return color('0.4, 0.4, 0, 0.9')
        elseif string.match(hs, 'a[0-9]+') then
            return color('0.5, 0, 0.3, 0.9')
        end
        return color('0, 0.4, 0.5, 0.9')
    end
    local function PosPlayerX(index)
        local x = posX[mode][index]
        return (pn == PLAYER_1) and x or SCREEN_WIDTH - x
    end
    -- 画面に表示するBPM（オプション用）
    local function RealBpm(mod, speed, ...)
        local steps = ...
        if mode ~= 'Options' or GAMESTATE:IsCourseMode() then
            return '??? - ???'
        end
        steps = steps or GAMESTATE:GetCurrentSteps(pn)
        return string.format('%s\navg. %s',
            W_PLAYER:HighspeedBPM(pn, mod, speed, steps),
            bpmAverage < 0 and '???' or
            mod == 'x' and math.round(bpmAverage * speed)
            or (mod == 'm' and math.round(bpmAverage)
                or (mod == 'c' and speed or '???'))
        )
    end
    local Actor = LoadActor(W:Object('radius'), {
        Id = 'HS' .. W:PNum(pn),
        Width = w,
        Height = h,
        Color = color('0, 0, 0, 0'),
        Radius = 8,
    })
    local Border = LoadActor(W:Object('radius'), {
        Id = 'HS' .. W:PNum(pn),
        Width = w,
        Height = h,
        Color = color('0, 0, 0, 0.5'),
        Radius = 8,
        Fill = false,
        Mini = false,
    })
    return Def.ActorFrame({
        InitCommand = function(self)
            self:xy(PosPlayerX(1), posY[mode][1])
            -- あとから参加した時に表示できるように透明度を0にするだけにしておく
            self:diffusealpha(W_PLAYER:IsEnabled(pn) and 1 or 0)
        end,
        ShowCommand = function(self, params)
            if not W_PLAYER:IsEnabled(pn) or (params and params.Player ~= pn) then return end
            self:diffusealpha(1)
            local sleep = params and params.Sleep or 0.3
            local wait = params and params.Wait or 0.2
            self:xy(PosPlayerX(1), posY[mode][1])
            if sleep > 0 then
                self:sleep(sleep)
            end
            if wait > 0 then
                self:linear(wait)
            end
            self:xy(PosPlayerX(2), posY[mode][2])
        end,
        HideCommand = function(self, params)
            if not W_PLAYER:IsEnabled(pn) then return end
            local sleep = params and params.Sleep or 0
            local wait = params and params.Wait or 0.2
            self:xy(PosPlayerX(2), posY[mode][2])
            if sleep > 0 then
                self:sleep(sleep)
            end
            if wait > 0 then
                self:linear(wait)
            end
            self:xy(PosPlayerX(3), posY[mode][3])
        end,
        Def.Actor({
            ReloadHighSpeedCommand = function(self, params)
                if not W_PLAYER:IsEnabled(pn) or params.Player ~= pn then return end
                YA_SCROLL:Load(pn)
                local speed = YA_SCROLL:CurrentSpeed(pn)
                self:GetParent():playcommand('UpdateHighSpeed', { Player = pn, Mod = speed.Mod, Speed = speed.Speed })
            end,
            SpeedChoiceChangedMessageCommand = function(self, params)
                if params.pn ~= pn then return end
                YA_SCROLL:SetSpeed(params.pn, params.mode, (params.speed * ((params.mode == 'x') and 0.01 or 1)))
                self:GetParent():playcommand('UpdateHighSpeed',
                    { Player = pn, Mod = params.mode, Speed = (params.speed * ((params.mode == 'x') and 0.01 or 1)) })
            end,
        }),
        Actor:Actor() .. {
            UpdateHighSpeedCommand = function(self, params)
                if params.Player ~= pn then return end
                self:playcommand('UpdateRadius', {
                    Color = HsToColor(YA_SCROLL:ToDisplaySpeed({ Mod = params.Mod, Speed = params.Speed })),
                })
            end,
            ShowCommand = function(self, params)
                if mode ~= 'Options' then return end
                self:playcommand('UpdateRadius', {
                    Height = h * 2.5,
                    Wait   = params.Wait,
                })
                self:y(5)
            end,
            HideCommand = function(self)
                if mode ~= 'Options' then return end
                self:playcommand('UpdateRadius', {
                    Height = h,
                    Wait   = 0.2,
                })
                self:y(0)
            end,
        },
        Border:Actor() .. {
            ShowCommand = function(self, params)
                if mode ~= 'Options' then return end
                self:playcommand('UpdateRadius', {
                    Height = h * 2.5,
                    Wait   = params.Wait,
                })
                self:y(5)
            end,
            HideCommand = function(self)
                if mode ~= 'Options' then return end
                self:playcommand('UpdateRadius', {
                    Height = h,
                    Wait   = 0.2,
                })
                self:y(0)
            end,
        },
        -- ハイスピード設定
        LoadFont('Common Small Normal') .. {
            InitCommand = function(self)
                self:zoom(0.4)
                if mode ~= 'Options' then return end
            end,
            ShowCommand = function(self, params)
                if mode ~= 'Options' then return end
                if params.Wait > 0 then
                    self:y(0)
                    self:linear(params.Wait)
                end
                self:y(-5)
            end,
            HideCommand = function(self)
                if mode ~= 'Options' then return end
                self:linear(0.2)
                self:y(0)
            end,
            -- WAIEI Coreのメッセージ
            ChangeSpeedMessageCommand = function(self, params)
                self:settextf('SPEED %s', YA_SCROLL:ToDisplaySpeed({ Mod = params.Mod, Speed = params.Speed }))
                self:AddAttribute(0, { Length = 6, Diffuse = color('1.0, 1.0, 1.0, 0.5') })
            end,
            UpdateHighSpeedCommand = function(self, params)
                if params.Player ~= pn then return end
                self:playcommand('ChangeSpeed', params)
            end,
        },
        -- 実際のBPM
        LoadFont('Common Small Normal') .. {
            InitCommand = function(self)
                if mode ~= 'Options' then
                    self:visible(false)
                    return
                end
                self:zoom(0.4)
            end,
            ShowCommand = function(self, params)
                if mode ~= 'Options' then return end
                if params.Wait > 0 then
                    self:diffusealpha(0)
                    self:y(0)
                    self:linear(params.Wait)
                end
                self:y(10)
                self:diffusealpha(1)
            end,
            HideCommand = function(self)
                if mode ~= 'Options' then return end
                self:linear(0.2)
                self:y(0)
                self:diffusealpha(0)
            end,
            -- WAIEI Coreのメッセージ
            ChangeSpeedMessageCommand = function(self, params)
                self:settext(RealBpm(params.Mod, params.Speed))
            end,
            UpdateHighSpeedCommand = function(self, params)
                if params.Player ~= pn then return end
                bpmAverage = W_PLAYER:AverageBPM(pn, params.Mod, params.Speed)
                self:settext(RealBpm(string.lower(params.Mod), params.Speed))
            end,
            StepsChangedMessageCommand = function(self, params)
                if params.Player ~= pn then return end
                local speed = YA_SCROLL:CurrentSpeed(pn)
                bpmAverage = W_PLAYER:AverageBPM(pn, speed.Mod, speed.Speed, params.Steps)
                self:settext(RealBpm(string.lower(speed.Mod), speed.Speed, params.Steps))
            end,
        },
    })
end
return Def.ActorFrame({
    Def.ActorFrame({
        PlayerActor(PLAYER_1),
        PlayerActor(PLAYER_2),
        OnCommand = function(self)
            self:playcommand('ReloadHighSpeed', { Player = PLAYER_1 })
            self:playcommand('ReloadHighSpeed', { Player = PLAYER_2 })
        end,
        ShowSelectMusicNewPlayerCommand = function(self, params)
            self:playcommand('ReloadHighSpeed', { Player = params.Player })
            self:playcommand('Show', { Player = params.Player })
        end,
    }),
})
