-- 3D
local F = dofile(W:Object('model func.lua'))

--local models = F:Model(THEME:GetPathG('', '_tmp/rin/model.txt'))
--local bones  = F:Bone(THEME:GetPathG('', '_tmp/bones.txt'))
local models = F:Model(THEME:GetPathG('', '_tmp/model.txt'))
local bones  = F:Bone(THEME:GetPathG('', '_tmp/model.txt'))
local boneMax = 0	  -- 最大アニメーションカウント（多分全部同じ数のはず）
local zoom = 1.0 -- 表示倍率

boneMax = bones[0]['Count']

local actors = Def.ActorFrame({})

local function SetupVertecies(name, number, anime)
	local model   = models[name]['Pos'][number]
	local bonePos = bones[model['BoneId']]['Pos'][anime]
	local boneRot = bones[model['BoneId']]['Rot'][anime]
	local res = {X = model.X, Y = model.Y, Z = model.Z}
	local tmp = {}
	-- X軸回転
	tmp.X = res.X
	tmp.Y = res.Y * math.cos(boneRot.X) + res.Z * -math.sin(boneRot.X)
	tmp.Z = res.Y * math.sin(boneRot.X) + res.Z * math.cos(boneRot.X)
	res = {X = tmp.X, Y = tmp.Y, Z = tmp.Z}
	-- Y軸回転
	tmp.X = res.X * math.cos(boneRot.Y) + res.Z * math.sin(boneRot.Y)
	tmp.Y = res.Y
	tmp.Z = res.X * -math.sin(boneRot.Y) + res.Z * math.cos(boneRot.Y)
	res = {X = tmp.X, Y = tmp.Y, Z = tmp.Z}
	-- Z軸回転
	tmp.X = res.X * math.cos(boneRot.Z) + res.Y * -math.sin(boneRot.Z)
	tmp.Y = res.X * math.sin(boneRot.Z) + res.Y * math.cos(boneRot.Z)
	tmp.Z = res.Z
	res = {X = tmp.X, Y = tmp.Y, Z = tmp.Z}
	-- 座標移動
	res = {res.X + bonePos.X, -(res.Y + bonePos.Y), res.Z + bonePos.Z}
	
	return res
	--[[
	-- まとめ（ちょっとずれるので計算見直しが必要？）
	return {
		res.X * math.cos(boneRot.Z) * math.cos(boneRot.Y) + res.Y * -math.sin(boneRot.Z) + res.Z * math.sin(boneRot.Y) + bonePos.X,
		res.X * math.sin(boneRot.Z) + res.Y * math.cos(boneRot.Z) * math.cos(boneRot.X) + res.Z * -math.sin(boneRot.X) + bonePos.Y,
		res.X * -math.sin(boneRot.Y) + res.Y * math.sin(boneRot.X) + res.Z * math.cos(boneRot.X) * math.cos(boneRot.Y) + bonePos.Z,
	}
	--]]
end

local function SetBone(name, anime)
	local model = models[name]
	local vertices  = {}
	local color = Color('White')
	-- 頂点の計算を先に済ませておく
	local meshVertices = {}
	for i=0, #model['Pos'] do
		meshVertices[i] = SetupVertecies(name, i, anime)
	end
	-- 面の数だけ定義
	for i=1, #model['Face'] do
		-- 3点定義
		for key,number in ipairs(model['Face'][i]) do
			if meshVertices[number] then
				vertices[#vertices + 1] = {
					{
						meshVertices[number][1],
						meshVertices[number][2],
						meshVertices[number][3],
					},
					color,
					{
						models[name]['Pos'][number]['U'],
						models[name]['Pos'][number]['V'],
					},
				}
			end
		end
	end
	return vertices
end

local animeCount = 0
local timer = 0
local wait = 0.05
local old = nil
local function update(self, delta)
	timer = timer + delta
	if not old or timer-old >= wait then
		animeCount = math.floor(timer*50) % boneMax
		for name, model in pairs(models) do
			local child = self:GetChild(name)
			if child then
				child:stoptweening()
				child:linear(wait)
				child:SetVertices(SetBone(name, animeCount))
			end
		end
		old = timer
	end
end

for name, model in pairs(models) do
	actors[#actors+1] =  Def.ActorMultiVertex({
		Name = name,
		InitCommand = function(self, params)
            self:cullmode('CullMode_Back')
			self:SetDrawState({ Mode = 'DrawMode_Triangles' })
			--self:SetDrawState({ Mode = 'DrawMode_LineStrip' })
			self:SetVertices(SetBone(name, 0))
		end,
		SetTextureMessageCommand = function(self, params)
			if name == 'Face' then
				self:SetTexture(params.TextureFace)
			else
				self:SetTexture(params.TextureMain)
			end
		end
	})
end
local tex
actors[#actors+1] =  Def.Sprite({
	Name = 'TextureMain',
	InitCommand = function(self)
		self:Load(THEME:GetPathG('', '_tmp/tex_full.png'))
		self:visible(false)
	end
})
actors[#actors+1] =  Def.Sprite({
	Name = 'TextureFace',
	InitCommand = function(self)
		self:Load(THEME:GetPathG('', '_tmp/tex_full.png'))
		self:visible(false)
	end
})

actors.InitCommand = function(self)
	self:SetUpdateFunction(update)
	local texMain = self:GetChild('TextureMain'):GetTexture()
	local texFace = self:GetChild('TextureFace'):GetTexture()
	self:playcommand('SetTexture', {TextureMain = texMain, TextureFace = texFace})
end

return actors
--[[
return Def.ActorMultiVertex({
	SetVerticesMessageCommand = function(self, params)
		local scale     = params.Scale or 100
		local number    = params.Number or 8
		local faceColor = params.Color or Color('White')
		local texture   = params.Texture
		local texUv     = params.Uv
		local uv        = {}
		if texture then
			self:SetTexture(texture)
			uv = {texUv[1], texUv[2], texUv[3], texUv[4]}
		else
			uv = {0, 0, 0, 0}
		end
		self:visible(true)
		local vertices  = {}
		self:SetDrawState({ Mode = 'DrawMode_Fan' })
		vertices[#vertices + 1] = {
			{
				0,
				0,
				0,
			},
			faceColor,
			{(uv[3]-uv[1])/2 + uv[1], (uv[4]-uv[2])/2 + uv[2]},
		}
		for i = 0, number do
			local uvX = 1.0 * (uv[3]-uv[1])/2 * math.sin(math.pi*i*2/number) + (uv[3]-uv[1])/2 + uv[1]
			local uvY = -1.0 * (uv[4]-uv[2])/2 * math.cos(math.pi*i*2/number) + (uv[4]-uv[2])/2 + uv[2]
			vertices[#vertices + 1] = {
				{
					math.sin(2.0 * math.pi * i / number) * scale / 2,
					0,
					math.cos(2.0 * math.pi * i / number) * scale / 2,
				},
				faceColor,
				{uvX, uvY},
			}
		end
		if vertices and #vertices > 0 then
			self:SetVertices(vertices)
		end
	end,
})
--]]
