-- 3Dモデル用関数

-- Model情報の読み込み
local function LoadModel(self, filePath)
	local m = YA_FILE:Open(filePath)
	if not m then
		_SYS('ERROR')
		return {}
	end
	local file = m:File()
	local model = {}

	local line = ''
	local readMode = -1
	local meshCount = 0
	local meshName = ''
	-- 初期位置
	file:Seek(0)
	local lineCount = 0
	while true do
		-- 1行ずつ取得
		line = file:GetLine()
		lineCount = lineCount +1
		if line ~= '' then
			if readMode == 4 then
				-- 面
				local values = split(' ', line)
				if string.sub(line, 1, 1) == '"' then
					-- 次
					readMode = 0
				elseif #values ~= 8 then
					-- 終わり
					break
				else
					-- 面
					model[meshName]['Face'][meshCount] = {
						tonumber(values[2]), tonumber(values[3]), tonumber(values[4]),
					}
					meshCount = meshCount+1
				end
			end
			if readMode == 3 then
				-- 法線
				local values = split(' ', line)
				if #values ~= 1 then
					-- モデル
					model[meshName]['Normal'][meshCount] = {
						X = tonumber(values[2]),
						Y = tonumber(values[3]),
						Z = tonumber(values[4]),
					}
					meshCount = meshCount+1
				else
					-- 面数
					meshCount = 0
					readMode = 4
				end
			end
			if readMode == 2 then
				-- モデル
				local values = split(' ', line)
				if #values ~= 1 then
					-- モデル
					model[meshName]['Pos'][meshCount] = {
						X      = tonumber(values[2]),
						Y      = tonumber(values[3]),
						Z      = tonumber(values[4]),
						U      = tonumber(values[5]),
						V      = tonumber(values[6]),
						BoneId = tonumber(values[7]),
					}
					meshCount = meshCount+1
				else
					-- 法線数
					meshCount = 0
					readMode = 3
				end
			end
			if readMode == 1 then
				-- メッシュ数
				meshCount = 0
				model[meshName] = {
					Count  = tonumber(line),
					Pos    = {},	-- 位置とUV座標
					Normal = {},	-- 法線
					Face   = {},	-- 面
				}
				readMode = 2
			end
			if string.sub(line, 1, 1) == '"' and readMode == 0 then
				-- メッシュ名
				meshName = split('"', line)[2]
				readMode = 1
			end

			if (string.sub(line, 1, 7) == 'Meshes:') then
				readMode = 0	-- 初期状態
			end
		end
				
		-- 終端に到達した場合チェック終了
		if file:AtEOF() then
			break
		end
	end

	m:Close()
	return model
end

-- Bone情報の読み込み
local function LoadBone(self, filePath)
	local b = YA_FILE:Open(filePath)
	if not b then
		_SYS('ERROR')
		return {}
	end
	local file = b:File()
	local bones = {}

	local line = ''
	local readMode = -1
	local boneId = -1
	local boneName = ''
	-- 初期位置
	file:Seek(0)
	local lineCount = 0
	while true do
		-- 1行ずつ取得
		line = file:GetLine()
		lineCount = lineCount +1
		if line ~= '' then
			if readMode == 4 then
				-- 回転
				local values = split(' ', line)
				if #values == 1 or string.sub(line, 1, 1) == '"' then
					-- 次
					readMode = 0
				elseif tonumber(values[1]) ~= nil then
					-- Bone識別番号
					bones[boneId]['Rot'][math.floor(tonumber(values[1]))] = {
						X = tonumber(values[2]),
						Y = tonumber(values[3]),
						Z = tonumber(values[4]),
					}
				end
			end
			if readMode == 3 then
				-- 位置
				local values = split(' ', line)
				if #values ~= 1 then
					-- Bone識別番号
					bones[boneId]['Pos'][math.floor(tonumber(values[1]))] = {
						X = tonumber(values[2]),
						Y = tonumber(values[3]),
						Z = tonumber(values[4]),
					}
				elseif values[1] ~= nil then
					-- Bone回転？アニメーション数
					readMode = 4
				end
			end
			if readMode == 2 then
				-- モデル
				local values = split(' ', line)
				if #values ~= 1 then
					-- 中身のないモデル
				elseif values[1] ~= nil then
					-- Boneアニメーション数
					boneId = boneId+1
					bones[boneId] = {
						Count = tonumber(values[1]),
						Pos   = {},
						Rot   = {},
					}
					--boneMax = (bones[boneId]['Count'] > boneMax) and bones[boneName]['Count'] or boneMax
					readMode = 3
				end
			end
			if readMode == 1 then
				-- よくわからん
				readMode = 2
			end
			if string.sub(line, 1, 1) == '"' and readMode == 0 then
				-- Bone名
				boneName = split('"', line)[2]
				readMode = 1
			end

			if (string.sub(line, 1, 6) == 'Bones:') then
				readMode = 0	-- 初期状態
			end
		end
				
		-- 終端に到達した場合チェック終了
		if file:AtEOF() then
			break
		end
	end

	b:Close()
	return bones
end

return {
	Model = LoadModel,
	Bone  = LoadBone,
}