local width, showStyle = ...
width = width or 100
showStyle = (showStyle ~= nil) and showStyle or false
local height = 18

local path = string.match(W:Object('style/default.lua'), '(.+%/)[^/]+')
local iconList = FILEMAN:GetDirListing(path, false, false)
for i=1, #iconList do
    iconList[i] = string.lower(split(' ', iconList[i])[1])
end
iconList = ':'..table.concat(iconList, ':')..':'

-- スタイル
local stylePill = LoadActor(W:Object('radius'), {
    Id     = 'StylePill',
    Fill   = false,
    Width  = width*2,
    Height = height+4,
    Radius = height,
    Color  = color('#ffffff80'),

})
local styleFill = LoadActor(W:Object('radius'), {
    Id     = 'StyleFill',
    Width  = width*2,
    Height = height+4,
    Radius = height,
    Color  = color('#ffffff80'),

})

--  他スタイルが存在する場合の○アイコン
local styleList = {}
local function StyleActor(show)
    if not show then return Def.ActorFrame({}) end
    local w = 13
    local m = 6
    local currentStyle
    local a = Def.ActorFrame({
        OnCommand = function(self)
            styleList = W_STAGE:StyleList()
            self:playcommand('CheckStyle')
        end,
        PlayerJoinedMessageCommand = function(self, params)
            styleList = W_STAGE:StyleList()
            self:playcommand('CheckStyle')
        end,
        CallbackStepsMessageCommand = function(self, params)
            self:playcommand('CheckStyle')
        end,
        CheckStyleCommand = function(self, params)
            local mPn = GAMESTATE:GetMasterPlayerNumber()
            local step =  GAMESTATE:IsCourseMode() and GAMESTATE:GetCurrentTrail(mPn) or W:Steps(mPn)
            currentStyle = step and step:GetStepsType() or GAMESTATE:GetCurrentStyle():GetStepsType()
            local song = GAMESTATE:IsCourseMode() and GAMESTATE:GetCurrentCourse() or W:Song()
            local colList = {}
            local alpList = {}
            for i=1, m do
                colList[i] = color('0, 0, 0, 0')
                alpList[i] = 0
            end
            if song and currentStyle then
                local sIndex = 1
                local sList = {}
                for k,v in pairs(styleList) do
                    for i,steps in pairs(GAMESTATE:IsCourseMode() and song:GetAllTrails() or song:GetAllSteps()) do
                        if steps:GetStepsType() == v:GetStepsType() then
                            sList[#sList+1] = v
                            if currentStyle == v:GetStepsType() then
                                sIndex = #sList
                            end
                            break
                        end
                    end
                end
                for i=1, m do
                    local s = i - (m/2 - (sIndex-1))
                    if 1 <= s and s <= (#sList-1) then
                        local stepType = sList[(s>=sIndex) and s+1 or s]:GetStepsType()
                        if string.find(stepType, 'Single', 1, true) then
                            colList[i] = color('0, 0.5, 1, 0.15')
                        elseif string.find(stepType, 'Solo', 1, true) then
                            colList[i] = color('0.3, 0.9, 0, 0.15')
                        else
                            colList[i] = color('1, 0, 0.3, 0.15')
                        end
                        alpList[i] = 1
                    end
                end
            end
            self:playcommand('SetStyleSideIconColor', {Color = colList, Alpha = alpList})
        end,
    })
    for index=1, m do
        local i = index-(m/2+1)
        a[#a+1] = Def.ActorFrame({
            InitCommand = function(self)
                self:xy((i*w) + ((i<0) and -95 or 95+w), -3)
                self:rotationy((i<0) and 180 or 0)
            end,
            LoadActor('style')..{
                InitCommand = function(self)
                    self:animate(false)
                    self:setstate(0)
                end,
                SetStyleSideIconColorCommand = function(self, params)
                    self:diffuse(params.Color[index])
                end,
            },
            LoadActor('style')..{
                InitCommand = function(self)
                    self:animate(false)
                    self:setstate(1)
                end,
                SetStyleSideIconColorCommand = function(self, params)
                    self:diffusealpha(params.Alpha[index])
                end,
            },
        })
    end
    return a
end

return Def.ActorFrame({
    -- 他スタイルが存在する場合○アイコンを表示
    StyleActor(showStyle),
    styleFill:Actor()..{
        CallbackStepsCommand = function(self, params)
            local style = params.StepsType
            local col = color('1.0, 1.0, 1.0, 0.15')
            -- 雑判定
            if style and string.find(style, 'Single', 1, true) then
                col = color('0, 0.5, 1, 0.15')
            elseif style and string.find(style, 'Solo', 1, true) then
                col = color('0.3, 0.9, 0, 0.15')
            else
                col = color('1, 0, 0.3, 0.15')
            end
            self:playcommand('UpdateRadius', {Id = 'StyleFill', Color = col})
        end,
    },
    stylePill:Actor(),
    -- テキスト
    LoadFont('Common Small Normal')..{
        Text = '',
        InitCommand = function(self)
            self:zoom(0.5)
            self:maxwidth((width*2 - 60)/0.5)
            self:diffuse(Color('White'))
            self:xy(20, -1)
        end,
        CallbackStepsCommand = function(self, params)
            local text = string.upper(string.gsub(ToEnumShortString(params.StepsType), '_', ' ')) or ''
            if string.find(text, 'SINGLE$') and W_PLAYER:NumJoined() > 1 then
                text = string.gsub(text, 'SINGLE', 'VERSUS')
            end
            self:settext(text)
        end,
    },
    -- アイコン
    Def.Sprite({
        InitCommand=function(self)
            self:xy(-width + 10 + 15, 0)
            self:horizalign(center)
        end,
        CallbackStepsCommand = function(self, params)
            if string.find(iconList, string.lower(ToEnumShortString(params.StepsType), 1, true)) then
                self:Load(W:Object('style/'..ToEnumShortString(params.StepsType)))
            end
        end,
    }),
    Def.Quad({
        InitCommand=function(self)
            self:xy(-width + 45, 0)
            self:zoomto(1, 10)
            self:diffuse(1, 1, 1, 0.5)
        end,
    }),
})