-- ターゲットスコア
local pn, targetFileName, mini = ...
mini = mini or false

local isNoPlay = false

local targetMaxScore = 0
local targetMaxDP
local isSystem, systemName = W_TARGET:IsSystem(targetFileName)
local targetName = W_TARGET:Name(targetFileName)

local function TargetGroupActor(pn)
    local avatarPath = W_TARGET:Avatar(targetFileName)
    if isSystem and systemName == 'HighScore' then
        avatarPath = W_PLAYER:PlayerAvatar(pn)
    end
    return Def.ActorFrame({
        Name = 'Group',
        W_AVATAR:Actor(avatarPath, 'Target', {
            Mini  = false,
            Flip  = false,
            Type  = 0,
            -- ハイスコアが対象の場合、自分自身の名前でランダムカラーを設定する
            RandomNum = W_AVATAR:RandomNum((not isSystem or systemName ~= 'HighScore') and targetName or W_PLAYER:Name(pn)),
        })..{
            Name = 'TargetAvatar',
            OnCommand = function(self)
                self:zoom(0.36)
                self:xy(30, 0)
            end,
        },
        LoadFont('Common Normal')..{
            InitCommand = function(self)
                self:horizalign(right)
                self:xy(0, -8)
                self:diffuse(1, 1, 1, 1)
                self:strokecolor(Color('Outline'))
                self:zoom(0.5)
                self:settext('')
            end,
            -- 自身のスコアを設定
            SetTargetScoreMessageCommand = function(self, params)
                if isNoPlay then
                    self:settext('NO PLAY')
                else
                    self:settext(W_STRING:DP(0.0001 * math.floor(params.Diff*100), true))
                end
                self:diffuse(params.Color)
            end,
        },
        LoadFont('Common Normal')..{
            InitCommand = function(self)
                self:horizalign(right)
                self:xy(0, 8)
                self:diffuse(0.8, 0.8, 0.8, 1)
                self:strokecolor(Color('Outline'))
                self:zoom(0.5)
                self:maxwidth((mini and 60 or 140)/0.5)
                self:settext(targetName)
            end,
        },
    })
end

local function NormalActor(pn)
    local boxW, boxH = 260, 72
    local barW, barH = boxW - 10, 16
    local radius = LoadActor(W:Object('radius'), {
        Width  = boxW,
        Height = boxH,
        Radius = 6,
        Fill   = true,
        Color  = color('0, 0, 0, 0.25'),
    })
    return Def.ActorFrame({
        Name = 'Target',
        radius:Actor()..{
            InitCommand = function(self)
            end,
        },
        Def.ActorFrame({
            InitCommand = function(self)
                self:xy(0, -boxH/2 + barH/2 + 4)
            end,
            Def.Quad({
                InitCommand = function(self)
                    self:zoomto(barW+2, barH+2)
                    self:diffuse(0, 0, 0, 0.5)
                end,
            }),
            -- 対象のスコア
            Def.Quad({
                InitCommand = function(self)
                    self:horizalign(left)
                    self:x(-barW/2)
                    self:zoomto(0, barH)
                    self:diffuse(1, 1, 1, 0.75)
                end,
                -- 対象のスコアを設定
                SetTargetScoreMessageCommand = function(self, params)
                    self:zoomto(params.Target * barW / 100, barH)
                end,
            }),
            -- 自身のスコア
            Def.Quad({
                InitCommand = function(self)
                    self:horizalign(left)
                    self:x(-barW/2)
                    self:zoomto(0, barH)
                    self:diffuse(W_PLAYER:Color(pn))
                    self:diffusealpha(0.75)
                end,
                -- 自身のスコアを設定
                SetTargetScoreMessageCommand = function(self, params)
                    self:zoomto(params.Self * barW / 100, barH)
                end,
            }),
            -- 自身のスコア
            LoadFont('Common Normal')..{
                InitCommand = function(self)
                    self:x(barW/2 - 40)
                    self:diffuse(1, 1, 1, 1)
                    self:strokecolor(Color('Outline'))
                    self:zoom(0.62)
                    self:settextf('%4.1f%%', 0)
                end,
                -- 自身のスコアを設定
                SetTargetScoreMessageCommand = function(self, params)
                    if isNoPlay then
                        self:diffuse(1, 1, 1, 1)
                    else
                        self:diffuse(params.Color)
                    end
                    self:settext(W_STRING:DP(0.0001 * math.floor(params.Self*100)))
                end,
            },
        }),
        W_AVATAR:Actor(W_PLAYER:PlayerAvatar(pn), 'Player', {
            Mini  = false,
            Flip  = true,
            Type  = 0,
            Color = color('0, 0, 0, 0.3'),--BoostColor(W_PLAYER:Color(pn), 0.5),
            RandomNum = W_AVATAR:RandomNum(W_PLAYER:Name(pn)),
        })..{
            Name = 'PlayerAvatar',
            OnCommand = function(self)
                self:zoom(0.45)
                self:xy(-boxW/2 + 30, 5)
            end,
        },
        TargetGroupActor(pn)..{
            InitCommand = function(self)
                self:xy(boxW/2 - 60, 10)
            end,
        },
    })
end

local function MiniActor(pn)
    local boxW, boxH = 134, 52
    local radius = LoadActor(W:Object('radius'), {
        Width  = boxW-10,
        Height = boxH,
        Radius = 6,
        Fill   = true,
        Color  = color('0, 0, 0, 0.25'),
    })
    return Def.ActorFrame({
        Name = 'Target',
        radius:Actor()..{
            InitCommand = function(self)
                self:y(0)
            end,
            -- 自身のスコアを設定
            SetTargetScoreMessageCommand = function(self, params)
                if not isNoPlay then
                    local lead = W_TARGET:DiffToLead(params.Diff)
                    local col = color('0, 0, 0, 0.5')
                    if lead ~= 0 then
                        col = BoostColor(params.Color, 0.5)
                        col[4] = 0.66
                    end
                    self:playcommand('UpdateRadius', {
                        Color  = col,
                    })
                end
            end,
        },
        TargetGroupActor(pn)..{
            InitCommand = function(self)
                self:xy(boxW/2 - 60, 0)
            end,
        },
    })
end

local MainActor = mini and MiniActor(pn) or NormalActor(pn)

return Def.ActorFrame({
    InitCommand = function(self)
        if isSystem then
            if systemName == 'HighScore' then
                -- HighScoreがターゲット
                targetMaxScore, _, isNoPlay = W_SCORE:HighDPScore(pn, false)
            elseif string.match(systemName, '^%d+%%$') then
                -- DancePoints(%)がターゲット
                targetMaxScore = tonumber(string.match(systemName, '^(%d+)%%$'))
                isNoPlay = false
            else
                targetMaxScore = 0
                isNoPlay = true
            end
        else
            targetMaxScore, targetMaxDP, isNoPlay = W_TARGET:GetScore(targetFileName, pn)
        end
        local diff, col = W_STRING:DiffAndColor(0, 0)
        self:playcommand('SetTargetScore', {
            Self   = 0,
            Target = 0,
            Player = pn,
            Diff   = diff,
            Color  = col,
        })
    end,
    -- 判定時
    TargetScoreMessageCommand=function(self, params)
        -- 異なるプレイヤー
        if params.Player ~= pn then return end
        local targetScore = targetMaxScore * params.CurMaxDP / params.FullDP
        local diff, col
        if isNoPlay then
            diff, col = W_STRING:DiffAndColor(params.Self, params.Self)
        else
            diff, col = W_STRING:DiffAndColor(params.Self, targetScore)
        end
        self:playcommand('SetTargetScore', {
            Self   = params.Self,
            Target = targetMaxScore * params.CurMaxDP / params.FullDP,
            Player = pn,
            Diff   = diff,
            Color  = col,
        })
        -- 表情変更
        local playerAvatar = self:GetChild('Target'):GetChild('PlayerAvatar')
        local targetAvatar = self:GetChild('Target'):GetChild('Group'):GetChild('TargetAvatar')
        if diff > 0 then
            if not mini then
                playerAvatar:playcommand('WinEmotion',{})
            end
            targetAvatar:playcommand('LoseEmotion',{})
        elseif diff < 0 then
            if not mini then
                playerAvatar:playcommand('LoseEmotion',{})
            end
            targetAvatar:playcommand('WinEmotion',{})
        else
            if not mini then
                playerAvatar:playcommand('DefaultEmotion',{})
            end
            targetAvatar:playcommand('DefaultEmotion',{})
        end
    end,
    MainActor,
})