local selectButton = THEME:GetString('GameButton', 'Select')
local bodyText = {
    {
        ja = [[
WAIEI CitMinをご利用いただきありがとうございます。
本作はシンプルながらもアニメーションやデザインにこだわっており、
スコア計算式の変更といった過去作の機能の大半もそのまま使うことができます。

さらにgroup.iniを拡張したgroup.luaをサポートし、
パッケージ・プレイヤー側でより高度なカスタマイズもできるようになりました。
]],
        en = [[
Thank you for visiting WAIEI CitMin. This theme has the following features
- Seemingly simple but thoughtful design.
- Retains most of the features from the previous WAIEI series, such as changes to the score calculation formulas.
- Support for group.ini and group.lua, allowing more advanced customization on the package/player side.
]],
    },
    {
        ja = [[
テーマ専用のオプションはタイトルのオプションから
WAIEI OPTIONSを選ぶことで変更することができますが、
イベントモードの場合は&Select;を押すことで素早く切り替えることもできます。

&Select;はキー設定の「]]..selectButton..[[」に割り当てられているキーなので、
あらかじめ確認することをオススメします。
]],
        en = [[
The exclusive options for this topic are available in the Title Options. In event mode, you can also quickly switch between them by pressing &Select;.

Since &Select; is a key assigned to "]]..selectButton..[[" in the key settings, we recommend that you check it first.
]],
    },
    {
        ja = [[
WAIEI CitMin専用オプションをオススメの状態に一括設定します。
普段よく使用しているものをお選びください。

既に設定済みの場合、Defaultは現在の設定から変更しません。

設定後もWAIEI OPTIONSから変更可能です。
]],
        en = [[
Set WAIEI CitMin's exclusive options to the recommended state at once.
Please select the one you usually use the most.

If you have already set and selected "Default", it will not be changed from the current setting.
It can be changed from WAIEI OPTIONS even after setting.
]],
    },
    {
        ja = W:IsOFV() and [[
それでは、WAIEI CitMinをお楽しみください。

お使いのProject OutFoxはファイル認識処理に不具合があるため、
このままでは設定ファイルが正しく認識されません。
一度Project OutFoxを再起動をしてください。
]] or [[
不具合修正や機能追加は今後も続けるので
定期的なバージョンアップをよろしくお願いします。

それでは、WAIEI CitMinをお楽しみください。
]],
        en = W:IsOFV() and [[
We hope you will enjoy WAIEI CitMin.

The version of Project OutFox you are using has a problem with the file recognition process. If this is the case, the configuration file will not be recognized correctly.
Please restart Project OutFox.
]] or [[
We will continue to fix bugs and add features.
Please update your version regularly.

We hope you will enjoy WAIEI CitMin.
]],
    },
}
local graphicPath = 'Screen draw/Graphics/feature/300/'

local lang = THEME:GetCurLanguage()
local current = 0
local text = ''
return Def.ActorFrame({
    Def.Sprite({
        InitCommand = function(self)
            self:LoadBackground(W_CUSTOM:Path(string.format('%s%s.png', graphicPath, 1)))
            self:scaletofit(-SCREEN_CENTER_X, -100, SCREEN_CENTER_X, 100)
            self:y(-80)
        end,
        ChangedPageMessageCommand = function(self, params)
            local value = ''
            if params.Value then
                value = '_'..params.Value
            end
            self:stoptweening()
            self:diffusealpha(0)
            self:LoadBackground(W_CUSTOM:Path(string.format('%s%s%s.png', graphicPath, params.Page, value)))
            self:scaletofit(-SCREEN_CENTER_X, -100, SCREEN_CENTER_X, 100)
            self:y(-80)
            self:linear(0.2)
            self:diffusealpha(1)
        end,
        ChangedValueMessageCommand = function(self, params)
            self:playcommand('ChangedPage', params)
        end,
    }),
    -- 左右の矢印（選曲ホイールから拝借）
    Def.ActorFrame({
        InitCommand=function(self)
            self:zoom(2)
            self:y(-70)
        end,
        ChangedPageMessageCommand = function(self, params)
            self:visible(not not params.Value)
        end,
        LoadActor(THEME:GetPathG('ScreenSelectMusic/wheel/frame', 'over'), 180, 'timer'),
    }),
    Def.ActorFrame({
        InitCommand = function(self)
            self:xy(-310, 50)
        end,
        Def.Actor({
            ChangedPageMessageCommand = function(self, params)
                text = bodyText[params.Page][lang] or bodyText[params.Page].en or ''
                current = 0
            end,
        }),
        Def.Quad({
            InitCommand = function(self)
                self:horizalign(left)
                self:vertalign(top)
                self:xy(-1, -1)
                self:zoomto(622, 142)
                self:diffuse(1, 1, 1, 0.5)
            end,
        }),
        Def.Quad({
            InitCommand = function(self)
                self:horizalign(left)
                self:vertalign(top)
                self:zoomto(620, 140)
                self:diffuse(0, 0, 0, 0.85)
            end,
        }),
        Def.Quad({
            InitCommand = function(self)
                self:horizalign(left)
                self:vertalign(top)
                self:zoomto(10, 20)
                self:diffuse(1, 1, 1, 0)
            end,
            ChangedPageMessageCommand = function(self, params)
                self:finishtweening()
                self:xy(5, 5)
                self:queuecommand('Animation')
            end,
            AnimationCommand = function(self)
                local textList = split('\n', text)
                local function Move(self, curLine)
                    if curLine > #textList-1 then return end
                    local moveX = math.min(string.len(textList[curLine])*5, 600)
                    self:diffusealpha(0.5)
                    self:linear(string.len(textList[curLine]) * 0.0019)
                    self:addx(moveX)
                    self:diffusealpha(0)
                    self:sleep(0.001)
                    self:addx(-moveX)
                    self:addy(21)
                end
                self:diffuse(1, 1, 1, 0.3)
                self:sleep(0.05)
                self:diffuse(1, 1, 1, 0)
                self:sleep(0.05)
                self:diffuse(1, 1, 1, 0.3)
                self:sleep(0.05)
                --self:diffuse(1, 1, 1, 0)
                Move(self, 1)
                Move(self, 2)
                Move(self, 3)
                Move(self, 4)
                Move(self, 5)
                Move(self, 6)
            end,
        }),
        LoadFont('Common Normal')..{
            Text = bodyText[1][lang] or bodyText[1].en or '',
            InitCommand = function(self)
                self:horizalign(left)
                self:vertalign(top)
                self:xy(10, 10)
                self:zoom(0.66)
                self:vertspacing(10)
                self:wrapwidthpixels(600/0.66)
            end,
            ChangedPageMessageCommand = function(self, params)
                self:stoptweening()
                self:settext('')
                self:sleep(0.2)
                self:queuecommand('Clear')
            end,
            ClearCommand = function(self)
                self:diffuse(0, 0, 0, 0.1)
                self:settext(text)
                self:queuecommand('Trans')
            end,
            TransCommand = function(self)
                self:stoptweening()
                self:AddAttribute(0, {Length = current, Diffuse = color('1, 1, 1, 1')})
                self:sleep(0.005)
                if current < string.len(text) then
                    current = current + 1
                    self:queuecommand('Trans')
                end
            end,
        }
    }),
})