local pn, pss = ...
local width  = 170
local height = 35

local function LineColor(size)
    if size >= 0.99 then
        return color('#ccff99ff')
    elseif size <= 0 then
        return color('#e00000ff')
    elseif size <= 0.20 then
        return color('#ff9933ff')
    end
    return color('#ffffff80')
end

local function GraphActor()
    if GAMESTATE:IsCourseMode() then
        local h = height/16
        local function BorderActor(c, max)
            local col = color('1, 1, 1, 0.25')
            return LoadActor(W:Object('border'), {
                From = {
                    X     = 0,
                    Y     = height - height*c/max,
                    Color = col,
                },
                To = {
                    X     = width,
                    Y     = height - height*c/max,
                    Color = col,
                },
            })
        end
        return Def.ActorFrame({
            OnCommand= function(self)
                self:xy(-width/2, -height)
            end,
            -- 背景
            Def.Quad({
                InitCommand = function(self)
                    self:horizalign(left)
                    self:vertalign(bottom)
                    self:diffuse(color('#ffffff16'))
                    self:zoomto(width, 12 * h)
                    self:y(16 * h)
                end
            }),
            Def.Quad({
                InitCommand = function(self)
                    self:horizalign(left)
                    self:vertalign(bottom)
                    self:diffuse(color('#ffffff16'))
                    self:zoomto(width, 4 * h)
                    self:y(4 * h)
                end
            }),
            BorderActor( 4, 16),
            BorderActor( 8, 16),
            BorderActor(12, 16),
        })
    end
    return LoadActor(W:Object('graph steps'), {
        Width   = width,
        Size    = height/16,
        Max     = height,
        BgColor = color('#ffffff16'),
        Player  = pn,
        BgFade  = false,
    })..{
        OnCommand= function(self)
            local song = W:Song()
            self:playcommand('ChangedGameplaySong', {
                Song     = song,
                LastSec  = song:GetLastSecond(),
                FirstSec = song:GetFirstSecond(),
            })
        end,
    }
end

local function GetFirstSec()
    if GAMESTATE:IsCourseMode() then
        return 0
    end
    return W:Song():GetFirstSecond()
end

local function GetTotalSec()
    if GAMESTATE:IsCourseMode() then
        return GAMESTATE:GetCurrentTrail(pn):GetLengthSeconds()
    end
    return W:Song():GetStepsSeconds()
end

return Def.ActorFrame({
    -- 譜面グラフ
    GraphActor(),
    -- ライフグラフ
    Def.ActorMultiVertex({
        OnCommand = function(self)
            local sample = width
            local last = GetTotalSec()
            local lives = pss:GetLifeRecord(last, sample)
            local areaWidth = width * (last - GetFirstSec()) / last
            local v = {
                {{-width/2, -1 * lives[1] * height, 0}, LineColor(lives[1])}
            }
            for i=1, #lives do
                local x = areaWidth * (i-1) / (sample-1) - width/2 + (width-areaWidth)
                if x >= -width/2 then
                    v[#v+1] = {{x, -1 * lives[i] * height, 0}, LineColor(lives[i])}
                end
            end
            self:SetDrawState({Mode = 'DrawMode_LineStrip'})
            self:SetVertices(1, v)
        end,
    }),
})