local pn, pss = ...
local width = 140
local height = 24
local scoreList = {
    'TapNoteScore_W1',
    'TapNoteScore_W2',
    'TapNoteScore_W3',
    'TapNoteScore_W4',
    'TapNoteScore_W5',
    'TapNoteScore_Miss',
}
local noteScore = getenv('WAIEI_NoteScore')[pn] or {}
local columns = GAMESTATE:GetCurrentStyle():ColumnsPerPlayer()
local arrowRotate = {
    Default = {-90, 180, 0, 90},
    StepsType_Dance_Single = {-90, 180, 0, 90},
    StepsType_Dance_Double = {-90, 180, 0, 90, -90, 180, 0, 90},
    StepsType_Dance_Couple = {-90, 180, 0, 90},
    StepsType_Dance_Solo   = {-90, -45, 180, 0, 45, 90},
    StepsType_Dance_Threepanel = {(not W:IsOF()) and -90 or -45, 180, (not W:IsOF()) and 90 or 45},
    StepsType_Pump_Single = {-135, -45, 'pump_center', 45, 135},
    StepsType_Pump_Halfdouble = {'pump_center', 45, 135, -135, -45, 'pump_center'},
    StepsType_Pump_Double = {-135, -45, 'pump_center', 45, 135, -135, -45, 'pump_center', 45, 135},
    StepsType_Techno_Single4 = {-90, 180, 0, 90},
    StepsType_Techno_Single5 = {-135, -45, 'smx_center', 45, 135},
    StepsType_Techno_Single8 = {-135, -90, -45, 180, 0, 45, 90, 135},
    StepsType_Techno_Single9 = {-135, -90, -45, 180, 'smx_center', 0, 45, 90, 135},
    StepsType_Techno_Double4 = {-90, 180, 0, 90, -90, 180, 0, 90},
    StepsType_Techno_Double5 = {-135, -45, 'smx_center', 45, 135, -135, -45, 'smx_center', 45, 135},
    StepsType_Techno_Double8 = {-135, -90, -45, 180, 0, 45, 90, 135, -135, -90, -45, 180, 0, 45, 90, 135},
    StepsType_Techno_Double9 = {-135, -90, -45, 180, 'smx_center', 0, 45, 90, 135, -135, -90, -45, 180, 'smx_center', 0, 45, 90, 135},
    StepsType_Smx_Single = {-90, 180, 'smx_center', 0, 90},
    StepsType_Smx_Double6 = {-90, 'smx_center', 90, -90, 'smx_center', 90},
    StepsType_Smx_Double10 = {-90, 180, 'smx_center', 0, 90, -90, 180, 'smx_center', 0, 90},
}

-- 判定ラベル
local function LabelActor(params)
    return Def.ActorFrame({
        LoadFont('Common small normal')..{
            Text = string.upper(params.Text),
            OnCommand = function(self)
                self:horizalign(params.Horizalign or left)
                self:vertalign(top)
                self:diffuse(params.Color)
                self:zoom(0.35)
            end,
        },
    })
end

-- 判定数値（初期）
local function DetailActor(params)
    return Def.ActorFrame({
        LoadFont('waiei2 Score Numbers')..{
            Text = string.upper(params.Text),
            OnCommand = function(self)
                self:horizalign(params.Horizalign or right)
                self:vertalign(top)
                self:diffuse(params.Color)
                self:zoom(0.8)
                if params.MaxWidth then
                    self:maxwidth(params.MaxWidth / 0.8)
                end
            end,
        },
    })
end

local t = Def.ActorFrame({})

-- 各数値（初期表示）
local defaultValue = Def.ActorFrame({
    -- 詳細表示時は非表示
    HideDetailMessageCommand = function(self)
        self:stoptweening()
        self:linear(0.2)
        self:diffusealpha(1)
    end,
    ShowDetailMessageCommand = function(self, params)
        if pn ~= params.Player then return end
        self:stoptweening()
        self:linear(0.2)
        self:diffusealpha(0)
    end,
})
for i,score in pairs(scoreList) do
    defaultValue[#defaultValue+1] = DetailActor({
        Text  = pss:GetTapNoteScores(score),
        Color = W_JUDGMENT:Color(score),
    })..{
        OnCommand = function(self)
            self:xy(width/2, (i-1)*height)
        end,
    }
end
t[#t+1] = defaultValue

-- 各数値（詳細表示）
local detailValue = Def.ActorFrame({
    -- 詳細表示時に表示
    HideDetailMessageCommand = function(self)
        self:stoptweening()
        self:linear(0.2)
        self:diffusealpha(0)
    end,
    ShowDetailMessageCommand = function(self, params)
        if pn ~= params.Player then return end
        self:stoptweening()
        self:linear(0.2)
        self:diffusealpha(1)
    end,
    OnCommand = function(self)
        self:diffusealpha(0)
    end,
})
local detailWidth = 300
local rotate = arrowRotate[GAMESTATE:GetCurrentStyle():GetStepsType()]
                or arrowRotate.Default
local totalJudgment = {}
for c=1, columns do
    -- 矢印画像
    detailValue[#detailValue+1] = LoadActor((type(rotate[c] or 0) == 'string') and rotate[c] or 'arrow')..{
        OnCommand = function(self)
            self:x(width/2+200-detailWidth + ((c-0.5)*detailWidth/columns)+8
                     + (W:IsP1(pn) and 0 or -200))
            self:y(-height*0.75)
            self:diffusealpha(0.75)
            self:rotationz((type(rotate[c] or 0) == 'string') and 0 or rotate[c] or 0)
        end,
    }
    -- 数値
    for i,score in pairs(scoreList) do
        totalJudgment[c] = (totalJudgment[c] or 0) + (noteScore[c][score] or 0)
        detailValue[#detailValue+1] = DetailActor({
            Text     = noteScore[c][score] or 0,
            Color    = W_JUDGMENT:Color(score),
            MaxWidth = detailWidth/columns - 10,
        })..{
            OnCommand = function(self)
                self:x(width/2+200-detailWidth + (c*detailWidth/columns) - 3
                         + (W:IsP1(pn) and 0 or -200))
                self:y((i-1)*height)
            end,
        }
    end
end
-- 合計値
for i,score in pairs(scoreList) do
    detailValue[#detailValue+1] = DetailActor({
        Text  = pss:GetTapNoteScores(score),
        Color = W_JUDGMENT:Color(score),
        Horizalign = left,
    })..{
        OnCommand = function(self)
            self:xy((pn == PLAYER_1) and (-width/2) or (-width*2+10), (i-1)*height + height/2 - 4)
            self:vertalign(bottom)
            self:diffuse(W_JUDGMENT:Color(score))
            self:zoom(0.5)
        end,
    }
end

-- 縦線兼パーセンテージ
local barSize = {}
for c=1, columns do
    local posY = -7
    barSize[c] = {}
    for i,score in pairs(scoreList) do
        local size = (noteScore[c][score] or 0) * (height*6) / (totalJudgment[c] or 1)
        barSize[c][score] = {Y = posY, Height = size}
        posY = posY + size
    end
end
for c=1, columns do
    if (totalJudgment[c] or 0) > 0 then
        -- 各判定で分割してグラフ表示
        for i,score in pairs(scoreList) do
            detailValue[#detailValue+1] = Def.Quad({
                OnCommand = function(self)
                    self:vertalign(top)
                    self:diffuse(W_JUDGMENT:Color(score))
                    self:x(width/2+200-detailWidth + (c*detailWidth/columns) + 2
                            + (W:IsP1(pn) and 0 or -200))
                    self:y(barSize[c][score]['Y'])
                    self:zoomto(i%2 + 1, barSize[c][score]['Height'])
                end,
            })
        end
    else
        -- 0の時は半透明白棒
        detailValue[#detailValue+1] = Def.Quad({
            OnCommand = function(self)
                self:vertalign(top)
                self:diffuse(1.0, 1.0, 1.0, 0.33)
                self:x(width/2+200-detailWidth + (c*detailWidth/columns) + 2
                        + (W:IsP1(pn) and 0 or -200))
                self:y(-7)
                self:zoomto(2, height*6)
            end,
        })
    end
end

-- 左側に固定される項目
local leftColumn = Def.ActorFrame({
    HideDetailMessageCommand = function(self)
        self:stoptweening()
        self:linear(0.2)
        self:x(0)
    end,
    ShowDetailMessageCommand = function(self, params)
        if pn ~= params.Player then return end
        self:stoptweening()
        self:linear(0.2)
        self:x(W:IsP1(pn) and 0 or -200)
    end,
})

-- OK
leftColumn[#leftColumn+1] = DetailActor({
    Text  = pss:GetHoldNoteScores('HoldNoteScore_Held'),
    Color = W_JUDGMENT:Color('HoldNoteScore_Held'),
})..{
    OnCommand = function(self)
        self:xy(width/2, (#scoreList)*height)
    end,
}

-- MaxCombo
leftColumn[#leftColumn+1] = DetailActor({
    Text  = pss:MaxCombo(),
    Color = Color('Orange'),
})..{
    OnCommand = function(self)
        self:xy(width/2, (#scoreList+1)*height)
    end,
}
-- MIGS (Marvelous Incorporated Grading System)
local migs = pss:GetTapNoteScores('TapNoteScore_W1') * 3
                + pss:GetTapNoteScores('TapNoteScore_W2') * 2
                + pss:GetTapNoteScores('TapNoteScore_W3')
                - pss:GetTapNoteScores('TapNoteScore_W5') * 4
                - pss:GetTapNoteScores('TapNoteScore_Miss') * 8
                + pss:GetHoldNoteScores('HoldNoteScore_Held') * 6
detailValue[#detailValue+1] = DetailActor({
    Text  = migs,
    Color = color('#B0B0B0FF'),
})..{
    OnCommand = function(self)
        self:xy(width/2 + (W:IsP1(pn) and 200 or 0), (#scoreList)*height)
    end,
}
detailValue[#detailValue+1] = DetailActor({
    Text = pss:GetActualDancePoints(),
    Color = color('#B0B000FF'),
})..{
    OnCommand = function(self)
        self:xy(width/2 + (W:IsP1(pn) and 200 or 0), (#scoreList+1)*height)
    end,
}

-- グラフ
detailValue[#detailValue+1] = LoadActor('graph', pn, pss)..{
    InitCommand = function(self)
        self:xy(W:IsP1(pn) and 200 or 0, (#scoreList+3)*height+5)
    end,
}

for i,score in pairs(scoreList) do
    local judgment = {
        Text  = W_JUDGMENT:Name(score),
        Color = W_JUDGMENT:Color(score),
    }
    leftColumn[#leftColumn+1] = LabelActor(judgment)..{
        OnCommand = function(self)
            self:xy(-width/2, (i-1)*height)
        end,
    }
end
-- OK
leftColumn[#leftColumn+1] = LabelActor({
    Text  = W_JUDGMENT:Name('HoldNoteScore_Held'),
    Color = W_JUDGMENT:Color('HoldNoteScore_Held'),
})..{
    OnCommand = function(self)
        self:xy(-width/2, (#scoreList)*height)
    end,
}
--MaxCombo
leftColumn[#leftColumn+1] = LabelActor({
    Text  = 'MaxCombo',
    Color = Color('Orange'),
})..{
    OnCommand = function(self)
        self:xy(-width/2, (#scoreList+1)*height)
    end,
}

t[#t+1] = leftColumn

-- MIGS
detailValue[#detailValue+1] = LabelActor({
    Text  = 'MIGS',
    Color = color('#B0B0B0FF'),
})..{
    OnCommand = function(self)
        self:xy(-width/2 + (W:IsP1(pn) and 200 or 0), (#scoreList)*height)
    end,
}
-- EXSCORE
detailValue[#detailValue+1] = LabelActor({
    Text  = 'EXSCORE',
    Color = color('#B0B000FF'),
})..{
    OnCommand = function(self)
        self:xy(-width/2 + (W:IsP1(pn) and 200 or 0), (#scoreList+1)*height)
    end,
}

-- 境界線
for i=1, 8 do
    t[#t+1] = Def.Quad({
        OnCommand = function(self)
            self:diffuse(1.0, 1.0, 1.0, 0.1)
            self:y((i-1)*height - 7)
            self:zoomto(width + 30, 1)
        end,
        HideDetailMessageCommand = function(self)
            self:stoptweening()
            self:linear(0.2)
            self:x(0)
            self:zoomto(width + 30, 1)
        end,
        ShowDetailMessageCommand = function(self, params)
            if pn ~= params.Player then return end
            self:stoptweening()
            self:linear(0.2)
            self:x(W:IsP1(pn) and 100 or -100)
            self:zoomto(width + 230, 1)
        end,
    })
end
t[#t+1] = detailValue

return t
