local moveY = 95
local cntInPage = math.floor((SCREEN_HEIGHT - 100) / moveY)
local scoreList = Def.ActorFrame({})

local function ScoreActor(i, ss)
    local maxStage = STATSMAN:GetStagesPlayed()
    local function PlayerActor(pn)
        local pss = ss:GetPlayerStageStats(pn)
        if not GAMESTATE:IsPlayerEnabled(pn) or not pss then
            return Def.ActorFrame({})
        end
        return Def.ActorFrame({
            LoadActor('_parts/fullcombo_bg', pn, pss)..{
                OnCommand = function(self)
                    self:xy((pn==PLAYER_1) and -SCREEN_CENTER_X or SCREEN_CENTER_X, 14)
                    self:zoomy(0.5)
                end,
            },
            Def.ActorFrame({
                LoadActor('_parts/score', pn, pss, true)..{
                    Name = 'Score',
                    OnCommand = function(self)
                        local add = W:IsP1(pn) and -180 or 180
                        self:x(add)
                        self:zoom(0.8)
                    end,
                },
                LoadActor('_parts/difficulty', pn, ss)..{
                    OnCommand = function(self)
                        local add = 320-60 + math.min((SCREEN_WIDTH-640)/6)
                        self:xy((pn==PLAYER_1) and -add+10 or add-5, 30)
                        self:zoom(0.65)
                    end,
                },
            }),
        })
    end
    local song = ss:GetPlayedSongs()[1]
    return Def.ActorFrame({
        OnCommand = function(self)
            self:xy(SCREEN_CENTER_X, (maxStage - (i-1))*moveY - 10)
        end,
        Def.Quad({
            OnCommand = function(self)
                self:diffuse(0, 0, 0, 0.8)
                self:xy(0, 12)
                self:zoomto(SCREEN_WIDTH, 80)
            end,
        }),
        PlayerActor(PLAYER_1),
        PlayerActor(PLAYER_2),
        -- バナーはアス比を維持して切り抜く
        W_GRAPHICS:FixedRatio(
            128*0.8,
            40*0.8,
            song:GetBannerPath()
                or THEME:GetPathG('Common fallback', 'banner')
        )..{
            OnCommand = function(self)
                if song:GetDisplaySubTitle() ~= '' then
                    self:y(10)
                else
                    self:y(12)
                end
            end,
        },
        LoadFont('Common Small Normal')..{
            OnCommand = function(self)
                self:diffuse(YA_GROUP:MenuColor(song))
                self:diffusealpha(1)
                self:settext(song:GetDisplayMainTitle())
                if song:GetDisplaySubTitle() ~= '' then
                    self:y(33)
                else
                    self:y(38)
                end
                self:zoom(0.4)
                self:maxwidth(120/0.4)
            end,
        },
        LoadFont('Common Small Normal')..{
            OnCommand = function(self)
                self:diffuse(BoostColor(YA_GROUP:MenuColor(song), 0.85))
                self:diffusealpha(1)
                self:settext(song:GetDisplaySubTitle())
                self:y(42)
                self:zoom(0.32)
                self:maxwidth(120/0.32)
            end,
        },
        LoadFont('Common Bold')..{
            OnCommand = function(self)
                self:diffuse(Color('White'))
                self:settext(W_STAGE:BySS(ss))
                self:y(-15)
                self:zoom(0.5)
                self:maxwidth(120/0.5)
            end,
        },
    })
end

local i = 0
while(true) do
    i = i+1
    local ss = STATSMAN:GetPlayedStageStats(i)
    if not ss then
        break
    end
    scoreList[#scoreList+1] = ScoreActor(i, ss)
end
local stageCount = i-1
local cur = 0

return Def.ActorFrame({
    LoadActor(THEME:GetPathB('ScreenWithMenuElements', 'background')),
    -- キー操作
    Def.Actor({
        CodeMessageCommand = function(self, params)
            if params.Name == 'Up' then
                cur = cur - 1
            end
            if params.Name == 'Down' then
                cur = cur + 1
            end
            cur = math.max(math.min(cur, stageCount-cntInPage), 0)
            self:GetParent():playcommand('Scroll', {Cursor = cur})
        end,
    }),
    Def.ActorScroller({
        NumItemsToDraw = stageCount,
        SecondsPerItem = 0.2,
        UseMask = false,
        TransformFunction=function(self, offset, itemIndex, numItems)
            self:y(moveY * offset)
        end,
        ScrollCommand = function(self, params)
            self:SetDestinationItem(params.Cursor)
        end,
        scoreList,
    }),
    Def.Quad({
        OnCommand = function(self)
            self:horizalign(left)
            self:vertalign(top)
            self:xy(SCREEN_LEFT, SCREEN_TOP)
            self:zoomto(SCREEN_WIDTH, 25)
            self:diffuse(0, 0, 0, 0.8)
        end,
    }),
    Def.Quad({
        OnCommand = function(self)
            self:horizalign(left)
            self:vertalign(bottom)
            self:xy(SCREEN_LEFT, SCREEN_BOTTOM)
            self:zoomto(SCREEN_WIDTH, 25)
            self:diffuse(0, 0, 0, 0.8)
        end,
    }),
    Def.Quad({
        OnCommand = function(self)
            self:horizalign(left)
            self:vertalign(top)
            self:xy(SCREEN_LEFT, SCREEN_TOP+25)
            self:zoomto(SCREEN_WIDTH, 25)
            self:diffuse(0, 0, 0, 0.66)
        end,
    }),
    Def.Quad({
        OnCommand = function(self)
            self:horizalign(left)
            self:vertalign(bottom)
            self:xy(SCREEN_LEFT, SCREEN_BOTTOM-25)
            self:zoomto(SCREEN_WIDTH, 25)
            self:diffuse(0, 0, 0, 0.66)
        end,
    }),
    Def.HelpDisplay({
        File = 'Common Small Normal',
        InitCommand=function(self)
            self:diffuse(Color('White'))
            self:zoom(0.4);
            self:xy(SCREEN_CENTER_X, SCREEN_HEIGHT-12)
            self:SetTipsColonSeparated(THEME:GetString('ScreenEvaluation', 'Summary'..((stageCount <= 3) and 'Under3Stage' or '')..'HelperText'))
        end,
        OffCommand = function(self)
            self:sleep(0.2)
            self:linear(0.3)
            self:addy(30)
        end,
    }),
})
