local lyricList = {}
local default = W_OPTION:ReadPref('DefaultLyricType', 'Default')
local actorMax = tonumber(W_OPTION:ReadPref('LyricActorMax', '50'))

local curIndex = 1
local function Update(self)
    local ms = GAMESTATE:GetCurMusicSeconds()
    for i=curIndex, #lyricList do
        if lyricList[i].Time <= ms then
            self:GetChild('LyricFrame'):GetChild('Lyric'..((i-1)%actorMax+1)):playcommand('ShowLyric', {Index = i})
            self:playcommand('LyricOrder')
            curIndex = i+1
        else
            break
        end
    end
end

-- 歌詞エフェクト呼び出し
local LyricEffect
local function LoadLyricEffect()
    LyricEffect = dofile(THEME:GetPathO('', 'LyricEffects.lua'))
    return LyricEffect
end

local lyricType, lyricFunction, lyricArgs = '', nil, {}
local function LyricActor()
    local la = Def.ActorFrame({
        FOV = 60,
        Name = 'LyricFrame',
        LyricOrderCommand = function(self)
            self:SortByDrawOrder()
        end,
    })
    for i=1, actorMax do
        -- Attributeやrainbowを使用した時に半透明が使えるようにフレームで囲む
        la[#la+1] = Def.ActorFrame({
            Name = 'Lyric'..i,
            ShowLyricCommand = function(self, params)
                self:finishtweening()
                self:draworder(params.Index)
                self:diffusealpha(1)
            end,
            LoadFont('Common Normal')..{
                InitCommand = function(self)
                    self:diffuse(Color('White'))
                    self:diffusealpha(0)
                end,
                LoadLyricCommand = function(self)
                    self:settext('')
                end,
                ShowLyricCommand = function(self, params)
                    self:strokecolor(Color('Black'))
                    self:xy(SCREEN_CENTER_X, SCREEN_CENTER_Y+135)
                    self:vertspacing(5)
                    self:settext(lyricList[params.Index].Text)
                    self:diffuse(lyricList[params.Index].Color)
                    self:diffusealpha(1)
                    self:strokecolor(Color('Black'))
                    if not lyricFunction then
                        -- 文字列定義の場合
                        LyricEffect(lyricType, self, lyricList, params.Index, lyricArgs)
                    else
                        -- function定義の場合
                        lyricFunction(self, lyricList, params.Index, lyricArgs)
                    end
                end,
                OffCommand = function(self)
                    self:finishtweening()
                end,
            },
        })
    end
    return la
end

return Def.ActorFrame({
    OnCommand = function(self)
        if not LyricEffect then
            LoadLyricEffect()
        end
        self:SetUpdateFunction(Update)
        -- ここでLoadLyricCommandを呼ぶ必要なし
    end,
    ChangedGameplaySongMessageCommand = function(self, params)
        self:playcommand('LoadLyric')
    end,
    LoadLyricCommand = function(self)
        lyricList = {}
        curIndex = 1
        if W:Song():HasLyrics() then
            lyricType, lyricFunction, lyricArgs = YA_GROUP:LyricType(W:Song())
            if lyricType == 'Default' then
                lyricType = default
            end
            local path = W:Song():GetLyricsPath()
            if not string.find(string.lower(path), '^.+.lrc$') then
                return
            end
            local f = RageFileUtil.CreateRageFile()
            if not f:Open(path, 1) then
                f:destroy()
                return
            end
            local curColor = Color('White')
            while true do
                if f:AtEOF() then
                    break
                end
                local lyricLine = string.gsub(f:GetLine(), '^'..string.char(0xef, 0xbb, 0xbf)..'(.?)', '%1')
                if lyricLine ~= '' then
                    local head, body = string.match(lyricLine, '%[([^%]]+)%](.*)')
                    if head and head ~= '' then
                        if string.lower(head) == 'colour' or string.lower(head) == 'color' then
                            curColor = color(string.gsub(body, '0x', '#'))
                        else
                            local t1, t2 = string.match(head, '(%d+):([0-9.]+)')
                            lyricList[#lyricList+1] = {
                                Color = curColor,
                                Time  = tonumber(t2) + tonumber(t1)*60,
                                Text  = string.gsub(body, '|', '\n'),
                            }
                        end
                    end
                end
            end
            f:Close()
            f:destroy()
            table.sort(lyricList, function(a, b)
                return a.Time < b.Time
            end)
        end
    end,
    LyricActor(),
})