local child
local player = Var('Player')
local ShowComboAt = THEME:GetMetric('Combo', 'ShowComboAt')
local Pulse = THEME:GetMetric('Combo', 'PulseCommand')
local PulseLabel = THEME:GetMetric('Combo', 'PulseLabelCommand')

local NumberMinZoom = THEME:GetMetric('Combo', 'NumberMinZoom')
local NumberMaxZoom = THEME:GetMetric('Combo', 'NumberMaxZoom')
local NumberMaxZoomAt = THEME:GetMetric('Combo', 'NumberMaxZoomAt')

local LabelMinZoom = THEME:GetMetric('Combo', 'LabelMinZoom')
local LabelMaxZoom = THEME:GetMetric('Combo', 'LabelMaxZoom')

local continueMinJudgList = {
    TapNoteScore_W1 = 0,
    TapNoteScore_W2 = 1,
    TapNoteScore_W3 = 2,
    TapNoteScore_W4 = 3,
}
local continueMinJudg = nil
local judgmentColor = {
    TapNoteScore_W1 = W_JUDGMENT:Color('TapNoteScore_W1'),
    TapNoteScore_W2 = W_JUDGMENT:Color('TapNoteScore_W2'),
    TapNoteScore_W3 = W_JUDGMENT:Color('TapNoteScore_W3'),
    TapNoteScore_W4 = W_JUDGMENT:Color('TapNoteScore_W4'),
}
-- 最低継続コンボより低い判定はコンボを継続させない
local minComboStr = string.gsub(W_OPTION:ReadPref('ContinueCombo', 'TapNoteScore_W4'), 'TapNoteScore_W', '')
local minCombo = tonumber(minComboStr)
local recountColor = W_OPTION:ReadPref('ComboColor', 'DDR X') == 'DDR X'

for k,v in pairs(continueMinJudgList) do
    if v >= minCombo then
        continueMinJudgList[k] = nil
        judgmentColor[k] = Color('White')
    end
end

local function IsComboContinueTns(tns)
    if not tns then
        return true
    end
    for k,v in pairs(continueMinJudgList) do
        if tns == k then
            return false
        end
    end
    return true
end

local comboParams = {}

local t = Def.ActorFrame ({
    LoadFont( 'Combo', 'numbers' ) .. {
        Name = 'Number',
        OnCommand = THEME:GetMetric('Combo', 'NumberOnCommand'),
    },
    LoadFont('Common Normal') .. {
        Name = 'Label',
        OnCommand = THEME:GetMetric('Combo', 'LabelOnCommand'),
    },
    
    InitCommand = function(self)
        self:vertalign(bottom)
        child = self:GetChildren()
        child.Number:visible(false)
        child.Label:visible(false)
        child.Label:strokecolor(0, 0, 0, 0.5)
    end,
    --[[
    -- 25コンボごと
    TwentyFiveMilestoneCommand=function(self,parent)
        self:skewy(-0.125)
        self:decelerate(0.325)
        self:skewy(0)
    end,
    -- とーすてぃ時
    ToastyAchievedMessageCommand=function(self,params)
        if params.PlayerNumber == player then
            self:thump(2)
            self:effectclock('beat')
        end;
    end,
    --]]
    -- 判定時
    JudgmentMessageCommand = function(self, params)
        -- 異なるプレイヤー
        if params.Player ~= player then return end
        -- コンボ継続監視対象外の判定は無視
        if not IsComboContinueTns(params.TapNoteScore) then
            local judg = continueMinJudgList[params.TapNoteScore]
            continueMinJudg = (not continueMinJudg or continueMinJudg < judg) and judg or continueMinJudg
        end
        if W:IsOF() then
            -- ロングノートの終点でコンボアニメを発生させない
            if params.HoldNoteScore and params.HoldNoteScore ~= 'HoldNoteScore_None' then
                return
            end
            self:playcommand('ComboUpdate', comboParams)
        end
    end,
    -- OutFoxはCombo→Judgの順でメッセージが飛んでいるので苦虫を噛んだような顔をしながら無理やり対応
    ComboCommand = function(self, params)
        if W:IsOF()then
            comboParams = params
        end
        -- Roll連打でコンボを有効にしたときにOFでコンボ加算の反応がないので
        -- もう一度苦虫を噛みつぶした顔をしながらOFでは2回呼び出す方法で対応
        self:playcommand('ComboUpdate', params)
    end,
    ComboUpdateCommand = function(self, params)
        -- MISSコンボは非対応
        -- local iCombo = params.Misses or params.Combo
        local iCombo = params.Combo
        if not iCombo or iCombo < ShowComboAt then
            continueMinJudg = (iCombo and (not params.Misses)) and continueMinJudg or nil
            child.Number:visible(false)
            child.Label:visible(false)
            return
        end
        child.Number:finishtweening()
        child.Label:finishtweening()

        child.Label:settext('COMBO!!')
        child.Label:visible(false)

        params.Zoom = scale( iCombo, 0, NumberMaxZoomAt, NumberMinZoom, NumberMaxZoom )
        params.Zoom = clamp( params.Zoom, NumberMinZoom, NumberMaxZoom )
        params.Zoom = params.Zoom * ((iCombo > 999) and 0.8 or 1)
        
        params.LabelZoom = scale( iCombo, 0, NumberMaxZoomAt, LabelMinZoom, LabelMaxZoom )
        params.LabelZoom = clamp( params.LabelZoom, LabelMinZoom, LabelMaxZoom )
        
        child.Number:visible(true)
        child.Label:visible(true)
        child.Number:settext( string.format('%i', (iCombo > 9999) and 9999 or iCombo) )
        child.Number:y((iCombo > 999) and 28 or 25)

        -- FullCombo Rewards
        if params.FullComboW1 or (recountColor and continueMinJudg == continueMinJudgList.TapNoteScore_W1) then
            child.Number:diffuse(judgmentColor.TapNoteScore_W1)
            child.Number:glowshift()
        elseif params.FullComboW2 or (recountColor and continueMinJudg == continueMinJudgList.TapNoteScore_W2) then
            child.Number:diffuse(judgmentColor.TapNoteScore_W2)
            child.Number:glowshift()
        elseif params.FullComboW3 or (recountColor and continueMinJudg == continueMinJudgList.TapNoteScore_W3) then
            child.Number:diffuse(judgmentColor.TapNoteScore_W3)
            child.Number:stopeffect()
        elseif params.FullComboW4 or (recountColor and continueMinJudg == continueMinJudgList.TapNoteScore_W4) then
            child.Number:diffuse(judgmentColor.TapNoteScore_W4)
            child.Number:stopeffect()
        else
            child.Number:diffuse(Color('White'))
            child.Number:stopeffect()
        end
        if params.FullComboW1 then
            child.Label:diffuse(judgmentColor.TapNoteScore_W1)
        elseif params.FullComboW2 then
            child.Label:diffuse(judgmentColor.TapNoteScore_W2)
        elseif params.FullComboW3 then
            child.Label:diffuse(judgmentColor.TapNoteScore_W3)
        elseif params.FullComboW4 then
            child.Label:diffuse(judgmentColor.TapNoteScore_W4)
        else
            child.Label:diffuse(Color('White'))
            child.Label:settext('COMBO')
        end
        -- Pulse
        Pulse(child.Number, params)
        PulseLabel(child.Label, params)
    end,
})

return t
