local pn = ...
local width = 120
local height = 28
local mpn = GAMESTATE:GetMasterPlayerNumber()

local function DifficultyActor(dif)
    return Def.ActorFrame({
        SetDifficulyListCommand = function(self, params)
            self:diffusealpha(params[dif] and 1 or 0.5)
        end,
        -- 選択
        Def.Quad({
            OnCommand = function(self)
                self:zoomto(width + 20, height-1)
                self:blend('BlendMode_Add')
                self:diffuse(W_PLAYER:Color(pn))
                if pn == PLAYER_1 then
                    self:faderight(1)
                else
                    self:fadeleft(1)
                end
            end,
            SetDifficulyListCommand = function(self, params)
                self:visible(params.Trail:GetDifficulty() == dif)
            end,
        }),
        -- 背景
        Def.Quad({
            OnCommand = function(self)
                self:x(10)
                self:zoomto(width, height-3)
            end,
            SetDifficulyListCommand = function(self, params)
                self:diffuse(BoostColor(W_DIF:Color(dif),
                    (params.Trail:GetDifficulty() == dif) and 0.33 or 0.25))
            end,
        }),
        Def.Quad({
            OnCommand = function(self)
                self:x(10)
                self:zoomto(width, height-3)
                self:diffuse(0, 0, 0, 0.5)
                self:fadeleft(1)
            end,
        }),
        -- 難易度名
        LoadFont('Common Small Bold')..{
            Text = W_DIF:Course(dif),
            OnCommand = function(self)
                self:horizalign(left)
                self:xy(-width/2+15, -7)
                self:zoom(0.33)
            end,
            SetDifficulyListCommand = function(self, params)
                self:diffuse(W_DIF:Color(dif))
                self:diffusealpha((params.Trail:GetDifficulty() == dif)
                     and 1.0 or 0.5)
            end,
        },
        -- 難易度
        LoadFont('Common Small Bold')..{
            OnCommand = function(self)
                self:horizalign(left)
                self:xy(-width/2+15, 4)
                self:zoom(0.5)
            end,
            SetDifficulyListCommand = function(self, params)
                self:diffuse(1.0, 1.0, 1.0, 1.0)
                self:diffusealpha((params.Trail:GetDifficulty() == dif)
                     and 1.0 or 0.5)
                if params[dif] then
                    self:settext(((params[dif].Min > 0) and string.format('%2d', params[dif].Min) or ' ?')
                        ..' - '..((params[dif].Max > 0) and string.format('%d', params[dif].Max) or '?'))
                else
                    self:settext('')
                end
            end,
        },
        -- グレード
        LoadActor('../grade')..{
            OnCommand = function(self)
                self:xy(width/2-20, 1)
                self:zoom(1.33)
                self:animate(false)
            end,
            SetDifficulyListCommand = function(self, params)
                self:setstate(0)
                if not params[dif] then
                    return
                end
                local profile = PROFILEMAN:GetProfile(pn)
                local hList = profile:GetHighScoreListIfExists(params.Course, params[dif].Trail)
                if hList then
                    local scores = hList:GetHighScores()
                    if scores and scores[1] then
                        local grade = Enum.Reverse(Grade)[scores[1]:GetGrade()]
                        if grade < 20 then
                            self:setstate(math.min(grade+1, 7))
                        else
                            self:setstate(8)
                        end
                    end
                end
            end,
        },
        -- フルコン
        LoadActor('../fullcombo')..{
            OnCommand = function(self)
                self:xy(width/2-2, 1+4)
                self:animate(false)
            end,
            SetDifficulyListCommand = function(self, params)
                self:visible(false)
                if not params[dif] then
                    return
                end
                local fc = 0
                local profile = PROFILEMAN:GetProfile(pn)
                local hList = profile:GetHighScoreListIfExists(params.Course, params[dif].Trail)
                if hList then
                    local scores = hList:GetHighScores()
                    for k,v in pairs(scores) do
                        local scoreFC = W_SCORE:ScoreFC(v, params[dif].Trail:GetRadarValues(pn))
                        if scoreFC > 0 then
                            if fc == 0 or scoreFC < fc then
                                fc = scoreFC
                            end
                        end
                    end
                end
                if fc > 0 then
                    self:visible(true)
                    if fc == 1 then
                        self:setstate(1)
                        self:diffuse(GameColor.Judgment["JudgmentLine_W1"])
                        self:glowblink()
                        self:effectperiod(0.20)
                        self:zoom(1)
                    elseif fc == 2 then
                        self:setstate(1)
                        self:diffuse(GameColor.Judgment["JudgmentLine_W2"])
                        self:glowshift()
                        self:zoom(0.85)
                    elseif fc == 3 then
                        self:setstate(1)
                        self:diffuse(GameColor.Judgment["JudgmentLine_W3"])
                        self:stopeffect()
                        self:zoom(0.7)
                    elseif fc == 4 then
                        self:setstate(2)
                        self:diffuse(BoostColor(GameColor.Judgment["JudgmentLine_W4"],1.5))
                        self:stopeffect()
                        self:zoom(1)
                    end
                end
            end,
        },
    })
end

return Def.ActorFrame({
    OnCommand = function(self)
        if not W_PLAYER:IsEnabled(pn) then
            self:visible(false)
            return
        end
        self:playcommand('CurrentTrailP1Changed')
    end,
    PlayerJoinedMessageCommand = function(self, params)
        if pn == params.Player then
            self:visible(true)
            self:playcommand('CurrentTrailP1Changed')
        end
    end,
    CurrentTrailP1ChangedMessageCommand = function(self)
        if not W_PLAYER:IsEnabled(pn) then return end
        local trail = GAMESTATE:GetCurrentTrail(mpn)
        local course = GAMESTATE:GetCurrentCourse() or nil
        local dList = {
            Difficulty_Easy   = W_DIF:TrailMinMax(trail, 'Difficulty_Easy'),
            Difficulty_Medium = W_DIF:TrailMinMax(trail, 'Difficulty_Medium'),
            Difficulty_Hard   = W_DIF:TrailMinMax(trail, 'Difficulty_Hard'),
        }
        self:playcommand('SetDifficulyList', {
            Course = course,
            Trail  = trail,
            Difficulty_Easy   = dList.Difficulty_Easy,
            Difficulty_Medium = dList.Difficulty_Medium,
            Difficulty_Hard   = dList.Difficulty_Hard,
        })
    end,
    CurrentTrailP2ChangedMessageCommand = function(self)
        self:playcommand('CurrentTrailP1Changed')
    end,
    LoadActor(W:Object('boxshadow'), 'Score'..(W:IsP1(pn) and 1 or 2), {
        Color  = color('#00000064'),
        Center = BoostColor(W_PLAYER:Color(pn), 0.5),
        Length = 20,
    })..{
        OnCommand = function(self)
            self:y((height*3+24)/2 - height/2)
            self:playcommand('UpdateBoxShadow', {
                Id     = 'Score'..(W:PNum(pn)),
                Width  = width+20+1.5,
                Height = height*3+23+2,
            })
        end,
    },
    Def.Quad({
        OnCommand = function(self)
            self:y((height*3+24)/2 - height/2)
            self:zoomto(width+20, height*3+24)
            self:diffuse(color('#2A2A2AE0'))
        end,
    }),
    DifficultyActor('Difficulty_Easy')..{
        OnCommand = function(self)
            self:y(height*0)
        end,
    },
    DifficultyActor('Difficulty_Medium')..{
        OnCommand = function(self)
            self:y(height*1)
        end,
    },
    DifficultyActor('Difficulty_Hard')..{
        OnCommand = function(self)
            self:y(height*2)
        end,
    },
    -- スコア
    Def.ActorFrame({
        OnCommand = function(self)
            self:y(height*3 - height/2)
        end,
        -- 背景
        Def.Quad({
            OnCommand = function(self)
                self:vertalign(top)
                self:zoomto(width+20, 24)
                self:diffuse(0, 0, 0, 1)
            end,
        }),
        LoadFont('Common Small Bold')..{
            Text = 'HIGHSCORE',
            OnCommand = function(self)
                self:horizalign(left)
                self:vertalign(top)
                self:zoom(0.3)
                self:xy(-width/2 - 5, 5)
                self:diffuse(1, 1, 1, 0.8)
            end,
        },
        -- スコア
        LoadFont('Common Small Bold')..{
            Text = '0,000,000',
            OnCommand = function(self)
                self:horizalign(right)
                self:vertalign(top)
                self:zoom(0.35)
                self:xy(-5, 15)
                self:maxwidth((width/2)/0.35)
                self:diffuse(1, 1, 1, 1)
            end,
            SetDifficulyListCommand = function(self, params)
                local score = 0
                local profile = PROFILEMAN:GetProfile(pn)
                local hList = profile:GetHighScoreListIfExists(params.Course, params.Trail)
                if hList then
                    local scores = hList:GetHighScores()
                    for k,v in pairs(scores) do
                        local sc = v:GetScore()
                        if sc > score then
                            score = sc
                        end
                    end
                end
                self:settext(W_STRING:Number(score))
            end,
        },
        -- DP
        LoadFont('waiei2 Score Numbers')..{
            Text = '88.88%',
            OnCommand = function(self)
                self:horizalign(right)
                self:zoom(0.8)
                self:xy(width/2 + 5, 14)
                self:diffuse(1, 1, 1, 1)
            end,
            SetDifficulyListCommand = function(self, params)
                local dp = 0
                local profile = PROFILEMAN:GetProfile(pn)
                local hList = profile:GetHighScoreListIfExists(params.Course, params.Trail)
                if hList then
                    local scores = hList:GetHighScores()
                    for k,v in pairs(scores) do
                        local point = v:GetPercentDP()
                        if point > dp then
                            dp = point
                        end
                    end
                end
                if dp >= 1 then
                    self:settextf('%d%%', math.floor(dp*100+0.9))
                else
                    self:settextf('%5.2f%%', dp*100)
                end
            end,
        },
    }),
})
