-- functions
local labelParams = {
  HasSplitBpm = false,
  HasTrueBpm = false,
  ShowTrueBpm = false,
  IsRandom = false,
}

-- BPMアニメーション用のカウンター制御
local fps = 1.0/50
local dtSum = 0
local AnimeDtSum = 0 -- 0になるとBPMアニメーションのリセット
local bpmAnimeCnt = 0
local latestBpmAnimeCnt = 0
local function update(self, dt)
  -- 0.02秒に1回判定
  dtSum = dtSum + dt
  if dtSum >= fps then
    dtSum = dtSum - fps
    AnimeDtSum = AnimeDtSum + fps
    bpmAnimeCnt = math.floor(AnimeDtSum*100) % 400  -- 0.01秒単位、0～399でループ
    if latestBpmAnimeCnt ~= bpmAnimeCnt then
      self:GetParent():playcommand('UpdateBpmNumber', {Count = bpmAnimeCnt})
      latestBpmAnimeCnt = bpmAnimeCnt
    end
  end
end

-- SplitBPM楽曲かどうかを判定
local function HasSplitBpm(song)
  local stepType = GAMESTATE:GetCurrentStyle():GetStepsType()
  local steps = song:GetStepsByStepsType(stepType)
  for s=1,#steps do
      -- EDIT譜面は判定対象外
      if steps[s]:GetDifficulty() ~= 'Difficulty_Edit'
          and song:IsStepsUsingDifferentTiming(steps[s]) then
          return true
      end
  end
  return false
end

-- 実際のBPM情報を取得する
local function GetTrueBpm(song)
  local ret = {
    IsRandom = false,
    HasTrueBpm = false,
    SongBpm = {0, 0},
    PlayerBpm = {
      [PLAYER_1] = {0, 0},
      [PLAYER_2] = {0, 0},
    },
  }
  -- 楽曲とプレイヤーの最小BPMと最大BPMを求める
  local function GetBpmRange(steps)
    local min = 99999999
    local max = 0
    if steps then
    local timing = steps:GetTimingData()
      for _,v in pairs(timing:GetActualBPM()) do
        if min > v then min = v end
        if max < v then max = v end
      end
    end
    return {math.round(min), math.round(max)}
  end
  -- BPMがランダムかどうか
  ret.IsRandom = (song:IsDisplayBpmRandom() or song:IsDisplayBpmSecret())
  -- DISAPLY BPMを四捨五入して最小~最大BPMを求める
  ret.SongBpm = (function(song)
    local v = song:GetDisplayBpms()
    return {math.round(v[1]), math.round(v[2])}
  end)(song)
  -- 各プレイヤーの最小~最大BPMを求めてDISPLAY BPMと異なるならHasTrueBpm = true
  for _,pn in pairs(PlayerNumber) do
    if W_PLAYER:IsEnabled(pn) then
      ret.PlayerBpm[pn] = GetBpmRange(GAMESTATE:GetCurrentSteps(pn))
      if ret.PlayerBpm[pn][1] ~= ret.SongBpm[1] or ret.PlayerBpm[pn][2] ~= ret.SongBpm[2] then
        ret.HasTrueBpm = true
      end
    end
  end
  return ret
end

local currentSong = nil
return Def.ActorFrame({
  CodeCommand = function(self, params)
    if params.Name == 'SwitchUp' then
      labelParams.ShowTrueBpm = not labelParams.ShowTrueBpm
      self:GetParent():playcommand('ChangedShowTrueBpm', labelParams)
    end
    if params.Name == 'SwitchDown' then
      labelParams.ShowTrueBpm = not labelParams.ShowTrueBpm
      self:GetParent():playcommand('ChangedShowTrueBpm', labelParams)
    end
  end,
  OnCommand = function(self)
    self:SetUpdateFunction(update)
    self:playcommand('UpdateStepsMessage', {Song = W:Song()})
  end,
  PlayerJoinedMessageCommand = function(self)
    self:playcommand('UpdateStepsMessage', {Song = nil})
  end,
  -- 曲を切り替えた時点でTRUE BPMの表示はFALSEに戻し、再判定する
  UpdateStepsMessageCommand = function(self, params)
    if params.Song then
      -- 曲が変わった判定（譜面だけ変えた場合は処理しない）
      if currentSong ~= params.Song then
        currentSong = params.Song
        labelParams.ShowTrueBpm = false
      end
      local trueBpm = GetTrueBpm(params.Song)
      self:GetParent():playcommand('UpdateTrueBpms', trueBpm)

      labelParams.HasSplitBpm = HasSplitBpm(params.Song)
      labelParams.HasTrueBpm = trueBpm.HasTrueBpm
      labelParams.IsRandom = trueBpm.IsRandom
    else
      labelParams.ShowTrueBpm = false
      labelParams.HasSplitBpm = false
      labelParams.HasTrueBpm = false
      labelParams.IsRandom = false
    end
    self:GetParent():playcommand('UpdateBpmLabels', labelParams)
  end,
  
})