local text, col = ...

local BPMTYPE = {
  RANDOM = 'Random',
  CHANGE = 'Change',
  FIXED  = 'Fixed',
}
local curBpmType = BPMTYPE.FIXED
local curBpms = {0, 0}
return Def.ActorFrame({
  -- Label
  LoadFont('Common Small Normal')..{
    Text = text,
    InitCommand = function(self)
      self:diffuse(col)
      self:horizalign(left)
      self:y(-2)
      self:zoom(0.35)
    end,
    UpdateBpmLabelsCommand = function(self, params)
      self:playcommand('ChangedShowTrueBpm', params)
    end,
    ChangedShowTrueBpmCommand = function(self, params)
      if text == 'BPM' then
        self:visible(not params.ShowTrueBpm)
      else
        self:visible(params.ShowTrueBpm)
      end
    end,
  },
  -- 数字
  LoadFont("Common Normal")..{
    InitCommand = function(self)
        self:settextf('%03.0f', 0)
        self:diffuse(Color('White'))
        self:xy(60, 0)
        self:zoom(0.7)
        self:maxwidth(80 / 0.7)
    end,
    UpdateBpmLabelsCommand = function(self, params)
      self:playcommand('ChangedShowTrueBpm', params)
    end,
    ChangedShowTrueBpmCommand = function(self, params)
      if not W:Song() then
        curBpmType = BPMTYPE.FIXED
        curBpms = {0, 0}
        self:diffuse(color('1, 1, 1, 0.5'))
        self:settextf('%03.0f', 0)
      end
      if text == 'BPM' then
        self:visible(not params.ShowTrueBpm)
      else
        self:visible(params.ShowTrueBpm)
      end
    end,
    UpdateTrueBpmsCommand = function(self, params)
      -- 楽曲
      curBpms = (text == 'BPM')
              and params.SongBpm
              or ((text == 'Player1') and params.PlayerBpm[PLAYER_1] or params.PlayerBpm[PLAYER_2])
      self:settextf('%03.0f', curBpms[1])
      -- ランダム
      if text == 'BPM' and params.IsRandom then
        curBpmType = BPMTYPE.RANDOM
        self:diffuse(color('1, 0.5, 0.4, 1'))
        self:settextf('%03.0f', 0 )
      -- 可変
      elseif curBpms[1] ~= curBpms[2] then
        curBpmType = BPMTYPE.CHANGE
        self:diffuse(color('0.6, 0.95, 1.0, 1'))
      -- 固定
      else
        curBpmType = BPMTYPE.FIXED
        self:diffuse(Color('White'))
      end
    end,
    UpdateBpmNumberCommand = function(self, params)
      if text == 'BPM' and curBpmType == BPMTYPE.RANDOM then
        local num = (params.Count/5)%10
        self:settextf('%d%d%d', num, num, num)
      elseif curBpmType == BPMTYPE.CHANGE then
        local bpms = {curBpms[1], (curBpms[2] > 99999) and 100000 or curBpms[2]}
        local diff = bpms[2] - bpms[1]
        -- 最大値→最小値
        if 100 <= params.Count and params.Count < 200 then
          self:settextf('%03.0f', bpms[2] - diff * (params.Count-100) / 100)
        -- 最小値
        elseif 200 <= params.Count and params.Count < 300 then
          self:settextf('%03.0f', bpms[1])
        -- 最小値→最大値
        elseif 300 <= params.Count and params.Count < 400 then
          self:settextf('%03.0f', bpms[1] + diff * (params.Count-300) / 100)
        -- 最大値
        else
          if bpms[2] > 99999 then
            self:settext('99999+')
          else
            self:settextf('%03.0f', bpms[2])
          end
        end
      else
        if curBpms[1] > 99999 then
          self:settext('99999+')
        else
          self:settextf('%03.0f', curBpms[1])
        end
      end
    end,
  },
})