local id = nil

local w=192
local h=170 + 8
local rSize = 16
local margin = 32

local bnRatio  = 256/80
local bnWidth  = w - 2
local bnHeight = bnWidth * (1.0/bnRatio)
local Banner = LoadActor(W:Object('banner'), {
    Song  = nil,
    Size  = bnWidth-2,
    Ratio = bnRatio,
    Cache = -1,
    Rate  = 0.333333,
})

local activeColor  = color('0.4, 0.55, 0.75, 1')
local Radius = LoadActor(W:Object('radius'), {
    Width  = w,
    Height = h,
    Radius = rSize,
    Color  = activeColor,
    Cell   = {1, 1, 0, 1, 1, 0, 0, 0, 0},
})

local function BannerActor()
    -- バナー
    return Def.ActorFrame({
        Banner:Actor()..{
            UpdateWheelChildCommand = function(self, params)
                self:playcommand('UpdateBanner', {
                    Path = params.Text
                            and SONGMAN:GetSongGroupBannerPath(params.Text)
                            or THEME:GetPathG('common fallback', 'banner'),
                })
            end,
        },
    })
end

local function WheelActor()
    return Def.ActorFrame({
        -- 色付きの背景
        Radius:Actor()..{
            UpdateWheelChildCommand = function(self, params)
                local order = GAMESTATE:GetSortOrder() or ''
                local text = params.Text
                local groupColor = (ToEnumShortString(order) == 'Preferred') and YA_GROUP:GroupColor(text) or nil
                self:playcommand('UpdateRadius', {
                    Id    = id,
                    Color = groupColor and groupColor or activeColor,
                })
            end,
        },
        Def.Quad({
            InitCommand = function(self)
                self:zoomto(w - margin, h - rSize)
                self:horizalign(left)
                self:vertalign(top)
                self:xy(-w/2+margin, -h/2)
                self:diffuse(0.2, 0.2, 0.25, 1)
            end,
        }),
        Def.Quad({
            InitCommand = function(self)
                self:zoomto(bnWidth+0.2, bnHeight+2)
                self:vertalign(bottom)
                self:xy(0, h/2)
                self:diffuse(color('#666666ff'))
            end,
        }),
        Def.ActorFrame({
            OnCommand = function(self)
                self:xy(0, h/2 - bnHeight/2-1)
            end,
            -- バナー部分
            Def.ActorFrame({
                BannerActor(),
            }),
        }),
        -- 開いているとき
        Def.Quad({
            InitCommand = function(self)
                self:zoomto(20, h)
                self:horizalign(left)
                self:vertalign(bottom)
                self:xy(w/2, h/2)
                self:diffuse(0.2, 0.2, 0.25, 1)
                self:faderight(0.6)
            end,
        }),
        Def.Quad({
            InitCommand = function(self)
                self:diffuse(color('#666666ff'))
                self:xy(w/2 -1, 0)
                self:zoomto(1.2, h)
            end,
        }),
        -- タイトル
        LoadFont('Common Small Normal')..{
            UpdateWheelChildCommand = function(self, params)
                local order = GAMESTATE:GetSortOrder() or ''
                local text = params.Text
                if text then
                    -- カスタムソート時のみgroup.iniのNAMEをチェック
                    if ToEnumShortString(order) == 'Preferred' then
                        text = YA_GROUP:GroupName(text)
                    end
                    self:diffuse(Color('White'))
                    self:diffusealpha(params.HasFocus and 1 or 0.8)
                    self:settext(text)
                    self:horizalign(left)
                    self:zoom(0.85)
                    self:wrapwidthpixels((w-margin-20)/0.85)
                    self:maxwidth((w-margin-20)/0.85)
                    self:x(-w/2 + margin + 10)
                    self:y(-h/2 + (h-26-bnHeight)/2 + 26)
                end
            end
        },
        Def.Quad({
            InitCommand = function(self)
                self:zoomto(w-margin, 1.2)    -- 1だと720pで表示されなくなることがある
                self:horizalign(left)
                self:vertalign(top)
                self:xy(-w/2 + margin, -h/2 + 30)
                self:diffuse(color('#666666ff'))
            end,
        }),
        -- 曲数
        Def.ActorFrame({
            InitCommand = function(self)
                self:xy(w/2-10, -h/2 + 15)
            end,
            -- Song/Songs表記
            LoadFont('Common Small Normal')..{
                InitCommand = function(self)
                    self:horizalign(right)
                    self:vertalign(middle)
                    self:zoom(0.5)
                    self:diffuse(color('#ffffffff'))
                    self:settext(GAMESTATE:IsCourseMode() and 'Courses' or 'Songs')
                end,
                UpdateWheelChildCommand = function(self, params)
                    self:diffusealpha(params.HasFocus and 1 or 0.8)
                end,
            },
            -- 曲数はmetrictで定義（MusicWheelItem, SectionCount）
        }),
    })
end
return Def.ActorFrame({
    SetCommand = function(self, params)
        -- -- 多重呼び出しを回避
        if not params.Index then return end
        -- 多重呼び出しを回避
        if not self.Params then
            self.Params = {Id = math.random(), Index = nil, HasFocus = nil}
            Radius:Id(id)
        end
        -- グループ切り替え時に表示が変わるように設定
        if params.HasFocus and params.Type == 'SectionExpanded' then
            self.Params.Index = nil
        end
        if self.Params and params and params.Index and params.HasFocus ~= nil then
            if params.Index ~= self.Params.Index or params.HasFocus ~= self.Params.HasFocus then
                params.Id = self.Params.Id
                self.Params.Index = params.Index
                self.Params.HasFocus = params.HasFocus
                self:finishtweening()
                self:playcommand('UpdateWheel', params)
            end
        end
    end,
    UpdateWheelCommand = function(self, params)
        if self.Params.Id ~= params.Id then return end
        self:playcommand('UpdateWheelChild', params)
    end,
    WheelActor(),
})