local w=192
local h=170

-- IDに一意な値を設定する手段がないので
-- Graphics/Objectsではなく、Wheel用に用意したオブジェクトを使用する
local Banner = LoadActor(W:Object('banner'), {
    Song  = nil,
    Size  = w,
    Ratio = 256/80,
    Cache = -1,
    Rate  = 0.333333,
})

local TopBorder = LoadActor(W:Object('radius'), {
    Width  = w,
    Height = 16,
    Radius = 8,
    Color  = {1, 1, 1, 1},
    Cell   = {1, 1, 1, 0, 0, 0, 0, 0, 0},
})

local MusicLength = LoadActor(W:Object('radius'), {
    Width  = 80,
    Height = 16,
    Radius = 8,
    Color  = {1, 1, 1, 1},
})

-- 背景
local function BgActor()
    return Def.ActorFrame({
        -- 影
        LoadActor(W:Object('boxshadow'), 'wheel', {
            Length = 10,
            Color  = {0.1, 0.1, 0.1, 0.4},
            Center = {0.1, 0.1, 0.1, 0.4},
        })..{
            OnCommand = function(self)
                self:y(6)
                self:playcommand('UpdateBoxShadow', {
                    Id         = 'wheel',
                    Width      = w,
                    Height     = h-8,
                    Wait       = 0.5,
                    AnimeAlpha = true,
                })
            end,
        },
        -- 上部バー
        TopBorder:Actor()..{
            OnCommand = function(self)
                self:y(-h/2 + 8)
            end,
            UpdateWheelChildCommand = function(self, params)
                self:playcommand('UpdateRadius', {
                    Color = {1, 1, 1, 1},
                })
                self:rainbow()
            end,
        },
        -- 背景
        Def.Quad({
            InitCommand = function(self)
                self:zoomto(w, h - 7)
                self:vertalign(top)
                self:y(-h/2 + 7)
                self:diffuse(0.25, 0.25, 0.3, 1)
            end,
        }),
        Def.Quad({
            InitCommand = function(self)
                self:y(-h/2 + 140)
                self:zoomto(w, 1)
                self:diffuse(color('#666666ff'))
            end,
        }),
    })
end

-- バナー
local function BannerActor()
    return Def.ActorFrame({
        InitCommand = function(self)
            self:xy(0, -h/2+38)
        end,
        -- バナー
        Def.Sprite({
            OnCommand = function(self)
                self:LoadBackground(THEME:GetPathG('Common', 'fallback banner'))
                self:scaletofit(-w/2, -h/2, w/2, h/2)
            end,
        }),
    })
end

-- 曲名
local function SongTitleActor()
    return Def.ActorFrame({
        InitCommand = function(self)
            self:xy(0, -h/2 + 85)
        end,
        -- タイトル
        LoadFont('Common Small Normal')..{
            Text = 'Random',
            InitCommand=function(self)
                self:diffuse(color('#d8d8d8ff'))
                self:horizalign(left)
                self:vertalign(top)
                self:xy(-w/2+10, 0)
            end,
        },
    })
end

-- 曲の長さ
local function SongTimeActor()
    return Def.ActorFrame({
        InitCommand = function(self)
            self:xy(0, -h/2 + 115)
        end,
        -- 長さ
        LoadFont('Common Small Normal')..{
            Text = '?:??',
            InitCommand=function(self)
                local textZoom = 0.85
                self:diffuse(color('#a0a0a0ff'))
                self:horizalign(left)
                self:vertalign(top)
                self:zoom(textZoom)
                self:maxwidth((w/2-20)/textZoom)
                self:xy(-w/2+10, 0)
            end,
        },
    })
end

-- ジャンル
local function GenreActor()
    return Def.ActorFrame({
        LoadFont('Common Small Normal')..{
            InitCommand = function(self)
                self:horizalign(right)
                self:xy(w/2 - 10, -h/2 + 155)
                self:zoom(0.5)
                self:maxwidth((w-20)/0.5)
                self:diffuse(color('#999999ff'))
                self:settext(string.upper(ProductFamily()))
            end,
        },
    })
end

return Def.ActorFrame({
    SetCommand=function(self, params)
        -- 多重呼び出しを回避
        if not self.Params then
            self.Params = {Id = math.random(), Song = nil, HasFocus = nil}
            TopBorder:Id('B'..self.Params.Id)
            MusicLength:Id('ML'..self.Params.Id)
        end
        if self.Params and params and params.Song and params.HasFocus ~= nil then
            if params.Song ~= self.Params.Song or params.HasFocus ~= self.Params.HasFocus then
                params.Id = self.Params.Id
                self.Params.Song = params.Song
                self.Params.HasFocus = params.HasFocus
                self:finishtweening()
                self:playcommand('UpdateWheel', params)
            end
        end
    end,
    UpdateWheelCommand = function(self, params)
        if self.Params.Id ~= params.Id then return end
        self:playcommand('UpdateWheelChild', params)
    end,
    -- 背景
    BgActor(),
    -- -- バナー
    BannerActor(),
    -- 曲名
    SongTitleActor(),
    -- 長さ
    SongTimeActor(),
    -- ジャンル
    GenreActor(),
})
