local w=192
local h=170
local wheelMode = W_OPTION:ReadPref('JacketWheel', false) and 'Jacket' or 'Banner'

-- IDに一意な値を設定する手段がないので
-- Graphics/Objectsではなく、Wheel用に用意したオブジェクトを使用する
local Banner = LoadActor(W:Object('banner'), {
    Song  = nil,
    Size  = w,
    Ratio = (wheelMode == 'Jacket') and 192/162 or 256/80,
    Cache = -1,
    Rate  = 0.333333,
    Mode  = wheelMode,
})

local TopBorder = LoadActor(W:Object('radius'), {
    Width  = w,
    Height = 16,
    Radius = 8,
    Color  = {1, 1, 1, 1},
    Cell   = {1, 1, 1, 0, 0, 0, 0, 0, 0},
})

local MusicLength = LoadActor(W:Object('radius'), {
    Width  = 80,
    Height = 16,
    Radius = 8,
    Color  = {1, 1, 1, 1},
})

-- 背景
local function BgActor()
    return Def.ActorFrame({
        -- 影
        LoadActor(W:Object('boxshadow'), 'wheel', {
            Length = 10,
            Color  = {0.1, 0.1, 0.1, 0.4},
            Center = {0.1, 0.1, 0.1, 0.4},
        })..{
            OnCommand = function(self)
                self:y(6)
                self:playcommand('UpdateBoxShadow', {
                    Id         = 'wheel',
                    Width      = w,
                    Height     = h-8,
                    Wait       = 0.5,
                    AnimeAlpha = true,
                })
            end,
        },
        -- 上部バー
        TopBorder:Actor()..{
            OnCommand = function(self)
                self:y(-h/2 + 8)
            end,
            UpdateWheelChildCommand = function(self, params)
                local song = params.Song
                local col = {1, 1, 1, 0.8}
                if song then
                    col = YA_GROUP:MenuColor(song)
                else
                    col = {1, 1, 1, 0.8}
                end
                col[4] = params.HasFocus and 1 or 0.75
                self:playcommand('UpdateRadius', {
                    Color = col,
                })
            end,
        },
        -- 背景
        Def.Quad({
            InitCommand = function(self)
                self:zoomto(w, h - 7)
                self:vertalign(top)
                self:y(-h/2 + 7)
                self:diffuse(0.2, 0.2, 0.25, 0.75)
            end,
        }),
        -- 背景
        Def.Quad({
            InitCommand = function(self)
                self:zoomto(w, h - 7)
                self:vertalign(top)
                self:y(-h/2 + 7)
                self:diffuse(0.1, 0.1, 0.1, 0.5)
            end,
            UpdateWheelChildCommand = function(self, params)
                local song = params.Song
                local col = {1, 1, 1, 0}
                if song then
                    col = YA_GROUP:MenuColor(song)
                else
                    col = {1, 1, 1, 0}
                end
                col[4] = 0.5
                self:diffuse(col)
                self:diffusebottomedge(0, 0, 0, 0)
            end,
        }),
        Def.Quad({
            InitCommand = function(self)
                self:y(-h/2 + 140)
                self:zoomto(w, 1)
                self:diffuse(color('#666666ff'))
            end,
        }),
    })
end

local function OverActor()
    if wheelMode ~= 'Jacket' then
        return Def.Actor({})
    end
    return Def.Quad({
        InitCommand = function(self)
            self:vertalign(bottom)
            self:y(h/2)
            self:zoomto(w, h/3+4)
            self:diffuse(color('#000000c0'))
            self:fadetop(0.1)
        end,
    })
end

-- バナー
local function BannerActor()
    return Def.ActorFrame({
        InitCommand = function(self)
            self:xy(0, -h/2+((wheelMode == 'Jacket') and 89 or 38))
        end,
        -- バナー
        Banner:Actor()..{
            UpdateWheelChildCommand=function(self, params)
                self:playcommand('UpdateBanner', {
                    Path = (wheelMode == 'Jacket')
                        and W_GRAPHICS:SongJacket(params.Song)
                        or W_GRAPHICS:SongBanner(params.Song),
                })
                if wheelMode == 'Jacket' then
                    -- 横長の場合は上に寄せる
                    local dw = self:GetWidth()
                    local dh = self:GetHeight()
                    if dw/dh > w/h then
                        local calcH = w*dh/dw
                        self:y(-h/2+89 - (h-calcH)/2)
                        -- タイトルと被るところまで高さがある場合はフェードさせる
                        self:fadebottom(calcH > (h-h/3) and 0.2 or 0)
                    else
                        self:fadebottom(0)
                    end
                end
            end,
        }
    })
end

-- 曲名
local function SongTitleActor()
    return Def.ActorFrame({
        InitCommand = function(self)
            self:xy(0, -h/2 + 85)
        end,
        -- タイトル
        LoadFont('Common Small Normal')..{
            InitCommand=function(self)
                self:diffuse(color('#d8d8d8ff'))
                self:horizalign(left)
                self:vertalign(top)
                self:xy(-w/2+10, (wheelMode == 'Jacket') and 40 or 0)
            end,
            UpdateWheelChildCommand=function(self,params)
                local song = params.Song
                if song then
                    local textZoom = 0.9
                    self:zoom(textZoom)
                    self:maxwidth((w-20)/textZoom)
                    self:settext(W_STRING:CutWidth(song:GetDisplayMainTitle(), 13))
                    self:diffusealpha(params.HasFocus and 1 or 0.75)
                end
            end,
        },
    })
end

-- 曲の長さ
local function SongTimeActor()
    return Def.ActorFrame({
        InitCommand = function(self)
            self:xy(0, -h/2 + ((wheelMode == 'Jacket') and 148 or 115))
        end,
        -- 長さ
        LoadFont('Common Small Normal')..{
            InitCommand=function(self)
                local textZoom = 0.85
                self:diffuse(color('#a0a0a0ff'))
                self:horizalign(left)
                self:vertalign(top)
                self:zoom(textZoom)
                self:maxwidth((w/2-20)/textZoom)
                self:xy(-w/2+10, 0)
            end,
            UpdateWheelChildCommand=function(self,params)
                self:diffuse(color('#a0a0a0ff'))
                local song = params.Song
                if song then
                    -- 長さソートで不自然にならないように切り捨て
                    local sec = math.floor(song:MusicLengthSeconds())
                    if song:IsMarathon() then
                        self:diffuse(1.0, 0.3, 0.3, 1.0)
                    elseif song:IsLong() then
                        self:diffuse(0.85, 0.95, 0.3, 1.0)
                    elseif sec < 60 then
                        self:diffuse(0.2, 0.75, 1.0, 1.0)
                    end
                    self:settextf('%d:%02d', math.floor(sec/60), sec%60)
                    self:diffusealpha(params.HasFocus and 1 or 0.5)
                end
            end,
        },
        Def.ActorFrame({
            InitCommand=function(self)
                self:xy(w/2-10-40, 6)
            end,
            MusicLength:Actor()..{
                UpdateWheelChildCommand=function(self, params)
                    local song = params.Song
                    local col = color('0, 0, 0, 0')
                    if song then
                        -- 長さソートで不自然にならないように切り捨て
                        local sec = math.floor(song:MusicLengthSeconds())
                        if song:IsMarathon() then
                            col = {0.9, 0.3, 0.3, params.HasFocus and 1 or 0.5}
                        elseif song:IsLong() then
                            col = {0.55, 0.65, 0.3, params.HasFocus and 1 or 0.5}
                        elseif sec < 60 then
                            col = {0.2, 0.75, 1.0, params.HasFocus and 1 or 0.5}
                        end
                    end
                    self:playcommand('UpdateRadius', {
                        Color = col,
                    })
                end,
            },
            LoadFont('Common Small Bold')..{
                InitCommand=function(self)
                    local textZoom = 0.5
                    self:diffuse(color('#ffffffff'))
                    self:zoom(textZoom)
                    self:maxwidth((w/2-20)/textZoom)
                end,
                UpdateWheelChildCommand=function(self,params)
                    local song = params.Song
                    if song then
                        -- 長さソートで不自然にならないように切り捨て
                        local sec = math.floor(song:MusicLengthSeconds())
                        if song:IsMarathon() then
                            self:settext('MARATHON')
                        elseif song:IsLong() then
                            self:settext('LONG')
                        elseif sec < 60 then
                            self:settext('SHORT')
                        else
                            self:settext('')
                        end
                        self:diffusealpha(params.HasFocus and 1 or 0.5)
                    end
                end,
            },
        }),
    })
end

-- ジャンル
local function GenreActor()
    if wheelMode == 'Jacket' then
        return Def.Actor({})
    end
    return Def.ActorFrame({
        LoadFont('Common Small Normal')..{
            InitCommand = function(self)
                self:horizalign(right)
                self:xy(w/2 - 10, -h/2 + 155)
                self:zoom(0.5)
                self:maxwidth((w-20)/0.5)
                self:diffuse(color('#999999ff'))
                self:settext('')
            end,
            UpdateWheelChildCommand=function(self, params)
                local text = ((params.Song and params.Song:GetGenre() ~= '')
                                and string.upper(params.Song:GetGenre())
                                or string.upper(ProductFamily()))
                self:settext(W_STRING:CutWidth(text, 22))
            end,
        },
    })
end

return Def.ActorFrame({
    SetCommand=function(self, params)
        -- 多重呼び出しを回避
        if not self.Params then
            self.Params = {Id = math.random(), Song = nil, HasFocus = nil}
            TopBorder:Id('B'..self.Params.Id)
            MusicLength:Id('ML'..self.Params.Id)
        end
        if self.Params and params and params.Song and params.HasFocus ~= nil then
            if params.Song ~= self.Params.Song or params.HasFocus ~= self.Params.HasFocus then
                params.Id = self.Params.Id
                self.Params.Song = params.Song
                self.Params.HasFocus = params.HasFocus
                self:finishtweening()
                self:playcommand('UpdateWheel', params)
            end
        end
    end,
    UpdateWheelCommand = function(self, params)
        if self.Params.Id ~= params.Id then return end
        self:playcommand('UpdateWheelChild', params)
    end,
    -- 背景
    BgActor(),
    -- バナー
    BannerActor(),
    -- オーバーレイ
    OverActor(),
    -- 曲名
    SongTitleActor(),
    -- 長さ
    SongTimeActor(),
    -- ジャンル
    GenreActor(),
    -- デバッグ
    -- LoadFont('Common Normal')..{
    --     UpdateWheelCommand = function(self, params)
    --         local parent = self:GetParent()
    --         if parent.Params.Id == params.Id then
    --             self:settext(params.Song and params.Song:GetDisplayMainTitle()..'\n'..params.Id or 'Group')
    --         end
    --     end,
    -- },
})
