local width, height = ...
local optionAll = dofile(THEME:GetPathO('', 'Options.lua'))
local option = {Menu = {}, OptionList = {}}
option.OptionList = optionAll.OptionList
for i=1, #optionAll.Menu do
    local key = optionAll.Menu[i]
    -- UnJoin設定は二人プレイの時だけ表示
    if key ~= 'UnJoin' or W_PLAYER:NumJoined() >= 2 then
        option.Menu[#(option.Menu)+1] = optionAll.Menu[i]
    end
end
local cursor = 1

-- メニューの縦幅
local menuHeight = 60
local menuHeightHalf = menuHeight/2

local function MenuActor()
    local function MenuChildActor(i, k, v)
        local retActor = Def.ActorFrame({})
        retActor[#retActor+1] = LoadFont('Common Normal')..{
            Text = THEME:GetString('OptionTitles', k),
            OnCommand = function(self)
                self:horizalign(left)
                self:x(-100, i * menuHeightHalf)
                self:zoom(0.5)
            end,
            ScrollMenuCommand = function(self, params)
                self:stoptweening()
                -- メニュー表示アニメ中にキー入力があるとOnのコマンドが反映されない
                self:playcommand('On')
                self:linear(0.1)
                if i > params.Cursor then
                    self:y(i * menuHeightHalf + menuHeightHalf)
                else
                    self:y(i * menuHeightHalf)
                end
                self:addy((i == params.Cursor) and 0 or 5)
                self:diffuse(BoostColor(Color('White'), (i == params.Cursor) and 1 or 0.66))
            end,
        }
        if v.Type == 'None' then
            retActor[#retActor+1] = LoadFont('Common Normal')..{
                Text = k,
                OnCommand = function(self)
                    self:xy(0, (0.5+i) * menuHeightHalf + menuHeightHalf/2)
                    self:zoom(0.8)
                end,
                ScrollMenuCommand = function(self, params)
                    self:diffusealpha((i == params.Cursor) and 1 or 0)
                end,
            }
        else
            retActor[#retActor+1] = Def.ActorFrame({
                LoadFont('Common Normal')..{
                    Text = '&MenuLeft;',
                    OnCommand = function(self)
                        self:horizalign(left)
                        self:xy(-100, (0.5+i) * menuHeightHalf + menuHeightHalf/2)
                        self:zoom(0.8)
                    end,
                    SetOptValueMessageCommand = function(self, params)
                        local c = (params[k] == 1) and 0.33 or 1
                        self:diffuse(c, c, c, 1)
                    end,
                },
                LoadFont('Common Normal')..{
                    Text = '&MenuRight;',
                    OnCommand = function(self)
                        self:horizalign(right)
                        self:xy(100, (0.5+i) * menuHeightHalf + menuHeightHalf/2)
                        self:zoom(0.8)
                    end,
                    SetOptValueMessageCommand = function(self, params)
                        local c = (params[k] == #v.List) and 0.33 or 1
                        self:diffuse(c, c, c, 1)
                    end,
                },
                LoadFont('Common Normal')..{
                    Text = '',
                    OnCommand = function(self)
                        self:xy(0, (0.5+i) * menuHeightHalf + menuHeightHalf/2)
                        self:zoom(0.8)
                    end,
                    SetOptValueMessageCommand = function(self, params)
                        self:settext(v.Display[params[k]] or '')
                    end,
                },
                ScrollMenuCommand = function(self, params)
                    self:diffusealpha((i == params.Cursor) and 1 or 0)
                end,
            })
        end
        return retActor
    end

    local menuActor = Def.ActorFrame({})
    for i=1, #option.Menu do
        menuActor[#menuActor+1] = MenuChildActor(i, option.Menu[i], option.OptionList[option.Menu[i]])
    end
    return menuActor
end

return Def.ActorFrame({
    OnCommand = function(self)
        self:addy(200)
        self:diffusealpha(0)
        self:linear(0.2)
        self:addy(-200)
        self:diffusealpha(1)
    end,
    OffMessageCommand = function(self)
        self:linear(0.2)
        self:addy(200)
        self:diffusealpha(0.8)
    end,
    LoadActor(W:Object('boxshadow'), 'Background', {
        Length = 20,
        Color  = {0, 0, 0, 0.8},
        Center = {0, 0.1, 0.2, 0.5},
    })..{
        OnCommand = function(self, params)
            self:playcommand('UpdateBoxShadow', {
                Id         = 'Background',
                Width      = width,
                Height     = height,
            })
        end,
    },
    LoadActor('_parts/player', PLAYER_1, height),
    LoadActor('_parts/player', PLAYER_2, height),
    Def.ActorFrame({
        OnCommand = function(self)
            self:x(-9)    -- スクロールバーの分だけ左に寄せる
            self:playcommand('ScrollMenu', {Cursor = 1, Init = true})
        end,
        ScrollMessageCommand = function(self, params)
            cursor = params.Cursor
            self:playcommand('ScrollMenu', {Cursor = cursor})
        end,
        Def.Quad({
            OnCommand = function(self)
                self:y(-height/2 + 10 + menuHeight + menuHeightHalf)
                self:blend('BlendMode_Add')
                self:diffuse(0, 0.4, 0.8, 0.75)
                self:zoomto(240, menuHeight)
                self:fadeleft(0.5)
                self:faderight(0.5)
            end,
        }),
        Def.Quad({
            OnCommand=function(self)
                self:zoomto(SCREEN_WIDTH, SCREEN_HEIGHT-height/2)
                self:vertalign(bottom)
                self:y(-height/2)
                self:clearzbuffer(true)
                self:zwrite(true)
                self:blend('BlendMode_NoEffect')
            end,
        }),
        Def.Quad({
            OnCommand=function(self)
                self:zoomto(SCREEN_WIDTH, SCREEN_HEIGHT-(height/2+menuHeight*3))
                self:vertalign(top)
                self:y(-height/2+menuHeight*3)
                self:zwrite(true)
                self:blend('BlendMode_NoEffect')
            end,
        }),
        MenuActor()..{
            OnCommand = function(self)
                self:ztest(true)
                self:y(-height/2 + 20 + menuHeight)
                self:playcommand('ScrollMenu', {Cursor = 1})
            end,
            ScrollMenuCommand = function(self, params)
                self:stoptweening()
                self:linear(0.2)
                self:y(-height/2 + 20 + menuHeight - cursor*menuHeightHalf)
            end,
        },
        -- スクロールバー
        LoadActor(W:Object('scrollbar'), {
            Width  = 6,
            Height = menuHeightHalf*5,
            Count  = #option.Menu,
        })..{
            OnCommand = function(self)
                self:xy(130, -menuHeight + menuHeightHalf/2)
            end,
        },
    }),
    LoadFont('Common Small Normal')..{
        Text = 'HELP TEXT',
        OnCommand = function(self)
            --self:horizalign(left)
            self:xy(0, -height/2+menuHeight*3+50)
            self:zoom(0.45)
            self:wrapwidthpixels(280/0.45)
            self:vertspacing(8)
            self:settext(THEME:GetString('OptionExplanations', option.Menu[1]))
        end,
        ScrollMessageCommand = function(self, params)
            self:settext(THEME:GetString('OptionExplanations', option.Menu[params.Cursor]))
        end,
    },
    Def.HelpDisplay({
        File = 'Common Small Normal',
        InitCommand=function(self)
            self:diffuse(Color('White'))
            self:zoom(0.4);
            self:xy(0, height/2-12)
            self:SetSecsBetweenSwitches(3.0)
            self:SetTipsColonSeparated(THEME:GetString('ScreenSelectMusicOptions', 'HelperText'))
        end,
    }),
})
