local iconList = split(',', THEME:GetMetric('ScreenSelectPlayMode', 'ChoiceNames'))

local function LineActor(width)
    return Def.Quad({
        InitCommand = function(self)
            self:diffuse(1, 1, 1, 0.9)
            self:zoomto(width, 1)
            self:fadeleft(1)
            self:faderight(1)
        end,
    })
end

local w, h = 100, 100
local function IconActor(index, name)
    local radius = LoadActor(W:Object('radius'), {
        Width  = w+2,
        Height = h+13,
        Cell   = {1, 1, 1, 1, 1, 1, 0, 0, 0},
        Color  = color('0.3, 0.3, 0.3, 0.8'),
        Radius = 5,
    })
    return Def.ActorFrame({
        Name = name,
        radius:Actor()..{
            InitCommand = function(self)
                self:y(12)
            end,
        },
        Def.Quad({
            InitCommand = function(self)
                self:zoomto(w,h)
                self:diffuse(0.1, 0.1, 0.1, 0.75)
                self:y(13)
                self:diffuseshift()
                self:effectcolor1(0.1, 0.1, 0.1, 0.75)
            end,
            UpdateCursorCommand = function(self, params)
                if params.Index == index then
                    self:effectcolor2(0.5, 0.55, 0.6, 0.75)
                else
                    self:effectcolor2(0.1, 0.1, 0.1, 0.75)
                end
            end,
        }),
        LoadActor('_parts/icon/mode_'..string.lower(name))..{
            InitCommand = function(self)
                self:zoom(0.25)
            end,
        },
        LineActor(w)..{
            InitCommand = function(self)
                self:y(35)
            end,
        },
        LoadFont('Common Small Normal')..{
            Text = THEME:GetString('ScreenSelectPlayMode', name..'Title'),
            InitCommand = function(self)
                self:zoom(0.5)
                self:y(48)
            end,
        }
    })
end

local iconActors = Def.ActorFrame({})
for i=#iconList, 1, -1 do
    iconActors[#iconActors+1] = IconActor(i, iconList[i])..{
        OnCommand = function(self)
            self:xy(SCREEN_CENTER_X, SCREEN_HEIGHT + 100)
            self:decelerate(0.2)
            self:xy(SCREEN_CENTER_X, SCREEN_HEIGHT - 120)
            self:decelerate(0.2)
            self:x(SCREEN_CENTER_X - 220 + (440*(i-1)/(#iconList-1)))
        end,
        OffCommand = function(self)
            self:decelerate(0.2)
            self:xy(SCREEN_CENTER_X, SCREEN_HEIGHT - 120)
            self:decelerate(0.2)
            self:xy(SCREEN_CENTER_X, SCREEN_HEIGHT + 100)
        end,
    }
end

local scroller
local old = nil
local function update(self)
    if scroller then
        local i = scroller:GetDestinationItem()+1
        if old ~= i then
            self:playcommand('UpdateCursor', {
                Index = i,
            })
            old = i
        end
    end
end

local actors = Def.ActorFrame({
    OnCommand = function(self, params)
        scroller = SCREENMAN:GetTopScreen():GetChild('Scroller')
        self:SetUpdateFunction(update)
    end,
    -- 下部のアイコン一覧
    iconActors..{
        InitCommand = function(self)
            self:xy(-SCREEN_CENTER_X, -SCREEN_CENTER_Y)
        end,
    },
    -- 上部のアイコンと説明
    Def.ActorFrame({
        OnCommand = function(self)
            self:zoomx(0)
            self:sleep(0.2)
            self:decelerate(0.2)
            self:zoomx(1)
        end,
        OffCommand = function(self)
            self:sleep(0.2)
            self:decelerate(0.18)
            self:zoomy(0)
            self:addy(-100)
        end,
        Def.Quad({
            InitCommand = function(self)
                self:diffuse(0.1, 0.1, 0.1, 0.75)
                self:zoomto(SCREEN_WIDTH+2, 220)
                self:y(-80)
            end,
        }),
        LineActor(SCREEN_WIDTH+2)..{
            InitCommand = function(self)
                self:y(-190)
            end,
        },
        LineActor(SCREEN_WIDTH+2)..{
            InitCommand = function(self)
                self:y(30)
            end,
        },
        Def.Sprite({
            InitCommand = function(self)
                self:zoom(0.5)
                self:y(-120)
                self:Load(THEME:GetPathG('', 'ScreenSelectPlayMode/_parts/icon/mode_'..iconList[1]))
            end,
            UpdateCursorCommand = function(self, params)
                self:Load(THEME:GetPathG('', 'ScreenSelectPlayMode/_parts/icon/mode_'..iconList[params.Index]))
                self:finishtweening()
                self:diffusealpha(0)
                self:addx(20)
                self:linear(0.2)
                self:diffusealpha(1)
                self:addx(-20)
            end,
        }),
        LoadFont('Common Normal')..{
            Text = '',
            InitCommand = function(self)
                self:diffuse(Color('White'))
                self:diffusealpha(0.9)
                self:zoom(0.9)
                self:y(-42)
            end,
            UpdateCursorCommand = function(self, params)
                self:settext(THEME:GetString('ScreenSelectPlayMode', iconList[params.Index]..'Title'))
            end,
        },
        LoadFont('Common Small Normal')..{
            Text = '',
            InitCommand = function(self)
                self:diffuse(Color('White'))
                self:diffusealpha(0.8)
                self:zoom(0.5)
                self:y(-15)
                self:vertalign(top)
                self:wrapwidthpixels(400)
                self:vertspacing(10)
            end,
            UpdateCursorCommand = function(self, params)
                self:settext(THEME:GetString('ScreenSelectPlayMode', iconList[params.Index]..'Text'))
            end,
        },
        -- 左右の矢印（選曲ホイールから拝借）
        Def.ActorFrame({
            InitCommand=function(self)
                self:zoom(2)
                self:y(-75)
            end,
            LoadActor(THEME:GetPathG('ScreenSelectMusic/wheel/frame', 'over'), 150, 'timer'),
        }),
    }),
    Def.HelpDisplay({
        File = 'Common Small Normal',
        InitCommand=function(self)
            self:diffuse(Color('White'))
            self:zoom(0.4);
            self:xy(0, SCREEN_CENTER_Y-12)
            self:SetSecsBetweenSwitches(3.0)
            self:SetTipsColonSeparated(THEME:GetString('Screen', 'HelperTextDefault'))
        end,
        OffCommand = function(self)
            self:sleep(0.2)
            self:linear(0.3)
            self:addy(30)
        end,
    }),
})
return actors