-- ステップゾーン座標テンプレート
-- Group.ini/luaのReceptorPositionで意図的に定義しなくても利用可能にするための定義
--[[
    例）
    Group.ini
    #RECEPTORPOSITION:StepMania5;
    #RECEPTORPOSITION:StepMania3|songA|songB:StepMania5|songC;

    Group.lua
    ReceptorPosition = {
        StepMania3 = {'songA', 'songB'},
        StepMania5 = {'songC'},
    },
    -- OR
    ReceptorPosition = {
        {
            MyPosition = {
                Y = {Standard = -50, Reverse = 50},
            },
        },
        MyPosition = {'songA', 'songC'},
    },
--]]

-- グローバル関数にFindValueが存在するが、5.0と異なるので5.1のコードを利用
local function _FindValue_(tab, value)
    for key, name in pairs(tab) do
        if value == name then
            return key
        end
    end

    return nil
end

local data = {
    -- StepMania 3.9系
    -- キー名（大文字小文字は区別しない）：StepMania3
    stepmania3 = {
        X = {
            P1OnePlayerOneSide    = SCREEN_CENTER_X-180,
            P2OnePlayerOneSide    = SCREEN_CENTER_X+180,
            P1TwoPlayersTwoSides  = SCREEN_CENTER_X-180,
            P2TwoPlayersTwoSides  = SCREEN_CENTER_X+180,
            P1OnePlayerTwoSides   = SCREEN_CENTER_X,
            P2OnePlayerTwoSides   = SCREEN_CENTER_X,
            P1TwoPlayersSharedSides = SCREEN_CENTER_X,
            P2TwoPlayersSharedSides = SCREEN_CENTER_X,
        },
        Y = {Standard = -144, Reverse = 144},
    },
    -- StepMania 5.0系（5.1も同じ）
    -- キー名（大文字小文字は区別しない）：StepMania5
    stepmania5 = {
        X = {
            P1OnePlayerOneSide    = math.floor(scale((0.85/3),0,1,SCREEN_LEFT,SCREEN_RIGHT)),
            P2OnePlayerOneSide    = math.floor(scale((2.15/3),0,1,SCREEN_LEFT,SCREEN_RIGHT)),
            P1TwoPlayersTwoSides  = math.floor(scale((0.85/3),0,1,SCREEN_LEFT,SCREEN_RIGHT)),
            P2TwoPlayersTwoSides  = math.floor(scale((2.15/3),0,1,SCREEN_LEFT,SCREEN_RIGHT)),
            P1OnePlayerTwoSides   = SCREEN_CENTER_X,
            P2OnePlayerTwoSides   = SCREEN_CENTER_X,
            P1TwoPlayersSharedSides = SCREEN_CENTER_X,
            P2TwoPlayersSharedSides = SCREEN_CENTER_X,
        },
        Y = {Standard = -125, Reverse = 145},
    },
    -- Project OutFox(5.3)
    -- キー名（大文字小文字は区別しない）：ProjectOutFox
    projectoutfox = {
        X = {
            P1OnePlayerOneSide    = SCREEN_CENTER_X - (SCREEN_WIDTH*155/640),
            P2OnePlayerOneSide    = SCREEN_CENTER_X + (SCREEN_WIDTH*155/640),
            P1TwoPlayersTwoSides  = SCREEN_CENTER_X - (SCREEN_WIDTH*155/640),
            P2TwoPlayersTwoSides  = SCREEN_CENTER_X + (SCREEN_WIDTH*155/640),
            P1OnePlayerTwoSides   = SCREEN_CENTER_X,
            P2OnePlayerTwoSides   = SCREEN_CENTER_X,
        },
        Y = {Standard = -125, Reverse = 145},
    },
    -- Simply Love
    -- https://github.com/Simply-Love/Simply-Love-SM5
    -- キー名（大文字小文字は区別しない）：SimplyLove
    simplylove = {
        X = {
            P1OnePlayerOneSide    = SCREEN_CENTER_X - (clamp(SCREEN_WIDTH, 640, 854) * 0.25),
            P2OnePlayerOneSide    = SCREEN_CENTER_X + (clamp(SCREEN_WIDTH, 640, 854) * 0.25),
            P1TwoPlayersTwoSides  = SCREEN_CENTER_X - (clamp(SCREEN_WIDTH, 640, 854) * 0.25),
            P2TwoPlayersTwoSides  = SCREEN_CENTER_X + (clamp(SCREEN_WIDTH, 640, 854) * 0.25),
            P1OnePlayerTwoSides   = SCREEN_CENTER_X,
            P2OnePlayerTwoSides   = SCREEN_CENTER_X,
        },
        Y = {Standard = -125, Reverse = 145},
    },
    -- CyberiaStyle LAST APPROACH
    -- キー名（大文字小文字は区別しない）：CyberiaStyle
    cyberiastyle = {
        X = {
            P1OnePlayerOneSide   = math.floor(scale((0.72/3),0,1,SCREEN_LEFT,SCREEN_RIGHT)),
            P2OnePlayerOneSide   = math.floor(scale((2.28/3),0,1,SCREEN_LEFT,SCREEN_RIGHT)),
            P1TwoPlayersTwoSides = math.floor(scale((0.72/3),0,1,SCREEN_LEFT,SCREEN_RIGHT)),
            P2TwoPlayersTwoSides = math.floor(scale((2.28/3),0,1,SCREEN_LEFT,SCREEN_RIGHT)),
            P1OnePlayerTwoSides  = _FindValue_({'PlayMode_Battle', 'PlayMode_Rave'}, GAMESTATE:GetPlayMode()) and math.floor(scale((0.72/3),0,1,SCREEN_LEFT,SCREEN_RIGHT)) or SCREEN_CENTER_X,
            P2OnePlayerTwoSides  = _FindValue_({'PlayMode_Battle', 'PlayMode_Rave'}, GAMESTATE:GetPlayMode()) and math.floor(scale((2.28/3),0,1,SCREEN_LEFT,SCREEN_RIGHT)) or SCREEN_CENTER_X,
            P1TwoPlayersSharedSides = SCREEN_CENTER_X,
            P2TwoPlayersSharedSides = SCREEN_CENTER_X,
        },
        Y = {Standard = -125, Reverse = 145},
    },
    -- WAIEI2/WAIEI Citmin
    -- キー名（大文字小文字は区別しない）：WAIEI
    waiei = {
        X = {
            P1OnePlayerOneSide      = ((SCREEN_HEIGHT/SCREEN_WIDTH>0.7) and math.floor(scale((0.75/3),0,1,SCREEN_LEFT,SCREEN_RIGHT)) or math.floor(scale((0.85/3),0,1,SCREEN_LEFT,SCREEN_RIGHT))),
            P2OnePlayerOneSide      = ((SCREEN_HEIGHT/SCREEN_WIDTH>0.7) and math.floor(scale((2.25/3),0,1,SCREEN_LEFT,SCREEN_RIGHT)) or math.floor(scale((2.15/3),0,1,SCREEN_LEFT,SCREEN_RIGHT))),
            P1TwoPlayersTwoSides    = ((SCREEN_HEIGHT/SCREEN_WIDTH>0.7) and math.floor(scale((0.75/3),0,1,SCREEN_LEFT,SCREEN_RIGHT)) or math.floor(scale((0.85/3),0,1,SCREEN_LEFT,SCREEN_RIGHT))),
            P2TwoPlayersTwoSides    = ((SCREEN_HEIGHT/SCREEN_WIDTH>0.7) and math.floor(scale((2.25/3),0,1,SCREEN_LEFT,SCREEN_RIGHT)) or math.floor(scale((2.15/3),0,1,SCREEN_LEFT,SCREEN_RIGHT))),
            P1OnePlayerTwoSides     = SCREEN_CENTER_X,
            P2OnePlayerTwoSides     = SCREEN_CENTER_X,
            P1TwoPlayersSharedSides = SCREEN_CENTER_X,
            P2TwoPlayersSharedSides = SCREEN_CENTER_X,
        },
        Y = {Standard = -144, Reverse = 134},
    },
}

return function(key)
    -- 存在しないキーを指定した場合はwaieiが呼び出される
    return data[string.lower(key)] or data.waiei
end
