-- アバター
local FileType = {
    YA = 0,
    CS = 1,
    OF = 2,
}
-- パーツ種別
local PartType = {
    Normal    = 0, -- 通常
    Face      = 1, -- 輪郭(1が左側、2が右側)
    Eye       = 2, -- 目(0が白目含む全体、1が左向き時の左目、2が左向き時の右目)
    Half      = 3, -- 半分のサイズ(1が上側、2が下側)
    Accessory = 4, -- アクセサリー（描画順序の変更が可能）
}
-- 色種別
local ColorType = {
    Fixed  = 'Fixed',  -- 変更不可
    Normal = 'Normal', -- 変更可能
    Face   = 'Face',   -- 肌共通
    Hair   = 'Hair',   -- 髪色共通
}
-- 表情種別
local EmotionType = {
    Default = 'Default',
    Win     = 'Win',
    Lose    = 'Lose',
}
-- パーツ別表情パターン
local EmotionMode = {
    Fixed      = {EmotionType.Default},
    Expressive = {EmotionType.Default, EmotionType.Win, EmotionType.Lose},
}

local CommonList = {
    'FaceColor', 'HairTopColor', 'HairBottomColor', 'OddEyeColor'
}

-- Actor分定義
local parts = {
    {Key = 'face1',      Name = 'face',       Order =  14, PartType = PartType.Face,   ColorType = ColorType.Face,   EmotionMode = EmotionMode.Fixed, Pos = 2},
    {Key = 'face2',      Name = 'face',       Order =  20, PartType = PartType.Face,   ColorType = ColorType.Face,   EmotionMode = EmotionMode.Fixed, Pos = 1},
    {Key = 'ear1',       Name = 'ear',        Order =  10, PartType = PartType.Face,   ColorType = ColorType.Face,   EmotionMode = EmotionMode.Fixed, Pos = 1},
    {Key = 'ear2',       Name = 'ear',        Order =  16, PartType = PartType.Face,   ColorType = ColorType.Face,   EmotionMode = EmotionMode.Fixed, Pos = 2},
    {Key = 'back',       Name = 'back',       Order =   5, PartType = PartType.Normal, ColorType = ColorType.Hair,   EmotionMode = EmotionMode.Fixed},
    {Key = 'side1',      Name = 'side',       Order =  12, PartType = PartType.Face,   ColorType = ColorType.Hair,   EmotionMode = EmotionMode.Fixed, Pos = 1},
    {Key = 'side2',      Name = 'side',       Order =  22, PartType = PartType.Face,   ColorType = ColorType.Hair,   EmotionMode = EmotionMode.Fixed, Pos = 2},
    {Key = 'front',      Name = 'front',      Order =  60, PartType = PartType.Normal, ColorType = ColorType.Hair,   EmotionMode = EmotionMode.Fixed},
    {Key = 'accent',     Name = 'accent',     Order =  90, PartType = PartType.Normal, ColorType = ColorType.Hair,   EmotionMode = EmotionMode.Fixed},
    {Key = 'eye1',       Name = 'eye',        Order =  30, PartType = PartType.Eye,    ColorType = ColorType.Fixed,  EmotionMode = EmotionMode.Expressive, Pos = 0},
    {Key = 'eye2',       Name = 'eye',        Order =  31, PartType = PartType.Eye,    ColorType = ColorType.Normal, EmotionMode = EmotionMode.Expressive, Pos = 1},
    {Key = 'eye3',       Name = 'eye',        Order =  32, PartType = PartType.Eye,    ColorType = ColorType.Normal, EmotionMode = EmotionMode.Expressive, Pos = 2},
    {Key = 'pupil',      Name = 'pupil',      Order =  40, PartType = PartType.Half,   ColorType = ColorType.Normal, EmotionMode = EmotionMode.Expressive, Pos = 2},
    {Key = 'mouth',      Name = 'mouth',      Order =  50, PartType = PartType.Half,   ColorType = ColorType.Normal, EmotionMode = EmotionMode.Expressive, Pos = 2},
    {Key = 'complexion', Name = 'complexion', Order =  25, PartType = PartType.Normal, ColorType = ColorType.Fixed,  EmotionMode = EmotionMode.Expressive},
    {Key = 'brow',       Name = 'brow',       Order =  65, PartType = PartType.Normal, ColorType = ColorType.Fixed,  EmotionMode = EmotionMode.Expressive},
    {Key = 'accessory1', Name = 'accessory1', Order = 100, PartType = PartType.Normal, ColorType = ColorType.Normal, EmotionMode = EmotionMode.Fixed},
    {Key = 'accessory2', Name = 'accessory2', Order = 100, PartType = PartType.Normal, ColorType = ColorType.Normal, EmotionMode = EmotionMode.Fixed},
}

local acceOrderLv = {
     0,  --1:最背面
    21,  --2:顔とサイド髪の間
    55,  --3:前髪のうしろ
    70,  --4:アクセントヘアーのうしろ
    100, --5:最前面
}

-- Avatarsフォルダのパス
local function GetPath(self, ...)
    local relative = ...
    return THEME:GetCurrentThemeDirectory()..'Avatars/'..(relative or '')
end

local w, h = 128, 128

-- Ini以外は最前列レイヤーに表示させるためファイル種別を取得しておく
local function ChangeAvatar(file, randomNum)
    -- デフォルトアバターを取得
    if file and file == '<default>' then
        return IniFile.ReadFile(THEME:GetPathO('Avatar', 'Default')), 'FileType_Ini', '<default>'
    end
    local avatarFile = (file and FILEMAN:DoesFileExist(file)) and file or nil
    local avatarType = avatarFile and ActorUtil.GetFileType(avatarFile) or 'FileType_Nil'
    local ini = {}
    if avatarType == 'FileType_Ini' then
        ini = IniFile.ReadFile(avatarFile)
        if ini.citmin and ini.citmin.Version
            and tonumber(ini.citmin.Version) >= 30000000 and tonumber(ini.citmin.Version) <= W:Ver() then
            return ini, avatarType, avatarFile
        end
        ini = {}
        avatarType = nil
        avatarFile = nil
    end
    if not avatarFile then
        if randomNum >= 1 and randomNum <= 8 then
            avatarFile = W:Object('avatar/random'..randomNum)
        else
            avatarFile = W:Object('avatar/default')
        end
    end
    return ini, avatarType, avatarFile
end

-- アバターを表示(アバターファイルパス, ID)
local function AvatarActor(self, avatar, id, ...)
    local params = ...
    local flip     = params and params.Flip or false
    local miniMode = params and params.Mini or false
    local randomNum = params and params.RandomNum or 0    -- デフォルト画像用
    -- パーツ情報
    local data = {}
    -- fallback
    local fbGroup = 'waiei'
    local fbFile = '00'
    local function Actor()
        local function GetPartFilePath(group, name, file)
            -- アクセサリー対策で末尾の数値を無視する
            return string.format('%s%s/%s/%s.png', GetPath(self), group, string.match(name, '[^0-9]+'), file)
        end
        local a = Def.ActorFrame({
            InitCommand = function(self)
                self:SetDrawByZPosition(true)
                self:playcommand('DefaultEmotion')
            end,
            ReSortCommand = function(self)
                self:SortByDrawOrder()
            end,
            -- ini形式以外の場合の画像表示
            Def.Sprite({
                InitCommand = function(self)
                    self:visible(false)
                end,
                DefaultEmotionCommand = function(self)
                    self:visible(not data.IsIni)
                end,
                WinEmotionCommand = function(self)
                    self:playcommand('DefaultEmotion')
                end,
                LoseEmotionCommand = function(self)
                    self:playcommand('DefaultEmotion')
                end,
                SetPartCommand = function(self, params)
                    -- ファイルが存在しないまたはini形式の場合は強制的に非表示にする
                    if not data.File or data.IsIni then
                        self:visible(false)
                        return
                    end
                    self:visible(true)
                    -- あまりに小さく表示する場合はminiModeがおすすめ
                    if miniMode then
                        self:LoadBackground(data.File)
                    else
                        self:Load(data.File)
                    end
                    self:scaletofit(-w/2, -h/2, w/2, h/2)
                end,
            }),
        })
        for _,part in pairs(parts) do
            for _,emotion in pairs(part.EmotionMode) do
                a[#a+1] = Def.Sprite({ 
                    InitCommand = function(self)
                        self:visible(false)
                    end,
                    -- 通常時を表示
                    DefaultEmotionCommand = function(self)
                        if not data.IsIni then return end
                        data.Emotion = EmotionType.Default
                        self:visible((emotion == EmotionType.Default) and data[part.Name][emotion].Visible or false)
                    end,
                    -- 優勢時を表示
                    WinEmotionCommand = function(self)
                        if not data.IsIni then return end
                        data.Emotion = EmotionType.Win
                        if part.EmotionMode == EmotionMode.Fixed then
                            self:visible(true)
                        else
                            self:visible((emotion == EmotionType.Win) and data[part.Name][emotion].Visible or false)
                        end
                    end,
                    -- 劣勢時を表示
                    LoseEmotionCommand = function(self)
                        if not data.IsIni then return end
                        data.Emotion = EmotionType.Lose
                        if part.EmotionMode == EmotionMode.Fixed then
                            self:visible(true)
                        else
                            self:visible((emotion == EmotionType.Lose) and data[part.Name][emotion].Visible or false)
                        end
                    end,
                    -- パーツ情報を更新
                    SetPartCommand = function(self, params)
                        -- ファイルがini形式でない場合は強制的に非表示にする
                        if not data.IsIni then self:visible(false) return end
                        if params.Part ~= part.Name then return end
                        -- 対象のパーツ情報が存在しないなら処理を行わない
                        local partData = data[part.Name] or nil
                        if not partData then return end
                        -- 表情情報が存在せず、差分が出るパーツの場合は処理を行わない
                        if not partData[emotion] and part.EmotionMode == EmotionMode.Expressive then return end

                        local isAccessory = (part.Name == 'accessory1' or part.Name == 'accessory2')
                        -- 画像格納グループ
                        local group = data[part.Name][emotion].Group or nil
                        -- 画像ファイル
                        local file = data[part.Name][emotion].File or nil
                        if params.Update[emotion] then
                            -- 表示条件
                            if not group or not file then
                                -- 画像が意図的にnil設定の場合は非表示
                                data[part.Name][emotion].Visible = false
                            else
                                -- 意図的にfalseを設定しない場合は表示
                                data[part.Name][emotion].Visible = (partData[emotion].Visible == nil or partData[emotion].Visible == true)
                            end
                        end
                        if params.Update.Color then
                            -- メインカラー
                            data[part.Name].Color = partData.Color or color('#ffffffff')
                        end
                        -- 非表示に設定された場合はこれ以上処理を行わない
                        if data[part.Name][emotion].Visible == false then
                            self:visible(false)
                            return
                        end
                        -- 表情固定パーツまたは現在の表情の時のみ表示
                        self:visible(part.EmotionMode == EmotionMode.Fixed or data.Emotion == emotion)
                        -- パーツの変更
                        if params.Update[emotion] then
                            self:playcommand('UpdatePartFile', {IsAccessory = isAccessory})
                        end
                        -- 重ね順序の変更
                        if params.Update.Order then
                            self:playcommand('UpdateOrder', {IsAccessory = isAccessory})
                        end
                        -- ブレンドの変更
                        if params.Update.Blend then
                            self:playcommand('UpdateBlend', {IsAccessory = isAccessory})
                        end
                        -- カラーの変更
                        if params.Update.Color then
                            self:playcommand('UpdateColor')
                        end
                    end,
                    -- パーツ画像を変更
                    UpdatePartFileCommand = function(self, params)
                        -- ファイル存在チェック
                        if not FILEMAN:DoesFileExist(GetPartFilePath(data[part.Name][emotion].Group, part.Name, data[part.Name][emotion].File)) then
                            -- 画像が存在しない場合はfallback画像
                            data[part.Name][emotion].Group = fbGroup
                            data[part.Name][emotion].File = fbFile
                        end
                        -- fallbackになった可能性があるので改めて取得
                        local file = GetPartFilePath(data[part.Name][emotion].Group, part.Name, data[part.Name][emotion].File)
                        -- あまりに小さく表示する場合はminiModeがおすすめ
                        if miniMode then
                            self:LoadBackground(file)
                        else
                            self:Load(file)
                        end
                        -- ブレンド
                        self:playcommand('UpdateBlend', params)
                        -- 上下に分割するタイプのパーツ
                        if part.PartType == PartType.Half then
                            -- 上下半分サイズのパーツ
                            self:vertalign((part.Pos == 2) and top or bottom)
                            self:xy(0, 0)
                        -- 左右に分割するタイプのパーツ（顔、耳）
                        elseif part.PartType == PartType.Face then
                            -- 顔は左半分右半分で描画順序が異なる（もみあげより奥か手前か）
                            if part.Pos == 1 then
                                self:cropright(0.5)
                            else
                                self:cropleft(0.5)
                            end
                            self:xy(0, 0)
                        -- 目は1つのパーツに黒目と白目が定義されている
                        elseif part.PartType == PartType.Eye then
                            if part.Pos == 1 then
                                -- 左目
                                self:cropbottom(0.5)
                                self:cropleft(0.5)
                                self:xy(0, h/2)
                            elseif part.Pos == 2 then
                                -- 右目
                                self:cropbottom(0.5)
                                self:cropright(0.5)
                                self:xy(0, h/2)
                            else
                                -- 瞳
                                self:croptop(0.5)
                                self:xy(0, 0)
                            end
                        else
                            self:xy(0, 0)
                        end
                        -- 重ね順序
                        self:playcommand('UpdateOrder', params)
                    end,
                    -- 重ね順序を変更
                    UpdateOrderCommand = function(self, params)
                        -- 重ね順序
                        local value = params.Order or data[part.Name].Order or part.Order
                        self:draworder(params.IsAccessory and acceOrderLv[value] or value)
                    end,
                    -- ブレンドを変更
                    UpdateBlendCommand = function(self, params)
                        local value = params.Blend or data[part.Name].Blend or part.Blend
                        self:blend(string.format('BlendMode_%s', value or 'Normal'))
                    end,
                    -- 色を変更
                    UpdateColorCommand = function(self)
                        if part.ColorType == ColorType.Hair then
                            -- 髪の毛
                            local bottomColor = data.common.HairBottomColor or data.common.HairTopColor or color('#ffffff')
                            self:diffuse(data.common.HairTopColor or color('#ffffff'))
                            self:diffusebottomedge(bottomColor)
                        elseif part.ColorType == ColorType.Face then
                            -- 肌色
                            self:diffuse(data.common.FaceColor or color('#ffffff'))
                        elseif part.PartType == PartType.Eye then
                            if part.ColorType ~= ColorType.Fixed then
                                -- 目（オッドアイ含む）
                                local oddColor = data.common.OddEyeColor or data[part.Name].Color
                                if (not flip and part.Pos == 2) or (flip and part.Pos == 1) then
                                    self:diffuse(oddColor)
                                else
                                    self:diffuse(data[part.Name].Color)
                                end
                            else
                                -- 白目部分
                                self:diffuse(color('#ffffff'))
                            end
                        elseif part.ColorType ~= ColorType.Fixed then
                            -- 通常カラー
                            self:diffuse(data[part.Name].Color)
                        end
                    end,
                })
            end
        end
        return a
    end

    return Def.ActorFrame({
        -- 座標の調整で影響が出ないように子Actorを作成
        Def.ActorFrame({
            InitCommand = function(self)
                self:playcommand('ChangeAvatarFile', {
                    Id = id,
                    File = avatar,
                    RandomNum = randomNum,
                })
            end,
            -- アバターファイルそのものを変更
            ChangeAvatarFileMessageCommand = function(self, params)
                -- IDが指定されていない場合は直接playcommand経由で呼び出された想定
                if params.Id and params.Id ~= id then return end
                randomNum = params.RandomNum or randomNum
                local ini, avatarType, avatarFile = ChangeAvatar(params.File or nil, randomNum)
                -- 管理用変数を初期化
                data = {}
                if avatarType == 'FileType_Ini' then
                    -- ini未定義時の対応
                    if not ini.main then
                        ini.main = {Name = 'No Name'}
                    end
                    if not ini.common then
                        ini.common = {}
                    end
                    for _,v in pairs(CommonList) do
                        -- オッドアイが定義されていない場合は通常の目の色
                        if not ini.common[v] and v ~= 'OddEyeColor' then
                            ini.common[v] = '#FFFFFFFF'
                        end
                    end
                    for _,part in pairs(parts) do
                        if not ini[part.Name] then
                            ini[part.Name] = {}
                        end
                        if part.ColorType == ColorType.Normal then
                            if not ini[part.Name].Color then
                                ini[part.Name].Color = '#FFFFFFFF'
                            end
                        end
                        if string.find(part.Name, '^accessory.+') then
                            if not ini[part.Name].Order then
                                ini[part.Name].Order = #acceOrderLv
                                ini[part.Name].Blend = 'Normal'
                            end
                        end
                        for _,emotion in pairs(part.EmotionMode) do
                            if not ini[part.Name][emotion] and (part.EmotionMode == EmotionMode.Fixed or emotion == 'Default') then
                                ini[part.Name][emotion] = fbGroup..'/'..fbFile
                            end
                        end
                    end
                    -- 初期化
                    for _,part in pairs(parts) do
                        data[part.Name] = {}
                        if string.find(part.Name, '^accessory.+') then
                            -- オーダーは実際の値ではなくキー番号
                            for k,v in pairs(acceOrderLv) do
                                data[part.Name].Order = k
                                if v > part.Order then break end
                            end
                        end
                        for _,emotion in pairs(part.EmotionMode) do
                            data[part.Name][emotion] = {}
                        end
                    end
                    data.IsIni = true
                    data.File = avatarFile
                    data.Emotion = EmotionType.Default
                    data.common = {}
                    -- 表情別パーツが定義されていない場合はDefaultを設定する
                    for _,part in pairs(parts) do
                        if part.EmotionMode == EmotionMode.Expressive then
                            ini[part.Name].Win  = ini[part.Name].Win  or ini[part.Name].Default
                            ini[part.Name].Lose = ini[part.Name].Lose or ini[part.Name].Default
                        end
                    end
                    self:rotationy(flip and 180 or 0)
                    -- 後頭部分ずれるので微調整
                    self:xy(flip and -2 or 2, -3)
                    -- パーツを設定する
                    self:playcommand('ChangeAvatarParts', ini)
                    self:playcommand('DefaultEmotion')
                    self:playcommand('ReSort')
                else
                    data.IsIni = false
                    data.File = avatarFile
                    self:rotationy(0)
                    self:xy(0, 0)
                    -- 画像を表示する
                    self:playcommand('SetPart')
                end
            end,
            -- パーツを変更
            ChangeAvatarPartsMessageCommand = function(self, params)
                -- IDが指定されていない場合は直接playcommand経由で呼び出された想定
                if params.Id and params.Id ~= id then return end
                -- params[PartName][Emotion][...]という構成
                -- 共通パーツ
                local updateCommon = false
                if params.common then
                    for _,v in pairs(CommonList) do
                        if params.common[v] then
                            data.common[v] = color(params.common[v])
                            updateCommon = true
                        end
                    end
                end
                for _,part in pairs(parts) do
                    -- あくまで設定されていない値は更新せずそのままにする
                    local update = {
                        Color = updateCommon, Blend = false, Order = false,
                        Default = false, Win = false, Lose = false,
                    }
                    -- 色と重ね順
                    if params[part.Name] then
                        -- 表情別パーツ
                        for _,emotion in pairs(EmotionType) do
                            update[emotion] = false
                        end
                        for _,emotion in pairs(EmotionType) do
                            if params[part.Name][emotion] then
                                local emotionTemp = {}
                                if type(params[part.Name][emotion]) == 'string' then
                                    -- パスで指定されている場合
                                    local p = split('/', params[part.Name][emotion])
                                    emotionTemp.Group = p[1] or nil
                                    emotionTemp.File = p[2] or nil
                                else
                                    -- 配列で指定されている場合
                                    for _,v in pairs({'Group', 'File', 'Visible'}) do
                                        emotionTemp[v] = params[part.Name][emotion][v] or nil
                                    end
                                end
                                -- 表情別ではないパーツの場合全表情を更新する
                                local targetEmotion = (part.EmotionMode == EmotionMode.Fixed) and EmotionType or {emotion}
                                for _,v in pairs(targetEmotion) do
                                    update[v] = true
                                    data[part.Name][v] = {}
                                    data[part.Name][v].Group   = emotionTemp.Group
                                    data[part.Name][v].File    = emotionTemp.File
                                    data[part.Name][v].Visible = emotionTemp.Visible or nil
                                end
                            end
                            if params[part.Name].Order then
                                data[part.Name].Order = params[part.Name].Order
                                update.Order = true
                            end
                            if params[part.Name].Blend then
                                data[part.Name].Blend = params[part.Name].Blend
                                update.Blend = true
                            end
                            if params[part.Name].Color then
                                data[part.Name].Color = color(params[part.Name].Color)
                                update.Color = true
                            end
                        end
                    end
                    -- 何かしら更新がある時のみメッセージを飛ばす
                    for _,v in pairs(update) do
                        if v then
                            self:playcommand('SetPart', {Part = part.Name, Update = update})
                            break
                        end
                    end
                end
                self:playcommand('ReSort')
            end,
            SetAvatarEmotionMessageCommand = function(self, params)
                -- IDが指定されていない場合は直接playcommand経由で呼び出された想定
                if params.Id and params.Id ~= id then return end
                if params.Emotion == EmotionType.Default then
                    self:playcommand('DefaultEmotion')
                elseif params.Emotion == EmotionType.Win then
                    self:playcommand('WinEmotion')
                elseif params.Emotion == EmotionType.Lose then
                    self:playcommand('LoseEmotion')
                end
            end,
            CallGetAvatarDataMessageCommand = function(self, params)
                if params.Id and params.Id ~= id then return end
                MESSAGEMAN:Broadcast('GetAvatarData', {Id = params.Id or nil, Data = data})
            end,
            Actor(),
        }),
    })
end

-- フレーム付きアバター
local function AvatarWithFrame(self, avatar, id, ...)
    local params = ...
    local frameColor  = params and params.Color or color('#ffffff')
    local bottomColor = params and params.BottomColor or frameColor
    local frameType   = params and params.Type or 0
    local params = params or {}
    return Def.ActorFrame({
        LoadActor(W:Object('avatar/frame'))..{
            OnCommand = function(self)
                self:animate(false)
                self:setstate(frameType*4 + 1)
                self:zoom(2)
            end,
        },
        LoadActor(W:Object('avatar/frame'))..{
            OnCommand = function(self)
                self:animate(false)
                self:setstate(frameType*4 + 0)
                self:diffuse(frameColor)
                self:diffusebottomedge(bottomColor)
                self:cropright(0.01)
                self:clearzbuffer(true)
            end,
        },
        LoadActor(W:Object('avatar/frame'))..{
            OnCommand = function(self)
                self:animate(false)
                self:setstate(frameType*4 + 3)
                self:zwrite(true)
                self:blend('BlendMode_NoEffect')
            end,
        },
        Def.ActorFrame({
            OnCommand = function(self)
                self:zoom(0.9)
                self:ztest(true)
            end,
            W_AVATAR:Actor(avatar, id, params),
        }),
    })
end

-- ランダムアイコンは名前依存
local function GetRandomNum(self, text)
    if not text or string.sub(text, 0, 6) == 'Player' then
        return 0
    end
    return tonumber(string.byte(text))%9+1
end

-- アバター名を取得
local function GetAvatarName(self, path)
    local ini, avatarType, avatarFile = ChangeAvatar(path, 0)
    if avatarType == 'FileType_Ini' then
        return ini.main and ini.main.Name or ''
    end
    return ''
end

-- 各パーツの画像をすべて取得する
-- p1:パーツ名（eye, back等）
-- return {Group=グループフォルダ名, File=ファイル名, Name=拡張子を除いたファイル名}
local function GetParts(self, partName)
    local file = {}
    local path = GetPath(self)
    for _,group in pairs(FILEMAN:GetDirListing(path, true, false)) do
        -- userフォルダは保存フォルダなので除外
        if group ~= 'user' then
            -- アクセサリーだけ数字が付いてフォルダ名と一致しないので外す
            for _,v in pairs(FILEMAN:GetDirListing(path..group..'/'..string.gsub(partName, '%d+', '')..'/', false, false)) do
                file[#file+1] = {
                    Group = group,
                    File = v,
                    Name = string.gsub(v, '%.png', ''),
                    SortKey = (group == 'waiei') and '1' or '2'
                }
            end
        end
    end
    -- デフォルト画像のwaieiフォルダは最優先、それ以外は名前順にソート
    table.sort(file, function(a, b)
        return string.format('%s/%s/%s', a.SortKey, a.Group, a.File)<string.format('%s/%s/%s', b.SortKey, b.Group, b.File)
    end)
    -- ソートキーは不要なので削除
    for i=1, #file do
        file[i].SortKey = nil
    end
    return file
end

-- 指定したパーツの情報を取得
local function GetPartProperty(self, part, key)
    for i=1, #parts do
        if parts[i].Key == part then
            return parts[i][key]
        end
    end
    return nil
end

return {
    Path      = GetPath,
    Actor     = AvatarActor,
    Frame     = AvatarWithFrame,
    RandomNum = GetRandomNum,
    Name      = GetAvatarName,
    Parts     = GetParts,
    Property  = GetPartProperty,
}

--[[
Avatar.lua

Copyright (c) 2021 A.C

This software is released under the MIT License.
https://opensource.org/licenses/mit-license.php
--]]
