-- カスタム機能
-- 言語別テキスト
local loadedLangStr
local langStr = {}
local systemPath = THEME:GetCurrentThemeDirectory()..'Graphics/ScreenCustom/Customs/'

local global = {}

-- Customの情報を読み込み
local function LoadProperty(self, ...)
    local name = ...
    local property = {}
    name = name or self:Current()
    local file = self:Path('custom.ini', name)
    if not FILEMAN:DoesFileExist(file) then
        property = {}
        return nil
    end
    local ini = IniFile.ReadFile(file)
    local main = ini.Main or {}
    local lang = self:LoadLang(name)
    lang = lang and lang.Main or lang or {}

    property = {
        Name        = name,
        Title       = lang.Title or '',
        Description = string.gsub(lang.Description or '', '|', '\n'),
        Version     = main and main.Version or ini.Version or '1.0.0',
        Date        = main and main.Date    or ini.Date    or '',
        WAIEI       = main and main.WAIEI   or ini.WAIEI   or W:StrVer(),
    }
    return property
end

-- グローバル変数として保存
local function SetGlobal(self, key, val)
    global[key] = val
end

-- グローバル変数を読み込み
local function GetGlobal(self, key)
    return global[key]
end

-- 言語データを読み込み
local function LoadLanguage(self, ...)
    local dir = ...
    local file = self:Path('Languages/'..THEME:GetCurLanguage()..'.ini', dir)
    -- 現在の言語データが見つからない場合はen
    if not FILEMAN:DoesFileExist(file) then
        file = self:Path('Languages/en.ini', dir)
    end
    loadedLangStr = true
    -- enも無い場合は空
    if not FILEMAN:DoesFileExist(file) then
        langStr = {}
    end
    langStr = IniFile.ReadFile(file)
    return langStr
end

-- テキストを取得
local function GetString(self, section, key)
    if not loadedLangStr then
        LoadLanguage(self)
    end
    local text = ''
    if string.lower(section) ~= 'main' then
        text = langStr[section] and (langStr[section][key] or '')
    else
        text = langStr[key] or ''
    end
    text = string.gsub(text or '', '|', '\n')
    return text
end

-- 指定したCustomを開始
local function StartCustomScreen(self, name, ...)
    local system = ... -- boolean
    setenv('WAIEI_CUSTOM_NAME', name)
    setenv('WAIEI_CUSTOM_VALUE', {})
    -- システムに必須なCustomの場合Graphics/ScreenCustom/Customsに配置
    setenv('WAIEI_CUSTOM_SYSTEM', system or false)
    LoadLanguage(self)
    global = {}
    SCREENMAN:SetNewScreen('ScreenCustomExec')
end

-- 現在のCustom名を取得
local function GetCurrentCustomName(self)
    return getenv('WAIEI_CUSTOM_NAME') or ''
end

-- 現在のCustom名を終了
local function EndCustomScreen(self)
    setenv('WAIEI_CUSTOM_NAME', nil)
    setenv('WAIEI_CUSTOM_VALUE', nil)
    setenv('WAIEI_CUSTOM_SYSTEM', nil)
    global = {}
    SCREENMAN:SetNewScreen('ScreenCustom')
end

-- Customsフォルダのパス
local function GetCustomPath(self)
    return getenv('WAIEI_CUSTOM_SYSTEM', false)
        and systemPath
        or THEME:GetCurrentThemeDirectory()..'Customs/'
end

-- 現在のCustom内の任意のパス
local function GetPath(self, ...)
    local dir, name = ...
    name = name or self:Current()
    if name == '' then
        return self:CustomPath(self)..dir
    end
    return self:CustomPath()..name..'/'..dir
end

-- 現在のCustom内のパスをLoadActorする
local function LoadCustomActor(self, ...)
    local dir, name = ...
    return dofile(GetPath(self, dir, name))
end

-- 有効なカスタム一覧を取得
local function GetEnabledCustomList(self)
    local cList = {}
    for k,v in pairs(FILEMAN:GetDirListing(self:CustomPath(), true, false)) do
        local p = LoadProperty(self, v)
        if p then
            cList[#cList+1] = p
        end
    end
    table.sort(cList, function(a, b)
        return a.Title < b.Title
    end)
    return cList
end

-- カスタムで使える素材Actorを返却
local function LoadTemplateActor(self, name, ...)
    local path = THEME:GetCurrentThemeDirectory()..'Graphics/ScreenCustom/Templates/'..name;
    if FILEMAN:DoesFileExist(path..'.lua')
        or FILEMAN:DoesFileExist(path..'/default.lua')
        or W_GRAPHICS:GetFile(path) then
        return LoadActor(THEME:GetPathG('', 'ScreenCustom/Templates/'..name), ...)
    end
    return Def.ActorFrame({
        LoadFont('Common Small Normal')..{
            Text = string.format('Template "%s" is not found', name),
            OnCommand = function(self)
                self:zoom(0.5)
            end,
        },
    })
end

-- システム必須カスタムかどうか
local function IsSystemRequiredCuscom(self)
    return getenv('WAIEI_CUSTOM_SYSTEM', false)
end

-- 変数を取得
local function GetValue(self, key, ...)
    local v = getenv('WAIEI_CUSTOM_VALUE')
    if v and v[key] ~= nil then
        return v[key]
    end
    return ... or nil
end

-- 変数を設定
local function SetValue(self, key, value)
    local v = getenv('WAIEI_CUSTOM_VALUE') or {}
    v[key] = value
    setenv('WAIEI_CUSTOM_VALUE', v)
end

return {
    CustomList = GetEnabledCustomList,
    Start      = StartCustomScreen,
    End        = EndCustomScreen,
    Set        = SetGlobal,
    Get        = GetGlobal,
    Current    = GetCurrentCustomName,
    CustomPath = GetCustomPath,
    Path       = GetPath,
    LoadLang   = LoadLanguage,
    String     = GetString,
    Actor      = LoadCustomActor,
    Template   = LoadTemplateActor,
    IsSystem   = IsSystemRequiredCuscom,
    GV         = GetValue,
    SV         = SetValue,
}

--[[
Custom.lua

Copyright (c) 2021 A.C

This software is released under the MIT License.
https://opensource.org/licenses/mit-license.php
--]]
