local colorType
local nameType

local difColor = {
    DDR = {
        Difficulty_Beginner  = color('0.6, 1.0, 1.0, 1.0'),
        Difficulty_Easy      = color('1.0, 1.0, 0.6, 1.0'),
        Difficulty_Medium    = color('1.0, 0.6, 0.6, 1.0'),
        Difficulty_Hard      = color('0.7, 1.0, 0.6, 1.0'),
        Difficulty_Challenge = color('0.8, 0.6, 1.0, 1.0'),
        Difficulty_Edit      = color('0.85, 0.85, 0.9, 1.0'),
    },
    SM = {
        Difficulty_Beginner  = color('0.8, 0.6, 1.0, 1.0'),
        Difficulty_Easy      = color('0.7, 1.0, 0.6, 1.0'),
        Difficulty_Medium    = color('1.0, 1.0, 0.6, 1.0'),
        Difficulty_Hard      = color('1.0, 0.6, 0.6, 1.0'),
        Difficulty_Challenge = color('0.6, 1.0, 1.0, 1.0'),
        Difficulty_Edit      = color('0.85, 0.85, 0.9, 1.0'),
    },
}

local difName = {
    JP = {
        Difficulty_Beginner  = '習 - ★',
        Difficulty_Easy      = '楽 - ★★',
        Difficulty_Medium    = '踊 - ★★★',
        Difficulty_Hard      = '激 - ★★★★',
        Difficulty_Challenge = '鬼 - ★★★★',
        Difficulty_Edit      = '作 - ☆',
    },
    CLASSIC = {
        Difficulty_Beginner  = 'BEGINNER',
        Difficulty_Easy      = 'LIGHT',
        Difficulty_Medium    = 'STANDARD',
        Difficulty_Hard      = 'HEAVY',
        Difficulty_Challenge = 'CHALLENGE',
        Difficulty_Edit      = 'EDIT',
    },
    DDR = {
        Difficulty_Beginner  = 'BEGINNER',
        Difficulty_Easy      = 'BASIC',
        Difficulty_Medium    = 'DIFFICULT',
        Difficulty_Hard      = 'EXPERT',
        Difficulty_Challenge = 'CHALLENGE',
        Difficulty_Edit      = 'EDIT',
    },
    SM = {
        Difficulty_Beginner  = 'NOVICE',
        Difficulty_Easy      = 'EASY',
        Difficulty_Medium    = 'MEDIUM',
        Difficulty_Hard      = 'HARD',
        Difficulty_Challenge = 'EXPERT',
        Difficulty_Edit      = 'EDIT',
    },
}

local courseName = {
    Difficulty_Easy      = 'EASY',
    Difficulty_Medium    = 'NORMAL',
    Difficulty_Hard      = 'HARD',
}

-- 難易度一覧を取得
local function GetList(self)
    if not nameType then
        nameType = W_OPTION:ReadPref('DifficultyLabel', 'DDR')
    end
    return difName[nameType] or {}
end

-- 色を取得
local function GetColor(self, key)
    if not colorType then
        colorType = W_OPTION:ReadPref('DifficultyColor', 'DDR')
    end
    return difColor[colorType][key] or color('#FFFFFFFF')
end

-- 難易度名を取得
local function GetName(self, key, ...)
    if not nameType then
        nameType = W_OPTION:ReadPref('DifficultyLabel', 'DDR')
    end
    -- EDITでstepsを指定した場合はステップ名を取得
    local st = ...
    if st and key == 'Difficulty_Edit' then
        local editName = st:GetDescription()
        editName = (editName ~= '') and editName or st:GetChartName()
        return (editName ~= '') and editName or difName[nameType][key]
    end
    return difName[nameType][key] or ''
end

-- 色を設定
local function SetColorType(self, typeName)
    colorType = typeName
end

-- 難易度名を設定
local function SetNameType(self, typeName)
    nameType = typeName
end

-- 指定Trail、難易度の最小最大難易度を取得
-- 0の場合は「?」表記（enabledSecret = true）
-- return {Min, Max, Trail}
local function GetTrailDifficlutyMeterMinMax(self, trail, dif, ...)
    local enabledSecret = ...
    enabledSecret = (enabledSecret == nil) and true or enabledSecret
    local course = GAMESTATE:GetCurrentCourse() or nil
    local dList = nil
    local tEntry = trail and trail:GetTrailEntries() or {}
    if tEntry then
        local st = trail:GetStepsType()
        for k,v in pairs(course:GetAllTrails()) do
            if v:GetStepsType() == st and not dList and v:GetDifficulty() == dif then
                if enabledSecret and v:IsSecret() then
                    dList = {Min = 0, Max = 0, Trail = v}
                else
                    dList = {Min = 99999999, Max = 0, Trail = v}
                    for k,te in pairs(v:GetTrailEntries()) do
                        if dList.Max < te:GetSteps():GetMeter() then
                            dList.Max = te:GetSteps():GetMeter()
                        end
                        if dList.Min > te:GetSteps():GetMeter() then
                            dList.Min = te:GetSteps():GetMeter()
                        end
                    end
                end
            end
        end
    end
    return dList
end

-- コース難易度名を取得
local function GetCourseName(self, key)
    return courseName[key] or ''
end

-- 最後に選択されていた難易度
local function GetLatestSelectDifficulty(self)
    local pType = {
        PLAYER_1 = W_PLAYER:IsEnabled(PLAYER_1) and W_PLAYER:ProfileType(PLAYER_1) or 'ProfileType_Guest',
        PLAYER_2 = W_PLAYER:IsEnabled(PLAYER_2) and W_PLAYER:ProfileType(PLAYER_2) or 'ProfileType_Guest',
    }
    local pMain
    -- 片方がゲストでもう片方がPROFILEの場合PROFILEを優先
    if pType[PLAYER_1] ~= 'ProfileType_Guest' and pType[PLAYER_2] == 'ProfileType_Guest' then
        pMain = PLAYER_1
    elseif pType[PLAYER_1] == 'ProfileType_Guest' and pType[PLAYER_2] ~= 'ProfileType_Guest' then
        pMain = PLAYER_2
    -- 両方PROFILEの場合、MasterPlayerを優先
    elseif pType[PLAYER_1] ~= 'ProfileType_Guest' and pType[PLAYER_2] ~= 'ProfileType_Guest' then
        pMain = GAMESTATE:GetMasterPlayerNumber()
    end
    -- 両方ゲストの場合、Easy
    if not pMain then
        return 'easy'
    end
    -- XMLから取得する
    local path = PROFILEMAN:GetProfileDir('ProfileSlot_Player'..W:PNum(pMain))..'Stats.xml'
    if not FILEMAN:DoesFileExist(path) then
        return 'easy'
    end
    local val, match = '', ''
    local f = RageFileUtil.CreateRageFile()
    f:Open(path, 1)
    while(true) do
        if f:AtEOF() then
            break
        end
        val = f:GetLine()
        match = string.match(val, '<LastDifficulty>([^<]+)</LastDifficulty>')
        if match then
            match = string.lower(match)
            break
        end
    end
    f:Close()
    f:destroy()
    if match then
        for _,v in pairs({'beginner', 'easy', 'medium', 'hard', 'challenge', 'edit'}) do
            if match == v then
                return v
            end
        end
    end
    return 'easy'
end

return {
    Color        = GetColor,
    Name         = GetName,
    ColorType    = SetColorType,
    NameType     = SetNameType,
    Course       = GetCourseName,
    TrailMinMax  = GetTrailDifficlutyMeterMinMax,
    List         = GetList,
    LatestSelect = GetLatestSelectDifficulty,
}

--[[
Difficulty.lua

Copyright (c) 2021 A.C

This software is released under the MIT License.
https://opensource.org/licenses/mit-license.php
--]]
