local scores = {Meta = {}, Data = {}}
local filePath = THEME:GetCurrentThemeDirectory()..'./Other/NetScore.lua'
-- サーバー負荷軽減のための待機時間
local defaultTime = {
    -- カーソルを合わせてから接続までの待機時間
    Wait = 0.5,
    -- サーバーからデータ取得してから再取得処理をしない時間
    Reacquisition = 60.0 * 5,
}

local function LoadScore(self)
    if not NETMAN or not W:IsOFV() then
        return
    end
    -- ローカルに保存しているデータの読み込み
    if FILEMAN:DoesFileExist(filePath) then
        local luaData = ''
        local file = RageFileUtil:CreateRageFile()
        file:Open(filePath, 1)
        luaData = file:Read()
        file:Close()
        file:destroy()
        local luaString, errMsg
        -- loadstringはLua5.2以降廃止
        -- assertでエラーをキャッチするとluaDataの中身が出力されるのでここではキャッチしない
        if _VERSION == 'Lua 5.1' then
            scores, errMsg = loadstring(luaData)()
        else
            scores, errMsg = load(luaData)()
        end
        -- エラー発生時に変数の中身が出力されてもメッセージが流れないようにクリア
        luaData = nil
        if not luaString and errMsg then
            error('NetScore ERROR : '..errMsg, 0)
            return
        end
    end
    if not scores.Meta then scores.Meta = {} end
    if not scores.Data then scores.Data = {} end
end

local function SaveScore(self)
    if not NETMAN or not W:IsOFV() then
        return
    end
    local meta = 'Theme='..W:Ver()
    local data = ''
    for k,v in pairs(scores.Data or {}) do
        if W:DateNumber() - v.access_date <= defaultTime.Reacquisition then
            data = data..'\n'..k..'={\n'
            data = data..'access_date='..v.access_date..',\nscores={'
            for _,score in pairs(v.scores or {}) do
                data = string.format(
                    "%s\n{\nusername='%s',\ndp_actual=%s,\ndp_max=%s,\n},",
                    data,
                    score.username,
                    score.dp_actual,
                    score.dp_max
                )
            end
            data = data..'\n}\n},'
        end
    end
    local file = RageFileUtil:CreateRageFile()
    file:Open(filePath, 2)
    file:Write(string.format('return {\nMeta={\n%s\n},\nData={%s\n},\n}', meta, data))
    file:Close()
    file:destroy()
end

local function GetScoreActor(self, ...)
    local time = ... or defaultTime
    local reacquisition = time.Reacquisition or (defaultTime.Reacquisition)
    if not NETMAN or not W:IsOFV() or not W_NET:IsConnected() then return Def.Actor({}) end

    return Def.Actor({
        UpdateStepsMessageCommand = function(self, params)
            self:finishtweening()
            if params.Steps then
                self:queuecommand('NetLoading')
                local chartKey = params.Steps:GetChartKey()
                if scores.Data[chartKey] and W:DateNumber() - (scores.Data[chartKey].access_date or 0) < reacquisition then
                    local s = {}
                    for k,v in pairs(scores.Data[chartKey]) do
                        s[k] = v
                    end
                    s.temp = true
                    self:playcommand('ResponseScore', {
                        Player = params.Player,
                        ChartKey = chartKey,
                        Data = s,
                    })
                    return
                end
                -- カーソルを合わせてから接続までの待機時間
                self:sleep(time.Wait or defaultTime.Wait)
                self.Request = {
                    Player = params.Player,
                    ChartKey = chartKey,
                    TimingName = 'DDR',
                    Rate = 1.0
                }
                -- 負荷軽減のため0.5秒まつこと
                self:queuecommand('RequestHighScoresForChart')
            end
        end,
        RequestHighScoresForChartCommand = function(self)
            local params = self.Request
            local chartKey = params.ChartKey
            local timing = params.TimingName or 'DDR'
            local rate = params.Rate or 1.0
            NETMAN:HighScoresForChart(chartKey, timing, rate)
        end,
        ResponseHighScoresForChartMessageCommand = function(self, params)
            local chartKey = self.Request.ChartKey
            local meta = (params and params.Response) and params.Response.metadata or nil
            if not meta or (meta.chart_key and meta.chart_key ~= chartKey) then
                self:playcommand('ResponseScore', {
                    Player = self.Request.Player,
                    ChartKey = meta and meta.chart_key or '',
                    Data = {},
                })
                return
            end
            if not meta.num_results or meta.num_results <= 0 then
                -- スコアは存在しているのに何故か空返却されてしまった場合は無視する
                if scores.Data[chartKey] and #(scores.Data[chartKey].scores or {}) > 0 then
                    return
                end
                scores.Data[chartKey] = {
                    temp = false,
                    access_date = W:DateNumber(),
                }
                self:playcommand('ResponseScore', {
                    Player = self.Request.Player,
                    ChartKey = chartKey,
                    Data = scores.Data[chartKey],
                })
                return
            end
            scores.Data[chartKey]={scores = {}}
            local s = {}
            for _,v in pairs(params.Response.scores or {}) do
                s[#s+1] ={
                    username = v.username,
                    dp_actual = v.dp_actual,
                    dp_max = v.dp_max
                }
            end
            scores.Data[chartKey] = {
                temp = false,
                access_date = W:DateNumber(),
                scores = s,
            }
            self:playcommand('ResponseScore', {
                Player = self.Request.Player,
                ChartKey = chartKey,
                Data = scores.Data[chartKey],
            })
        end,
    })
end

return {
    Load = LoadScore,
    Save = SaveScore,
    GetScoreActor = GetScoreActor,
    IsConnected = function(self) return IsNetSMOnline() or (W:IsOFV() and NETMAN:IsConnectionEstablished()) end
}

--[[
Net.lua

Copyright (c) 2023 A.C

This software is released under the MIT License.
https://opensource.org/licenses/mit-license.php
--]]
