local function GetTapNoteScores(...)
    local score, pss = ...
    return score
        and {
            TapNoteScore_Miss = score:GetTapNoteScore('TapNoteScore_Miss'),
            TapNoteScore_W5 = score:GetTapNoteScore('TapNoteScore_W5'),
            TapNoteScore_W4 = score:GetTapNoteScore('TapNoteScore_W4'),
            TapNoteScore_W3 = score:GetTapNoteScore('TapNoteScore_W3'),
            TapNoteScore_W2 = score:GetTapNoteScore('TapNoteScore_W2'),
            TapNoteScore_W1 = score:GetTapNoteScore('TapNoteScore_W1'),
        }
        or {
            TapNoteScore_Miss = pss:GetTapNoteScores('TapNoteScore_Miss'),
            TapNoteScore_W5 = pss:GetTapNoteScores('TapNoteScore_W5'),
            TapNoteScore_W4 = pss:GetTapNoteScores('TapNoteScore_W4'),
            TapNoteScore_W3 = pss:GetTapNoteScores('TapNoteScore_W3'),
            TapNoteScore_W2 = pss:GetTapNoteScores('TapNoteScore_W2'),
            TapNoteScore_W1 = pss:GetTapNoteScores('TapNoteScore_W1'),
        }
end

-- ノートを減らすオプションを使用している
local function IsUseCutArrows(radar, ...)
    local score, pss = ...
    local max = radar:GetValue('RadarCategory_TapsAndHolds')
    local total = 0
    for _, v in pairs(GetTapNoteScores(score, pss)) do
        total = total + v
    end
    return total < max or (score and (score:GetMaxCombo() < max) or false)
end

-- 指定TapNoteScoreよりも低い判定が存在する
local function HasLowerThanTapNoteScore(tapNoteScore, ...)
    local score, pss = ...
    local noteScores = GetTapNoteScores(score, pss)
    local threshold = Enum.Reverse(TapNoteScore)[tapNoteScore] or 7 -- デフォルトW3
    -- Miss～指定判定一つ下までの値をチェックして1以上があればtrue
    for i = Enum.Reverse(TapNoteScore).TapNoteScore_Miss + 1, threshold do
        if noteScores[TapNoteScore[i]] and noteScores[TapNoteScore[i]] > 0 then
            return true
        end
    end
    return false
end

-- フルコンボカラーを取得
local function GetFullComboColor(self, pn, ...)
    local pss = ...
    if not GAMESTATE:IsPlayerEnabled(pn) then
        return nil
    end
    pss = pss or STATSMAN:GetCurStageStats():GetPlayerStageStats(pn)
    if pss:GetFailed()
        or pss:MaxCombo() <= 0
        or pss:GetTapNoteScores('TapNoteScore_Miss') > 0
        or pss:GetTapNoteScores('TapNoteScore_HitMine') > 0
        or pss:GetTapNoteScores('TapNoteScore_CheckpointMiss') > 0
        or pss:GetHoldNoteScores('HoldNoteScore_LetGo') > 0
        or pss:GetHoldNoteScores('HoldNoteScore_MissedHold') > 0 then
        return nil
    end
    -- ノートを減らすオプションを使用している
    local stepsOrTrail = GAMESTATE:IsCourseMode() and GAMESTATE:GetCurrentTrail(pn) or GAMESTATE:GetCurrentSteps(pn)
    if IsUseCutArrows(stepsOrTrail:GetRadarValues(pn), nil, pss) then
        return nil
    end
    if HasLowerThanTapNoteScore(THEME:GetMetric('Gameplay', 'MinScoreToContinueCombo'), nil, pss) then
        return nil
    end
    local best = pss:GetBestFullComboTapNoteScore()
    if best
        and (best == 'TapNoteScore_W1'
            or best == 'TapNoteScore_W2'
            or best == 'TapNoteScore_W3'
            or best == 'TapNoteScore_W4') then
            return W_JUDGMENT:Color(best)
    end
    return nil
end

-- HighScoreからフルコンボ状態を取得（0はフルコンじゃない、1～4はW1～W4に相当）
local function GetScoreFullCombo(self, score, radar)
    if score:GetMaxCombo() < 1 then return 0 end
    local missCount = score:GetTapNoteScore('TapNoteScore_W5')
                    + score:GetTapNoteScore('TapNoteScore_Miss')
                    + score:GetTapNoteScore('TapNoteScore_HitMine')
                    + score:GetHoldNoteScore('HoldNoteScore_LetGo')
    if (missCount > 0) then
        return 0
    end
    if IsUseCutArrows(radar, score) then
        return 0
    end
    if HasLowerThanTapNoteScore(THEME:GetMetric('Gameplay', 'MinScoreToContinueCombo'), score) then
        return 0
    end
    local noteScores = {
        W4 = score:GetTapNoteScore('TapNoteScore_W4'),
        W3 = score:GetTapNoteScore('TapNoteScore_W3'),
        W2 = score:GetTapNoteScore('TapNoteScore_W2'),
        W1 = score:GetTapNoteScore('TapNoteScore_W1'),
    }
    -- そもそもノート数が足りていない
    if noteScores['W4'] + noteScores['W3'] + noteScores['W2'] + noteScores['W1'] < radar:GetValue('RadarCategory_TapsAndHolds') then
        return 0
    end
    local point = missCount + noteScores['W4'] + noteScores['W3'] + noteScores['W2'] + noteScores['W1']
    -- 上位判定から数を引いていき、0以下になった時点でそれより下位の判定は無く、フルコンボとして考える
    for i=1, 4 do
        point = point-noteScores['W'..i]
        if point <= 0 then
            return i
        end
    end
    return 0
end

-- 現在の楽曲・コースを全て最高判定でプレイした時のDP(%ではなく数値)
local function GetPlayerMaxDPScore(self, pn)
    -- タップ、ホールド、チェックポイントの最高得点
    local maxWeight = {
        TapNoteScore     = tonumber(THEME:GetMetric('ScoreKeeperNormal', 'PercentScoreWeightW1') or 3),
        HoldNoteScore    = tonumber(THEME:GetMetric('ScoreKeeperNormal', 'PercentScoreWeightHeld') or 3),
        CheckpointScore  = tonumber(THEME:GetMetric('ScoreKeeperNormal', 'PercentScoreWeightCheckpointHit') or 3),
    }
    local radarValue = {}
    if GAMESTATE:IsCourseMode() then
        radarValue = {
            RadarCategory_TapsAndHolds = 0,
            RadarCategory_Holds        = 0,
            RadarCategory_Rolls        = 0,
        }
        -- TrailのGetRadarValuesだとなぜか-1しか返却されないのでStepsから取得
        for i,k in pairs(GAMESTATE:GetCurrentTrail(pn):GetTrailEntries()) do
            local radar = k:GetSteps():GetRadarValues(pn)
            radarValue = {
                RadarCategory_TapsAndHolds = radarValue.RadarCategory_TapsAndHolds + radar:GetValue('RadarCategory_TapsAndHolds'),
                RadarCategory_Holds        = radarValue.RadarCategory_Holds + radar:GetValue('RadarCategory_Holds'),
                RadarCategory_Rolls        = radarValue.RadarCategory_Rolls + radar:GetValue('RadarCategory_Rolls'),
            }
        end
    else
        local radar = GAMESTATE:GetCurrentSteps(pn):GetRadarValues(pn)
        radarValue = {
            RadarCategory_TapsAndHolds = radar:GetValue('RadarCategory_TapsAndHolds'),
            RadarCategory_Holds        = radar:GetValue('RadarCategory_Holds'),
            RadarCategory_Rolls        = radar:GetValue('RadarCategory_Rolls'),
        }
    end
    return math.max(radarValue.RadarCategory_TapsAndHolds * maxWeight.TapNoteScore
                + radarValue.RadarCategory_Holds * maxWeight.HoldNoteScore
                + radarValue.RadarCategory_Rolls * maxWeight.HoldNoteScore, 1)
end

-- 現在の楽曲（コース）のハイスコアを取得
-- return：ハイスコア、現在のスコア、未プレイかどうか
local function GetHighDPScore(self, pn, ...)
    local isEvaluation = ...
    isEvaluation = isEvaluation == nil and true or isEvaluation
    local pss = STATSMAN:GetCurStageStats():GetPlayerStageStats(pn)
    local profile = GetPlayerOrMachineProfile(pn)
    local hiscoreList = GAMESTATE:IsCourseMode()
                            and profile:GetHighScoreListIfExists(GAMESTATE:GetCurrentCourse(), GAMESTATE:GetCurrentTrail(pn))
                            or profile:GetHighScoreListIfExists(W:Song(), W:Steps(pn))
    local curPercent = (pss:GetPercentDancePoints() or 0) * 100
    local scoreList = hiscoreList and hiscoreList:GetHighScores() or nil
    local hiPercent = -1
    if scoreList and #scoreList > 0 then
        local curIsNewRecordCheck = not isEvaluation
        for i=1, #scoreList do
            local taegetPercent = (scoreList[i]:GetPercentDP() or 0) * 100
            if round(curPercent*100) ~= round(taegetPercent*100) or curIsNewRecordCheck then
                if taegetPercent > hiPercent then
                    hiPercent = taegetPercent
                end
            else
                -- リザルトかつ現在と同じスコアの場合、自身の可能性があるので一回飛ばす
                -- もし、同値の場合はもう1つ同じスコアのハイスコアが存在するはず
                curIsNewRecordCheck = true
            end
        end
    end

    return math.max(hiPercent, 0), curPercent, hiPercent < 0
end

-- グレード設定を一時変数に記録
local gradeType
local function GetGradeType(self)
    return gradeType or W_OPTION:ReadPref('GradeType', 'StepMania')
end
local function SetGradeType(self, value)
    gradeType = value
end
local function GetDDRAScoreByHighScore(self, highscore, radar)
    local tapAndHold = radar:GetValue('RadarCategory_TapsAndHolds')
    local maxScore = math.max(
        tapAndHold
        + radar:GetValue('RadarCategory_Holds')
        + radar:GetValue('RadarCategory_Rolls'), 1)
    local w1 = highscore:GetTapNoteScore('TapNoteScore_W1')
    local w2 = highscore:GetTapNoteScore('TapNoteScore_W2')
    local w3 = highscore:GetTapNoteScore('TapNoteScore_W3')
    local w4 = highscore:GetTapNoteScore('TapNoteScore_W4')
    local hd = highscore:GetHoldNoteScore('HoldNoteScore_Held')
    local missCount = highscore:GetTapNoteScore('TapNoteScore_W5')
                    + highscore:GetTapNoteScore('TapNoteScore_Miss')
                    + highscore:GetTapNoteScore('TapNoteScore_HitMine')
                    + highscore:GetHoldNoteScore('HoldNoteScore_LetGo')
    local isW1FC = w1 >= tapAndHold and missCount <= 0
    local isW2FC = w1+w2 >= tapAndHold and missCount <= 0
    return (math.round((w1 + w2 + w3*0.6 + w4*0.2 + hd) * 100000 / maxScore - (w2 + w3 + w4))*10), isW1FC, isW2FC
end

local function GetDDRAGradeByScore(score, isW1FC, isW2FC)
    if isW1FC then
        return 'Grade_Tier01'
    elseif isW2FC then
        return 'Grade_Tier02'
    elseif score >= 990000 then
        return 'Grade_A3n'
    elseif score >= 950000 then
        return 'Grade_A2p'
    elseif score >= 900000 then
        return 'Grade_Tier03'
    elseif score >= 890000 then
        return 'Grade_A2m'
    elseif score >= 850000 then
        return 'Grade_A1p'
    elseif score >= 800000 then
        return 'Grade_Tier04'
    elseif score >= 790000 then
        return 'Grade_A1m'
    elseif score >= 750000 then
        return 'Grade_Bp'
    elseif score >= 700000 then
        return 'Grade_Tier05'
    elseif score >= 690000 then
        return 'Grade_Bm'
    elseif score >= 650000 then
        return 'Grade_Cp'
    elseif score >= 600000 then
        return 'Grade_Tier06'
    elseif score >= 590000 then
        return 'Grade_Cm'
    elseif score >= 550000 then
        return 'Grade_Dp'
    end
    return 'Grade_Tier07'
end

-- グレードを取得
local function GetGrade(self, pss)
    gradeType = GetGradeType(self)
    if gradeType == 'DDR A' then
        if pss:GetGrade() == 'Grade_Failed' then
            return 'Grade_Failed'
        else
            return GetDDRAGradeByScore(GetDDRAScoreByHighScore(self, pss:GetHighScore(), pss:GetRadarPossible()))
        end
    end
    return pss:GetGrade()
end

    -- ハイスコアリストから最高グレードを取得
local function GetBestGrade(self, highscores, radar)
    gradeType = GetGradeType(self)
    local best = nil
    local bestGrade = nil
    for h=1, #highscores do
        if gradeType == 'StepMania' then
            local grade = Enum.Reverse(Grade)[highscores[h]:GetGrade()]
            if not best or best > grade then
                best = grade
                bestGrade = highscores[h]:GetGrade()
            end
        elseif gradeType == 'DDR A' then
            local score, isW1FC, isW2FC = GetDDRAScoreByHighScore(self, highscores[h], radar)
            if not best or best < score then
                best = score
                if highscores[h]:GetGrade() == 'Grade_Failed' then
                    bestGrade = 'Grade_Failed'
                else
                    bestGrade = GetDDRAGradeByScore(score, isW1FC, isW2FC)
                end
            end
        end
    end
    return bestGrade
end

return {
    -- PlayerStageStateからフルコンボ状態を色で取得
    FCColor       = GetFullComboColor,
    -- 指定したHighscoreからフルコンボ状態を取得
    ScoreFC       = GetScoreFullCombo,
    -- プレイヤーのハイスコアを取得（リザルトフラグが有効の場合は更新前のハイスコアを取得）
    HighDPScore   = GetHighDPScore,
    -- 現在の楽曲・コースを全て最高判定でプレイした時のDP(%ではなく数値)
    MaxDPScore    = GetPlayerMaxDPScore,
    -- グレードを取得
    GetGrade = GetGrade,
    -- ハイスコアリストから最高グレードを取得
    GetBestGrade = GetBestGrade,
    -- グレード設定を一時変数に記録
    GetGradeType = GetGradeType,
    SetGradeType = SetGradeType,
    -- HighScoreからDDRAのスコアを取得
    DDRAScoreByHS = GetDDRAScoreByHighScore,
}

--[[
Player.lua

Copyright (c) 2021 A.C

This software is released under the MIT License.
https://opensource.org/licenses/mit-license.php
--]]
