-- スコア比重
local scoreList = {
    TapNoteScore_W1 = {},
    TapNoteScore_W2 = {},
    TapNoteScore_W3 = {},
    TapNoteScore_W4 = {},
    TapNoteScore_W5 = {},
    TapNoteScore_Miss = {},
    HoldNoteScore_Held = {},
    HoldNoteScore_LetGo = {},
    HoldNoteScore_MissedHold = {},
    TapNoteScore_HitMine = {},
    TapNoteScore_CheckpointHit = {},
    TapNoteScore_CheckpointMiss = {},
}
for k,v in pairs(scoreList) do
    local short = ToEnumShortString(k)
    scoreList[k] = {
        Percent = tonumber(THEME:GetMetric('ScoreKeeperNormal', 'PercentScoreWeight'..short)),
        Grade   = tonumber(THEME:GetMetric('ScoreKeeperNormal', 'GradeWeight'..short)),
    }
end

-- ターゲットスコア
local targetPath = '/CSRealScore/'

-- ターゲットスコア一覧を取得
local function GetList(self, ...)
    local sort = ... or false
    local targetList = {}
    local dirList = FILEMAN:GetDirListing(targetPath, true, false)
    for i=1, #dirList do
        if string.match(dirList[i], '^[^_]+_.+$') then
            targetList[#targetList+1] = dirList[i]
        end
    end
    if sort then
        table.sort(targetList, function(a, b)
            return W_TARGET:Name(a) < W_TARGET:Name(b)
        end)
    end
    local returnList = {
        '*_HighScore',
    }
    for k,v in pairs(targetList) do
        returnList[#returnList+1] = v
    end
    return returnList
end

-- ハイスコア等特殊なターゲットか
local function IsSystem(self, id)
    local ret = split('_', id)
    if ret[1] ~= '*' then
        return false, nil
    end
    return true, ret[2]
end

-- 識別IDからプレイヤー名を取得
local function Id2Name(self, id)
    return string.match(id, '^[^_]+_(.+)$')
end

-- 自分のID
local function MyId(self, pn)
    local name = W_PLAYER:Name(pn)
    if not name or name == '' then
        return ''
    end
    return string.format('%s_%s', PROFILEMAN:GetProfile(pn):GetGUID(), name)
end

-- プレイヤーデータ内のファイルを取得
local function GetPath(self, id, path)
    local file = W_GRAPHICS:GetFile(targetPath..id..'/'..path, W_GRAPHICS:TypeList({'', 'ini'}))
    return file
end

-- プレイヤーデータのアバターファイルを取得
local function GetAvatar(self, id)
    -- WAIEI2式のwaieiAvatarは非対応
    local path = GetPath(self, id, 'CitMinAvatar')
    if not path then
        path = GetPath(self, id, 'CSAvatar')
    end
    if not path then
        path = GetPath(self, id, 'icon')
    end
    return path
end

-- 差分から優勢か同点、劣勢判定を行う
-- p1：差分
-- return：-1, 0, 1（優勢、同点、劣勢）
local function ConvertDiffToIsInTheLead(self, diff)
    local intDiff = math.ceil(diff*100)
    return (intDiff >= 1) and -1 or ((intDiff <= -1) and 1 or 0)
end

local function GetCurrentSongScoreFileName()
    -- set_DPの重み_Gradeの重み
    local dpWeight =  {
        scoreList.TapNoteScore_W1.Percent,
        scoreList.TapNoteScore_W2.Percent,
        scoreList.TapNoteScore_W3.Percent,
        scoreList.TapNoteScore_W4.Percent,
        scoreList.TapNoteScore_W5.Percent,
        scoreList.TapNoteScore_Miss.Percent,
        3,    -- HoldNoteScore_Held...CSだと値が固定されているので互換性維持
        scoreList.HoldNoteScore_LetGo.Percent,
              -- PercentScoreWeightMissedHold...CSだとJOIN対象外
        scoreList.TapNoteScore_HitMine.Percent,
        0,    -- TapNoteScore_AvoidMine...CSだと値が固定されているので互換性維持
        scoreList.TapNoteScore_CheckpointHit.Percent,
        scoreList.TapNoteScore_CheckpointMiss.Percent,
    }
    
    local gradeWeight =  {
        scoreList.TapNoteScore_W1.Grade,
        scoreList.TapNoteScore_W2.Grade,
        scoreList.TapNoteScore_W3.Grade,
        scoreList.TapNoteScore_W4.Grade,
        scoreList.TapNoteScore_W5.Grade,
        scoreList.TapNoteScore_Miss.Grade,
        6,    -- HoldNoteScore_Held...CSだと値が固定されているので互換性維持
        scoreList.HoldNoteScore_LetGo.Grade,
              -- PercentScoreWeightMissedHold...CSだとJOIN対象外
        scoreList.TapNoteScore_HitMine.Grade,
        0,    -- TapNoteScore_AvoidMine...CSだと値が固定されているので互換性維持
        scoreList.TapNoteScore_CheckpointHit.Grade,
        scoreList.TapNoteScore_CheckpointMiss.Grade,
    };

    return 'set_'..join('', dpWeight)..'_'..join('', gradeWeight)
end

-- 読みこむファイルパス
-- CSライバルの命名ルール
local function GetFilePath(self, id)
    local name = GetCurrentSongScoreFileName()
    local song = GAMESTATE:GetCurrentSong()
    local dirName = string.match(song:GetSongDir(), '^.+/([^/]+/)$')

    -- OriginalName優先
    return GetPath(self, id, 'Songs/'..YA_GROUP:OriginalName(song)..'/'..dirName..name)
        or GetPath(self, id, 'Songs/'..song:GetGroupName()..'/'..dirName..name)
end

-- 対象ライバルのスコア取得
-- return：Score(%), Score(DP), IsPlay
local function GetCurrentScore(self, id, pn)
    -- TODO
    if GAMESTATE:IsCourseMode() then return 0, 0, true end
    local steps = GAMESTATE:GetCurrentSteps(pn)
    -- エディットまたは自動生成は対象外
    if steps:IsAnEdit() or steps:IsAPlayerEdit() or steps:IsAutogen() then
        return 0, 0, true
    end
    local file = GetFilePath(self, id)
    if not file then return 0, 0, true end
    local maxDP, targetDP = W_SCORE:MaxDPScore(pn), 0
    local st, dif = steps:GetStepsType(), ToEnumShortString(steps:GetDifficulty())
    local hasScore = false
    local f = RageFileUtil.CreateRageFile()
    if f:Open(file, 1) then
        while true do
            if f:AtEOF() then
                break
            end
            local line = f:GetLine()
            if string.match(line, '^#'..st..'_'..dif..'/CS:.+') then
                hasScore = true
                local data = split(':', line)
                targetDP = tonumber(string.match(data[4], '(%d+).*$') or 0)
                break
            end
        end
        f:Close()
        f:destroy()
    end
    -- NO PLAY
    if not hasScore then
        return 0, 0, true
    end
    return targetDP*100/maxDP, targetDP, false
end

-- スコアを保存
local function SaveScore(self, pn, ...)
    local pss = ...
    pss = pss or STATSMAN:GetCurStageStats():GetPlayerStageStats(pn)
    -- TODO
    if GAMESTATE:IsCourseMode() then return end
    -- リザルト突入時のBPMが異常に高いとフリーズを起こすので下記変数に入れてSetCurrentSongでnilにしている
    local playedSong = getenv('WAIEI_PlayedSong', {})
    local song = playedSong.Song or nil
    local steps = playedSong.Steps[pn] or nil
    if not song or not steps then return end
    -- エディットまたは自動生成は対象外
    if steps:IsAnEdit() or steps:IsAPlayerEdit() or steps:IsAutogen() then
        return
    end
    local function GetFullCombo(pss)
        local fc = 5
        if pss and pss:GetFailed() then
            return fc
        end
        local best = pss:GetBestFullComboTapNoteScore()
        local bestList = {
            TapNoteScore_W1 = 1,
            TapNoteScore_W2 = 2,
            TapNoteScore_W3 = 3,
            TapNoteScore_W4 = 4,
        }
        fc = bestList[best] or fc
        return fc
    end
    local function GetGrade(pss, mode)
        local grade = pss:GetGrade()
        if grade == 'Grade_Failed' then
            grade = 'Grade_Tier21'
        end
        return grade
    end
    local function GetJudgmentsString(pss)
        local list = {
            'TapNoteScore_W1',
            'TapNoteScore_W2',
            'TapNoteScore_W3',
            'TapNoteScore_W4',
            'TapNoteScore_W5',
            'TapNoteScore_Miss',
            'HoldNoteScore_Held',
            'HoldNoteScore_LetGo',
        }
        local judgmentList = {}
        for i=1, #list do
            judgmentList[#judgmentList+1] = (split('_', list[i])[1] == 'TapNoteScore')
                    and pss:GetTapNoteScores(list[i])
                    or pss:GetHoldNoteScores(list[i])
        end
        return join(',', judgmentList)
    end
    local function GetSaveScore(pss, mode)
        if mode == 'CS' then
            return pss:GetActualDancePoints()
        end
        local list = {
            'TapNoteScore_W1',
            'TapNoteScore_W2',
            'TapNoteScore_W3',
            'TapNoteScore_W4',
            'TapNoteScore_W5',
            'TapNoteScore_Miss',
            'HoldNoteScore_Held',
            'HoldNoteScore_LetGo',
            'TapNoteScore_HitMine',
        }
        local migs = 0
        for i=1, #list do
            local add = (split('_', list[i])[1] == 'TapNoteScore')
                    and pss:GetTapNoteScores(list[i])
                    or pss:GetHoldNoteScores(list[i])
            migs = migs + (add * scoreList[list[i]].Grade)
        end
        return migs
    end
    local st, dif = steps:GetStepsType(), ToEnumShortString(steps:GetDifficulty())
    -- データ更新かどうかチェック
    local dirName = string.match(song:GetSongDir(), '^.+/([^/]+/)$')
    local file = string.format('%s%s/Songs/%s/%s%s',
                                    targetPath,
                                    MyId(self, pn),
                                    YA_GROUP:OriginalName(song),
                                    dirName,
                                    GetCurrentSongScoreFileName())
    local f
    local isUpdate = false
    local hasThemeName = false
    local hasScore = false
    local output = ''
    if FILEMAN:DoesFileExist(file) then
        f = RageFileUtil.CreateRageFile()
        if f:Open(file, 1) then
            while true do
                if f:AtEOF() then
                    break
                end
                local line = f:GetLine()
                local themeLine = string.match(line, '^#Theme:.+')
                if themeLine or (not hasThemeName and string.match(line, '^#StepsType.+')) then
                    -- 出力テーマ
                    output = output..'#Theme:'..W:Name()..';\n'
                    hasThemeName = true
                end
                if string.match(line, '^#'..st..'_'..dif..'/SM:.+') then
                    -- 更新対象
                elseif string.match(line, '^#'..st..'_'..dif..'/CS:.+') then
                    -- スコアを更新したか判定
                    local data = split(':', line)
                    isUpdate = pss:GetActualDancePoints() >= tonumber(string.match(data[4], '(%d+).*$') or 0)
                    hasScore = true
                elseif line ~= '' and not themeLine then
                    output = output..line..'\n'
                end
            end
            f:Close()
        end
        f:destroy()
        f = nil
        if not hasScore then
            isUpdate = true
        end
    else
        isUpdate = true
        output = output..string.format('#Prof:%s:%s;\n', PROFILEMAN:GetProfile(pn):GetGUID(), W_PLAYER:Name(pn))
        output = output..string.format('#SongDir:%s;\n', GAMESTATE:GetCurrentSong():GetSongDir())
        output = output..'#Theme:'..W:Name()..';\n'
    end
    if not isUpdate then return end
    -- 更新して保存
    local date = string.format('%04i-%02i-%02i_%02i-%02i', Year(), MonthOfYear()+1, DayOfMonth(), Hour(), Minute())
    for k,v in pairs({'CS', 'SM'}) do
        output = output..string.format('#%s_%s/%s:%s:', st, dif, v, date)
        output = output..string.format('%s/%s:%s:x:', GetGrade(pss, v), GetFullCombo(pss), GetSaveScore(pss, v))
        output = output..string.format('%s:%s,%s;\n', GetJudgmentsString(pss), GetLifeDifficulty(), GetTimingDifficulty())
    end
    f = RageFileUtil.CreateRageFile()
    if f:Open(file, 2) then
        f:Write(output)
        f:Close()
        f:destroy()
    end
end

return {
    List       = GetList,
    MyId       = MyId,
    IsSystem   = IsSystem,
    Name       = Id2Name,
    Path       = GetPath,
    DiffToLead = ConvertDiffToIsInTheLead,
    Avatar     = GetAvatar,
    File       = GetFilePath,
    GetScore   = GetCurrentScore,
    SaveScore  = SaveScore,
}

--[[
Target.lua

Copyright (c) 2021 A.C

This software is released under the MIT License.
https://opensource.org/licenses/mit-license.php
--]]
