-- sleepやアニメーション時間定義
local MIN_TIME = 2.0

-- ゲーム開始時の上下黒帯表示時間
local function GameStartSleep(self)
    local isEdit = W_PLAYER:HasEdit()
    local firstSecond = W:Song() and W:Song():GetFirstSecond() or 0
    local slow = (W_OPTION:ReadPref('StartMusicTiming') == 'Slowly') and MIN_TIME or 0
    return isEdit and 0.3 or math.min(math.max(firstSecond - 1.0 + slow, 0.3), 1.0)
end

-- ゲーム開始時のジャケット表示時間
local function GameJacketSleep(self)
    local isEdit = W_PLAYER:HasEdit()
    local firstSecond = W:Song() and W:Song():GetFirstSecond() or 0
    local slow = (W_OPTION:ReadPref('StartMusicTiming') == 'Slowly') and MIN_TIME or 0
    local add = isEdit and 0 or math.max(firstSecond - 4.0 + slow, 0.5)
    local time = GameStartSleep() + add
    -- ジャケットが消えてから譜面が到達するまで0.8秒（演出分含む）無い場合、確保する
    if firstSecond - time < 0.8 then
        time = math.max(time - 0.8, 0)
    end
    return math.max(time, 0.3)
end

-- 1歩目までの最低待機秒
local function GetMinStepSeconds(self)
    local song = W:Song()
    -- ファイルを取得
    local path = song:GetSongFilePath()
    local minSec, minBeat = 0, 999.0
    local suffix = string.match(path, '.+%.([^.]+)')
    if suffix == 'sm' or suffix == 'ssc' then
        local f = RageFileUtil.CreateRageFile()
        f:Open(path, 1)
        f:Seek(0)
        local firstBpm, offset = 60, 0
        local breakFlag = false
        local line = ''
        while(true) do
            if f:AtEOF() or breakFlag then
                break
            end
            line = f:GetLine()
            if line ~= '' then
                local k,v = string.match(line, '^#([^:]+):([^;]*);?$')
                if k then
                    if k == 'NOTES' or k == 'NOTEDATA' then
                        breakFlag = true
                    elseif (k == 'BGCHANGES') and v ~= '' then
                        local firstOffset = tonumber(split('=', v)[1]) or 0
                        if firstOffset < minBeat then
                            minBeat = firstOffset
                        end
                    elseif k == 'BPMS' and v ~= '' then
                        firstBpm = tonumber(string.match(v, '^[0-9.]+=([0-9.]*).+')) or 0
                    elseif k == 'OFFSET' and v ~= '' then
                        offset = tonumber(v) or 0
                    end
                end
            end
        end
        -- 譜面の開始位置
        local firstStepSec = math.max(song:GetFirstSecond(), 0)
        -- ムービー開始の秒
        local movieSec = math.max(minBeat * 60 / firstBpm, 0)

        -- OFAVの場合
        -- -offset = 0小節目の時
        -- firstStepSec = 曲開始から最初のステップまでの秒
        -- movieSec = 0小節目からカウントした値 = 動画開始
        -- 0 = 曲開始

        -- 譜面開始位置までの時間を設定
        -- ムービーの方が曲より早い場合はムービー開始位置までの差分を取得（ただし最大3.5秒）
        if movieSec < offset then
            minSec = math.min(offset - movieSec, 3.5)
        else
            minSec = offset
        end
        minSec = minSec + firstStepSec
        -- それでも曲の開始から最初のステップまでが1.5秒無い場合は加算
        -- （テーマ側のアニメーション分）
        if minSec < 1.5 then
            minSec = 1.5
        end
        f:Close()
        f:destroy()
    end
    local addOffset = (W_OPTION:ReadPref('StartMusicTiming') == 'Slowly') and MIN_TIME or 0
    local ret = minSec + addOffset
    return ret
end

-- 最低待機秒
local function GetMinMusicSeconds(self)
    local wait = 0
    if W_OPTION:ReadPref('StartMusicTiming') == 'Slowly' then
        wait = MIN_TIME
    end
    wait = wait + (W_PLAYER:HasEdit() and 1.0 or 0)
    return wait
end

return {
    GameStartSleep  = GameStartSleep,
    GameJacketSleep = GameJacketSleep,
    MinStepSeconds  = GetMinStepSeconds,
    MinMusicSeconds = GetMinMusicSeconds,
}

--[[
Timer.lua

Copyright (c) 2021 A.C

This software is released under the MIT License.
https://opensource.org/licenses/mit-license.php
--]]
